//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.CompositeUI.EventBroker;
using Microsoft.Practices.CompositeUI;

using GuidanceExplorer.Model;
using GuidanceExplorer.Controls;
using GuidanceExplorer.Services;

namespace GuidanceExplorer.Controls
{
    public partial class AuthoringToolkitControl : Form
    {
        private Dictionary<string, GuidanceType> _deletedTypes = new Dictionary<string, GuidanceType>();
        private Dictionary<string, GuidanceType> _addedTypes = new Dictionary<string, GuidanceType>();
        private bool _exitHandled = false;

        public AuthoringToolkitControl()
        {
            InitializeComponent();
            treeView.TreeView.AfterSelect += new TreeViewEventHandler(TreeView_AfterSelect);
        }

        void TreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (treeView.TreeView.SelectedNode != null)
            {
                btnEdit.Enabled = true;
                btnRemove.Enabled = true;
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            foreach (GuidanceType type in GuidanceType.AllGuidanceTypes)
            {
                if (!type.IsDeleted)
                {
                    GuidanceTypeNode typeNode = GuidanceTypeNode.New();
                    typeNode.ModelInstance = type;
                    typeNode.Text = type.ListDisplayName;
                    int i;
                    for (i = 0; i < treeView.TreeView.Nodes.Count; i++)
                    {
                        if (string.Compare(treeView.TreeView.Nodes[i].Text, typeNode.Text, true) > 0)
                        {
                            break;
                        }
                    }

                    treeView.TreeView.Nodes.Insert(i, typeNode);

                    //Set the icon to the guidance type and guidance type node.
                    string fullImageFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, type.ImageFileName));
                    if (File.Exists(fullImageFileName))
                    {
                        using (FileStream fileStream = new FileStream(fullImageFileName, FileMode.Open, FileAccess.Read))
                        {
                            Image nodeImage = Image.FromStream(fileStream);
                            int imageIndex = treeView.TreeView.ImageList.Images.Add(nodeImage, Color.Magenta);
                            typeNode.ImageIndex = typeNode.SelectedImageIndex = imageIndex;
                        }
                    }
                }
            }
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            GuidanceTypeNode modifiedNode = (GuidanceTypeNode)treeView.GetCurrentNode();
            GuidanceType modifiedType = modifiedNode.ModelInstance;
            GuidanceTypeEditControl editControl = new GuidanceTypeEditControl(modifiedType, treeView);
            editControl.Text = Messages.EDIT_GUIDANCE_TYPE;
            if (editControl.ShowDialog() == DialogResult.OK)
            {
                if (!_addedTypes.ContainsKey(modifiedType.Name) && !_deletedTypes.ContainsKey(modifiedType.Name))
                {
                    _deletedTypes.Add(modifiedType.Name, modifiedType.Clone());
                }

                treeView.TreeView.Nodes.Remove(modifiedNode);
                AddGuidanceType(editControl);
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            GuidanceTypeEditControl editControl = new GuidanceTypeEditControl(treeView);
            editControl.Text = Messages.NEW_GUIDANCE_TYPE;
            if (editControl.ShowDialog() == DialogResult.OK)
            {
                AddGuidanceType(editControl);
            }
        }

        private void AddGuidanceType(GuidanceTypeEditControl editControl)
        {
            GuidanceType type = GuidanceType.New(editControl.GuidanceTypeName);
            type.DisplayName = editControl.TypeDisplayName;
            type.ListDisplayName = editControl.TypeListDisplayName;
            type.NewGuidanceTypeDisplayName = "Template\\" + editControl.TypeDisplayName;

            type.Fields.Clear();

            int position = 0;
            foreach (Field f in editControl.Fields)
            {
                f.Position = position;
                type.Fields[f.Name] = f;
                position++;
            }

            type.TemplateFileName = editControl.TemplateFile;
            type.ExampleFileName = editControl.ExampleFile;

            if (File.Exists(editControl.ImageFile) || File.Exists(Path.Combine(ModelStorageService.TemplateFolder, editControl.ImageFile)))
            {
                type.ImageFileName = editControl.ImageFile;
            }
            else
            {
                type.ImageFileName = GuidanceType.Get(GuidanceType.DefaultTypeName) != null ? GuidanceType.Get(GuidanceType.DefaultTypeName).ImageFileName : string.Empty;
            }

            _addedTypes[type.Name] = type;

            GuidanceTypeNode typeNode = GuidanceTypeNode.New();
            typeNode.ModelInstance = type;
            typeNode.Text = type.ListDisplayName;

            int i;
            for (i = 0; i < treeView.TreeView.Nodes.Count; i++)
            {
                if (string.Compare(treeView.TreeView.Nodes[i].Text, typeNode.Text, true) > 0)
                {
                    break;
                }
            }

            treeView.TreeView.Nodes.Insert(i, typeNode);

            //Set the icon to the guidance type and guidance type node.
            string imagePath = Path.Combine(ModelStorageService.TemplateFolder, type.ImageFileName);
            if (File.Exists(imagePath))
            {
                using (FileStream fileStream = new FileStream(imagePath, FileMode.Open, FileAccess.Read))
                {
                    Image nodeImage = Image.FromStream(fileStream);
                    int imageIndex = treeView.TreeView.ImageList.Images.Add(nodeImage, Color.Magenta);
                    typeNode.ImageIndex = typeNode.SelectedImageIndex = imageIndex;
                }
            }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(Messages.CONFIRM_GUIDANCE_TYPE_REMOVAL, Messages.WARNING, MessageBoxButtons.YesNo,
                MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                GuidanceTypeNode typeNode = (GuidanceTypeNode)treeView.GetCurrentNode();

                if (!_deletedTypes.ContainsKey(typeNode.ModelInstance.Name))
                {
                    _deletedTypes.Add(typeNode.ModelInstance.Name, typeNode.ModelInstance);
                }
                else
                {
                    _deletedTypes[typeNode.ModelInstance.Name] = typeNode.ModelInstance;
                }

                if (_addedTypes.ContainsKey(typeNode.ModelInstance.Name))
                {
                    _addedTypes.Remove(typeNode.ModelInstance.Name);
                }

                treeView.TreeView.Nodes.Remove(typeNode);

                if (treeView.TreeView.Nodes.Count == 0)
                {
                    btnEdit.Enabled = false;
                    btnRemove.Enabled = false;
                }
                else
                {
                    btnEdit.Enabled = true;
                    btnRemove.Enabled = true;
                }
            }
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            string source;
            string destination;

            foreach (GuidanceType type in _deletedTypes.Values)
            {
                if (!_addedTypes.ContainsKey(type.Name))
                {
                    //GuidanceType.RemoveGuidanceType(type.Name);
                    //TODO: If there're no items that belong to this guidance type, just remove it
                    type.IsDeleted = true;
                }
            }

            //Copy files to GE folder
            foreach (GuidanceType type in _addedTypes.Values)
            {
                type.IsDeleted = false;
                if (File.Exists(type.ImageFileName))
                {
                    source = type.ImageFileName;
                    destination = Path.Combine(ModelStorageService.TemplateFolder, Path.GetFileName(type.ImageFileName));
                    if (string.Compare(Path.GetFullPath(source), Path.GetFullPath(destination), true) != 0)
                    {
                        File.Copy(type.ImageFileName, Path.Combine(ModelStorageService.TemplateFolder, Path.GetFileName(type.ImageFileName)), true);
                        type.ImageFileName = Path.GetFileName(type.ImageFileName);
                    }
                }

                if (File.Exists(type.TemplateFileName))
                {
                    source = type.TemplateFileName;
                    destination = Path.Combine(ModelStorageService.TemplateFolder, Path.GetFileName(type.TemplateFileName));
                    if (string.Compare(Path.GetFullPath(source), Path.GetFullPath(destination), true) != 0)
                    {
                        File.Copy(source, destination, true);
                        type.TemplateFileName = Path.GetFileName(type.TemplateFileName);
                    }
                }

                if (File.Exists(type.ExampleFileName))
                {
                    source = type.ExampleFileName;
                    destination = Path.Combine(ModelStorageService.TemplateFolder, Path.GetFileName(type.ExampleFileName));
                    if (string.Compare(Path.GetFullPath(source), Path.GetFullPath(destination), true) != 0)
                    {
                        File.Copy(type.ExampleFileName, Path.Combine(ModelStorageService.TemplateFolder, Path.GetFileName(type.ExampleFileName)), true);
                        type.ExampleFileName = Path.GetFileName(type.ExampleFileName);
                    }
                }
            }
            _exitHandled = true;
            MessageBox.Show(Messages.APP_RESTART_AUTH_TOOLKIT, Messages.GUIDANCE_EXPLORER, MessageBoxButtons.OK, MessageBoxIcon.Information);
            Application.Exit();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            FormClosingEventArgs closingArg = e as FormClosingEventArgs;
            if (closingArg != null)
            {
                closingArg.Cancel = true;
            }

            if (_deletedTypes.Count > 0 || _addedTypes.Count > 0)
            {
                DialogResult res = MessageBox.Show(Messages.SAVE_CHANGES, Messages.WARNING, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

                if (res == DialogResult.Yes)
                {
                    btnOk_Click(null, null);
                }
                else if (res == DialogResult.Cancel)
                {
                    return;
                }
                else
                {
                    foreach (GuidanceType type in _addedTypes.Values)
                    {
                        GuidanceType.RemoveGuidanceType(type.Name);
                    }

                    foreach (GuidanceType type in _deletedTypes.Values)
                    {
                        GuidanceType.AddGuidanceType(type);
                    }
                    _exitHandled = true;

                    if (closingArg != null)
                    {
                        closingArg.Cancel = false;
                    }
                    else
                    {
                        Close();
                    }
                }
            }
            else
            {
                if (closingArg != null)
                {
                    closingArg.Cancel = false;
                }
                else
                {
                    Close();
                }
            }
        }

        private void AuthoringToolkitControl_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!_exitHandled)
            {
                btnCancel_Click(null, e);
            }
        }
    }
}
