//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using GuidanceExplorer.Model;
using GuidanceExplorer.Services;
using GuidanceExplorer.Views;

namespace GuidanceExplorer.Controls
{
    public partial class GuidanceTypeEditControl : Form
    {
        private bool _hasChanged = false;
        private GuidanceType _type = null;
        private bool _exitHandled = false;
        private GuidanceExplorerTreeView _treeView;

        public GuidanceTypeEditControl(GuidanceExplorerTreeView treeView)
        {
            InitializeComponent();

            lstAvailable.DisplayMember = "Caption";
            lstAssigned.DisplayMember = "Caption";
            _treeView = treeView;

            foreach (Field f in GuidanceType.GetAllFields().Values)
            {
                lstAvailable.Items.Add(f);
            }

            btnRemoveAssigned.Enabled = false;
            lstAvailable.SelectedIndex = 0;
        }

        public GuidanceTypeEditControl(GuidanceType type, GuidanceExplorerTreeView treeView)
        {
            InitializeComponent();

            lstAvailable.DisplayMember = "Caption";
            lstAssigned.DisplayMember = "Caption";

            _treeView = treeView;
            _type = type;

            txtExample.Text = type.ExampleFileName;
            txtGuidanceType.Text = type.Name;
            txtIcon.Text = type.ImageFileName;
            txtTemplate.Text = type.TemplateFileName;
            txtDisplayName.Text = type.DisplayName;
            txtListDisplayName.Text = type.ListDisplayName;

            foreach (Field f in type.Fields.Values)
            {
                lstAssigned.Items.Add(f);
            }

            foreach (Field f in GuidanceType.GetAllFields().Values)
            {
                if (!type.Fields.ContainsKey(f.Name))
                {
                    lstAvailable.Items.Add(f);
                }
            }

            if (lstAvailable.Items.Count > 0)
            {
                lstAvailable.SelectedIndex = 0;
            }

            if (lstAssigned.Items.Count > 0)
            {
                lstAssigned.SelectedIndex = 0;
            }
        }

        private void btnBrowseTemplate_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = Messages.XML_FILES_FILTER;
            dialog.RestoreDirectory = true;
            dialog.CheckFileExists = true;
            dialog.CheckPathExists = true;
            dialog.Multiselect = false;
            dialog.InitialDirectory = ModelStorageService.TemplateFolder;

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                txtTemplate.Text = dialog.FileName;
            }
        }

        private void btnBrowseExample_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = Messages.XML_FILES_FILTER;
            dialog.Multiselect = false;
            dialog.RestoreDirectory = true;
            dialog.CheckFileExists = true;
            dialog.CheckPathExists = true;
            dialog.InitialDirectory = ModelStorageService.TemplateFolder;

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                txtExample.Text = dialog.FileName;
            }
        }

        private void brnBrowseIcon_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = Messages.IMAGE_FILES_FILTER;
            dialog.RestoreDirectory = true;
            dialog.CheckFileExists = true;
            dialog.CheckPathExists = true;
            dialog.Multiselect = false;
            dialog.InitialDirectory = ModelStorageService.TemplateFolder;

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                txtIcon.Text = dialog.FileName;
            }
        }

        private void btnCreateAvailable_Click(object sender, EventArgs e)
        {
            InputBox.ValidateInput validator = delegate(string userInput)
            {
                if (userInput == null || userInput.Length == 0)
                {
                    MessageBox.Show(Messages.CANNOT_ENTER_EMPTY_FIELDS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                else
                {
                    foreach (Field field in lstAssigned.Items)
                    {
                        if (string.Compare(field.Name, userInput, true) == 0)
                        {
                            MessageBox.Show(Messages.FIELD_EXISTS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return false;
                        }
                    }

                    foreach (Field field in lstAvailable.Items)
                    {
                        if (string.Compare(field.Name, userInput, true) == 0)
                        {
                            MessageBox.Show(Messages.FIELD_EXISTS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return false;
                        }
                    }

                    return true;
                }
            };

            InputBox input = new InputBox("Enter Field name", "Field name", string.Empty, validator);

            if (input.ShowDialog() == DialogResult.OK)
            {
                Field newField = Field.New();
                newField.Name = input.UserText;
                newField.Caption = input.UserText;
                newField.Name = input.UserText;
                newField.Distinct = true;
                newField.Type = FieldType.Combo;

                lstAvailable.Items.Add(newField);
            }
        }

        private void btnRenameAvailable_Click(object sender, EventArgs e)
        {
            Field f = (Field)lstAvailable.SelectedItem;
            InputBox input = new InputBox("Enter Field name", "Field name", f.Name);

            if (input.ShowDialog() == DialogResult.OK)
            {
                f.Name = input.UserText;
                f.Caption = input.UserText;
                f.Name = input.UserText;

                lstAvailable.Refresh();
            }
        }

        private void btnDeleteAvailable_Click(object sender, EventArgs e)
        {
            if (lstAvailable.SelectedItems != null && lstAvailable.SelectedItems.Count > 0)
            {
                if (MessageBox.Show(Messages.CONFIRM_FIELD_DELETION, Messages.WARNING, MessageBoxButtons.YesNo,
                    MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    List<Field> selectedFields = new List<Field>(lstAvailable.SelectedItems.Count);
                    foreach (Field f in lstAvailable.SelectedItems)
                    {
                        selectedFields.Add(f);
                    }

                    foreach (Field f in selectedFields)
                    {
                        lstAvailable.Items.Remove(f);
                    }
                }
            }
        }

        private void btnAddAssigned_Click(object sender, EventArgs e)
        {
            if (lstAvailable.SelectedItems != null && lstAvailable.SelectedItems.Count > 0)
            {
                List<Field> selectedFields = new List<Field>(lstAvailable.SelectedItems.Count);
                foreach (Field f in lstAvailable.SelectedItems)
                {
                    selectedFields.Add(f);
                }

                foreach (Field f in selectedFields)
                {
                    lstAvailable.Items.Remove(f);
                    lstAssigned.Items.Add(f);
                }

                if (lstAvailable.Items.Count > 0)
                {
                    lstAvailable.SelectedIndex = 0;
                }

                _hasChanged = true;
            }
        }

        private void btnRemoveAssigned_Click(object sender, EventArgs e)
        {
            if (lstAssigned.SelectedItems != null && lstAssigned.SelectedItems.Count > 0)
            {
                List<Field> selectedFields = new List<Field>(lstAssigned.SelectedItems.Count);
                foreach (Field f in lstAssigned.SelectedItems)
                {
                    selectedFields.Add(f);
                }

                foreach (Field f in selectedFields)
                {
                    lstAssigned.Items.Remove(f);
                    lstAvailable.Items.Add(f);
                }

                if (lstAssigned.Items.Count > 0)
                {
                    lstAssigned.SelectedIndex = 0;
                }

                _hasChanged = true;
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            DialogResult res = DialogResult.No;
            FormClosingEventArgs closingArg = e as FormClosingEventArgs;

            if (_type == null || _hasChanged || (_type != null && (string.Compare(_type.Name, txtGuidanceType.Text) != 0 ||
                string.Compare(_type.ExampleFileName, txtExample.Text) != 0 || string.Compare(_type.TemplateFileName, txtTemplate.Text) != 0 ||
                string.Compare(_type.ImageFileName, txtIcon.Text) != 0)))
            {
                res = MessageBox.Show(Messages.SAVE_CHANGES, Messages.WARNING, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);
            }

            if (closingArg != null)
            {
                closingArg.Cancel = true;
            }

            if (res == DialogResult.No)
            {
                DialogResult = DialogResult.Cancel;
                _exitHandled = true;

                if (closingArg != null)
                {
                    closingArg.Cancel = false;
                }
                else
                {
                    Close();
                }
            }
            else if (res == DialogResult.Yes)
            {
                btnOk_Click(null, null);
            }
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txtGuidanceType.Text) || string.IsNullOrEmpty(txtDisplayName.Text) ||
                string.IsNullOrEmpty(txtListDisplayName.Text))
            {
                MessageBox.Show(Messages.GUIDANCE_TYPE_NAME_NOT_SETTED, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                bool ignoreName = false;
                bool ignoreDisplayName = false;
                bool ignoreListName = false;

                if (_type != null)
                {
                    ignoreName = string.Compare(_type.Name, txtGuidanceType.Text, true) == 0;
                    ignoreDisplayName = string.Compare(_type.DisplayName, txtDisplayName.Text, true) == 0;
                    ignoreListName = string.Compare(_type.ListDisplayName, txtListDisplayName.Text, true) == 0;
                }

                foreach (TreeNode node in _treeView.TreeView.Nodes)
                {
                    GuidanceTypeNode typeNode = node as GuidanceTypeNode;
                    if (typeNode != null)
                    {
                        if (!ignoreName && string.Compare(typeNode.ModelInstance.Name, txtGuidanceType.Text, true) == 0)
                        {
                            MessageBox.Show(Messages.GUIDANCE_TYPE_NAME_EXISTS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                        else if (!ignoreDisplayName && string.Compare(typeNode.ModelInstance.DisplayName, txtDisplayName.Text, true) == 0)
                        {
                            MessageBox.Show(Messages.GUIDANCE_TYPE_DISPLAY_NAME_EXISTS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                        else if (!ignoreListName && string.Compare(typeNode.ModelInstance.ListDisplayName, txtListDisplayName.Text, true) == 0)
                        {
                            MessageBox.Show(Messages.GUIDANCE_TYPE_LIST_NAME_EXISTS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                    }
                }

                DialogResult = DialogResult.OK;
                _exitHandled = true;
                Close();
            }
        }

        public List<Field> Fields
        {
            get
            {
                List<Field> fields = new List<Field>(lstAssigned.Items.Count);

                foreach (Field f in lstAssigned.Items)
                {
                    fields.Add(f);
                }

                return fields;
            }
        }

        public string GuidanceTypeName
        {
            get { return txtGuidanceType.Text; }
        }

        public string ImageFile
        {
            get { return txtIcon.Text; }
        }

        public string ExampleFile
        {
            get { return txtExample.Text; }
        }

        public string TemplateFile
        {
            get { return txtTemplate.Text; }
        }

        private void lstAvailable_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstAvailable.SelectedIndex >= 0)
            {
                btnRenameAvailable.Enabled = true;
                btnDeleteAvailable.Enabled = true;
                btnAddAssigned.Enabled = true;
            }
            else
            {
                btnAddAssigned.Enabled = false;
                btnRenameAvailable.Enabled = false;
                btnDeleteAvailable.Enabled = false;
            }
        }

        private void lstAssigned_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstAssigned.SelectedIndex >= 0)
            {
                btnRemoveAssigned.Enabled = true;
            }
            else
            {
                btnRemoveAssigned.Enabled = false;
            }
        }

        private void GuidanceTypeEditControl_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!_exitHandled)
            {
                btnCancel_Click(null, e);
            }
        }

        public string TypeDisplayName
        {
            get { return txtDisplayName.Text; }
        }

        public string TypeListDisplayName
        {
            get { return txtListDisplayName.Text; }
        }
    }
}