//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace GuidanceExplorer.Model
{
    public class Folder : ILibraryNodes
    {
        /// <summary>
        /// Factory.
        /// </summary>
        /// <returns></returns>
        public static Folder New()
        {
            return new Folder();
        }

        #region Private members

        /// <summary>
        /// The columns for this folder.
        /// </summary>
        private ColumnCollection _columns = ColumnCollection.New();

        /// <summary>
        /// The list of views defined in this structure.
        /// </summary>
        private ViewCollection _views = ViewCollection.New();

        /// <summary>
        /// The list of folders defined in this structure.
        /// </summary>
        private FolderCollection _folders = FolderCollection.New();

        /// <summary>
        /// The parent folder reference.
        /// </summary>
        private Folder _parentFolder;

        /// <summary>
        /// The caption of the view.
        /// </summary>
        private string _caption;

        /// <summary>
        /// The outer container of the view.
        /// </summary>
        private FolderStructure _container;

        #endregion

        /// <summary>
        /// Private default constructor.
        /// </summary>
        private Folder() { }

        #region Public properties

        /// <summary>
        /// The caption of the view.
        /// </summary>
        public string Caption
        {
            get { return _caption; }
            set { _caption = value; }
        }

        /// <summary>
        /// The list of views defined in this structure.
        /// </summary>
        public ViewCollection Views
        {
            get { return _views; }
            set { _views = value; }
        }

        /// <summary>
        /// The list of all child views.
        /// </summary>
        public ViewCollection AllViews
        {
            get
            {
                ViewCollection views = ViewCollection.New();
                foreach (View view in _views.Values)
                {
                    views.Add(view.Caption, view);
                }

                foreach (Folder folder in _folders.Values)
                {
                    foreach (View view in folder.AllViews.Values)
                    {
                        views.Add(view.Caption, view);
                    }
                }
                return views;
            }
        }

        /// <summary>
        /// The list of folders defined in this structure.
        /// </summary>
        public FolderCollection Folders
        {
            get { return _folders; }
            set { _folders = value; }
        }

        /// <summary>
        /// The parent folder reference.
        /// </summary>
        public Folder ParentFolder
        {
            get { return _parentFolder; }
            set { _parentFolder = value; }
        }

        /// <summary>
        /// The outer container of the view.
        /// </summary>
        public FolderStructure Container
        {
            get { return _container; }
            set { _container = value; }
        }

        /// <summary>
        /// The columns for this folder.
        /// </summary>
        public ColumnCollection Columns
        {
            get { return _columns; }
            set { _columns = value; }
        }

        /// <summary>
        /// Folders does not have Items, they only can be iterated for contained items.
        /// </summary>
        public bool HasItems
        {
            get { return false; }
        }

        /// <summary>
        /// The items collection can't be set.
        /// </summary>
        public bool CanSetItems
        {
            get { return false; }
        }

        /// <summary>
        /// The list of items in the folder.
        /// </summary>
        public GuidanceItemCollection Items
        {
            get { throw new Exception("Folders does not have Items."); }
            set { throw new Exception("Folders does not have Items."); }
        }

        #endregion

        #region IEnumerable<GuidanceItem> Members

        public IEnumerator<GuidanceItem> GetEnumerator()
        {
            foreach (Folder folder in _folders.Values)
            {
                foreach (GuidanceItem item in folder)
                {
                    yield return item;
                }
            }

            foreach (View view in _views.Values)
            {
                if (!(view is SearchView))
                {
                    foreach (GuidanceItem item in view)
                    {
                        yield return item;
                    }
                }
            }
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion

        public override bool Equals(object obj)
        {
            Folder other = obj as Folder;
            if (other != null)
            {
                bool caption = (_caption == other._caption);
                bool columns = (_columns.Equals(other._columns));
                bool folders = (_folders.Equals(other._folders));
                bool parentFolder = (_parentFolder == other._parentFolder);
                bool views = (_views.Equals(other._views));
                return (caption && columns && folders && parentFolder && views);
            }
            else
            {
                return base.Equals(obj);
            }
        }

        public override int GetHashCode()
        {
            return _caption.GetHashCode();
        }

        /// <summary>
        /// Get the container library or null if this is a MyViews folder.
        /// </summary>
        /// <returns></returns>
        public Library GetContainerLibrary()
        {
            if (ParentFolder != null)
            {
                Folder tempFolder;
                for (tempFolder = ParentFolder; tempFolder.ParentFolder != null; tempFolder = tempFolder.ParentFolder)
                { }
                return tempFolder.Container as Library;
            }
            else
            {
                return Container as Library;
            }
        }

        /// <summary>
        /// Remove the folder from its container.
        /// </summary>
        public void Remove()
        {
            if (ParentFolder == null)
            {
                Container.Folders.Remove(this.Caption);
            }
            else
            {
                ParentFolder.Folders.Remove(this.Caption);
            }
        }

        public Folder Clone()
        {
            Folder newFolder = new Folder();

            newFolder._caption = Caption;
            newFolder._columns = Columns;
            newFolder._container = Container;
            newFolder._parentFolder = ParentFolder;

            foreach (Folder folder in _folders.Values)
            {
                Folder newChildFolder = folder.Clone();
                newFolder._folders.Add(newChildFolder.Caption, newChildFolder);
            }

            foreach (View view in _views.Values)
            {
                if (view is SearchView)
                {
                    newFolder._views.Add(view.Id, view);
                }
                else
                {
                    View newView = view.Clone();
                    newFolder._views.Add(newView.Id, newView);
                }
            }

            return newFolder;
        }
    }
}

