//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace GuidanceExplorer.Model
{
    public enum SearchViewScope { Guidance, Library, Folder, View, UserViews }

    public class SearchView : View
    {
        /// <summary>
        /// Factory.
        /// </summary>
        /// <returns>A new instance of a View.</returns>
        public static new SearchView New()
        {
            return new SearchView();
        }

        /// <summary>
        /// The criteria that will be used to populate this view.
        /// </summary>
        private Criteria _criteria;

        /// <summary>
        /// The container scope for this SearchView.
        /// </summary>
        private IGuidanceItemContainer _containerScope;

        /// <summary>
        /// The scope readed from the xml file.
        /// </summary>
        private SearchViewScope _readedScope;

        /// <summary>
        /// The scope name readed form the xmlfile.
        /// </summary>
        private string _readedScopeName;

        /// <summary>
        /// Default private constructor.
        /// </summary>
        private SearchView() { }

        /// <summary>
        /// SearchViews has Items.
        /// </summary>
        public override bool HasItems
        {
            get { return false; }
        }

        /// <summary>
        /// Items collection can't be set.
        /// </summary>
        public override bool CanSetItems
        {
            get { return false; }
        }

        /// <summary>
        /// The list of items in the view.
        /// </summary>
        public override GuidanceItemCollection Items
        {
            get { throw new Exception("Items collection can't be get."); }
            set { throw new Exception("Items collection can't be set."); }
        }

        /// <summary>
        /// The criteria that will be used to populate this view.
        /// </summary>
        public Criteria Criteria
        {
            get { return _criteria; }
            set { _criteria = value; }
        }

        /// <summary>
        /// The container scope for this SearchView.
        /// </summary>
        public IGuidanceItemContainer ContainerScope
        {
            get { return _containerScope; }
            set { _containerScope = value; }
        }

        /// <summary>
        /// The scope for this search view.
        /// </summary>
        public SearchViewScope Scope
        {
            get
            {
                if (_containerScope is Guidance)
                {
                    return SearchViewScope.Guidance;
                }
                else if (_containerScope is Library)
                {
                    return SearchViewScope.Library;
                }
                else if (_containerScope is Folder)
                {
                    return SearchViewScope.Folder;
                }
                else if (_containerScope is View)
                {
                    return SearchViewScope.View;
                }
                else if (_containerScope is UserViews)
                {
                    return SearchViewScope.UserViews;
                }
                throw new Exception("Invalid container type.");
            }
        }

        /// <summary>
        /// The scope readed from the xml file.
        /// </summary>
        public SearchViewScope ReadedScope
        {
            get { return _readedScope; }
            set { _readedScope = value; }
        }

        /// <summary>
        /// The scope name readed form the xmlfile.
        /// </summary>
        public string ReadedScopeName
        {
            get { return _readedScopeName; }
            set { _readedScopeName = value; }
        }

        #region IEnumerable Members

        public override IEnumerator<GuidanceItem> GetEnumerator()
        {
            _criteria.Reset();

            //Sort items on the date created.
            List<GuidanceItem> dateSorted = new List<GuidanceItem>(_containerScope);
            dateSorted.Sort(
                delegate(GuidanceItem x, GuidanceItem y)
                {
                    return DateTime.Compare(x.DateTime, y.DateTime) * -1;
                }
            );

            //Iterate through all the scoped items.
            foreach (GuidanceItem item in dateSorted)
            {
                if (_criteria.Evaluate(item)) //Evaluate the criteria in the item.
                {
                    //If succeed then add the item to the collection.
                    yield return item;
                }
            }
        }

        #endregion

    }
}

