//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace GuidanceExplorer.Model
{
    /// <summary>
    /// Represents a view which is a collection of items from any library in the model.
    /// </summary>
    public class View : ILibraryNodes
    {
        /// <summary>
        /// Factory.
        /// </summary>
        /// <returns>A new instance of a View.</returns>
        public static View New()
        {
            return new View();
        }

        #region Private members

        /// <summary>
        /// The list of items in the view.
        /// </summary>
        private GuidanceItemCollection _items = GuidanceItemCollection.New();

        /// <summary>
        /// The parent folder reference.
        /// </summary>
        private Folder _parentFolder;

        /// <summary>
        /// The outer container of the view.
        /// </summary>
        private FolderStructure _container;

        /// <summary>
        /// The columns for this view.
        /// </summary>
        private ColumnCollection _columns = ColumnCollection.New();

        /// <summary>
        /// The caption of the view.
        /// </summary>
        private string _caption;

        /// <summary>
        /// The unique id of the view.
        /// </summary>
        private string _id = Guid.NewGuid().ToString();

        /// <summary>
        /// The author of the view.
        /// </summary>
        private string _author;

        /// <summary>
        /// The description of this view.
        /// </summary>
        private string _description;

        /// <summary>
        /// The creation date for this view.
        /// </summary>
        private DateTime _creationDate;

        /// <summary>
        /// Whether the view will be display as a root element or within the container.
        /// </summary>
        private bool _rootView;

        #endregion

        /// <summary>
        /// Default private constructor.
        /// </summary>
        protected View() { }

        #region Public Properties

        /// <summary>
        /// The caption of the view.
        /// </summary>
        public string Caption
        {
            get { return _caption; }
            set { _caption = value; }
        }

        /// <summary>
        /// The unique id of the view.
        /// </summary>
        public string Id
        {
            get { return _id; }
            set
            {
                if (string.IsNullOrEmpty(value))
                {
                    _id = Guid.NewGuid().ToString();
                }
                else
                {
                    _id = value;
                }
            }
        }

        /// <summary>
        /// Views has Items.
        /// </summary>
        public virtual bool HasItems
        {
            get { return true; }
        }

        /// <summary>
        /// The items collection can be set.
        /// </summary>
        public virtual bool CanSetItems
        {
            get { return true; }
        }

        /// <summary>
        /// The list of items in the view.
        /// </summary>
        public virtual GuidanceItemCollection Items
        {
            get { return _items; }
            set { _items = value; }
        }

        /// <summary>
        /// The parent folder reference.
        /// </summary>
        public Folder ParentFolder
        {
            get { return _parentFolder; }
            set { _parentFolder = value; }
        }

        /// <summary>
        /// The outer container of the view.
        /// </summary>
        public FolderStructure Container
        {
            get { return _container; }
            set { _container = value; }
        }

        /// <summary>
        /// The columns for this view.
        /// </summary>
        public ColumnCollection Columns
        {
            get { return _columns; }
            set { _columns = value; }
        }

        /// <summary>
        /// The author of the view.
        /// </summary>
        public string Author
        {
            get { return _author; }
            set { _author = value; }
        }

        /// <summary>
        /// The description of this view.
        /// </summary>
        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        /// <summary>
        /// The creation date for this view.
        /// </summary>
        public DateTime CreationDate
        {
            get { return _creationDate; }
            set { _creationDate = value; }
        }

        /// <summary>
        /// Whether the view will be display as a root element or within the container.
        /// </summary>
        public bool RootView
        {
            get { return _rootView; }
            set { _rootView = value; }
        }

        #endregion

        #region IEnumerable Members

        public virtual IEnumerator<GuidanceItem> GetEnumerator()
        {
            foreach (GuidanceItem item in _items.Values)
            {
                yield return item;
            }
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        #endregion

        public override bool Equals(object obj)
        {
            View other = obj as View;
            if (other != null)
            {
                bool caption = (_caption == other._caption);
                bool columns = (_columns.Equals(other._columns));
                bool parentFolder = (_parentFolder == other._parentFolder);
                bool items = (_items.Equals(other._items));
                bool root = (_rootView.Equals(other._rootView));
                return (caption && columns && parentFolder && items && root);
            }
            else
            {
                return base.Equals(obj);
            }
        }

        public override int GetHashCode()
        {
            return _caption.GetHashCode();
        }

        public Library GetContainerLibrary()
        {
            if (ParentFolder != null)
            {
                Folder tempFolder;
                for (tempFolder = ParentFolder; tempFolder.ParentFolder != null; tempFolder = tempFolder.ParentFolder)
                { }
                return tempFolder.Container as Library;
            }
            else
            {
                return Container as Library;
            }
        }

        /// <summary>
        /// Remove the View from the container.
        /// </summary>
        public void Remove()
        {
            if (ParentFolder == null)
            {
                Container.Views.Remove(this._caption);
            }
            else
            {
                ParentFolder.Views.Remove(this._caption);
            }

        }

        public View Clone()
        {
            View newView = new View();
            newView._author = Author;
            newView._caption = Caption;
            newView._columns = Columns;
            newView._container = Container;
            newView._creationDate = CreationDate;
            newView._description = Description;
            newView._id = Guid.NewGuid().ToString();
            newView._items = Items.Clone();
            newView._parentFolder = ParentFolder;
            newView._rootView = RootView;

            return newView;
        }
    }
}

