//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using GuidanceExplorer.Model;
using Microsoft.Practices.CompositeUI;
using System.IO;
using System.Xml;
using GuidanceExplorer.Schemas;
using System.Reflection;

namespace GuidanceExplorer.Services
{
    [Service(typeof(IHtmlRenderService))]
    public class HtmlRenderService : IHtmlRenderService
    {
        #region IBuildHtmlForGuidanceItem Members
        /// <summary>
        /// Returns the document HTML content as string.
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to convert</param>
        /// <returns></returns>
        public Stream GetReadOnlyDocumentStream(GuidanceItemFileReference guidanceItem)
        {
            return GetDocumentStream(guidanceItem, ReadProtection.ReadOnly, false);
        }

        /// <summary>
        /// Returns the document HTML content as string for showing or exporting
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to convert</param>
        /// <returns></returns>
        public Stream GetDocumentStreamForShowing(GuidanceItemFileReference guidanceItem)
        {
            return GetDocumentStream(guidanceItem, ReadProtection.ReadOnly, true);
        }

        /// <summary>
        /// Returns the raw HTML content for the document 
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to process</param>
        /// <returns></returns>
        public Stream GetRawDocumentStream(GuidanceItemFileReference guidanceItem)
        {
            MemoryStream ms = new MemoryStream();
            StreamWriter sw = new StreamWriter(ms);
            sw.Write(GetDocumentText(guidanceItem.FileName));
            sw.Flush();
            ms.Position = 0;
            return ms;
        }

        /// <summary>
        /// Returns the document HTML content as string.
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to convert</param>
        /// <returns></returns>
        public Stream GetEditableDocumentStream(GuidanceItemFileReference guidanceItem)
        {
            return GetDocumentStream(guidanceItem, ReadProtection.ReadWrite, false);
        }

        /// <summary>
        /// Return the template for the guidance item.
        /// </summary>
        /// <param name="guidanceItemFileReference"></param>
        /// <returns></returns>
        public Stream GetReadOnlyTemplateStream(GuidanceItemFileReference guidanceItemFileReference)
        {
            string templateFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItemFileReference.GuidanceType.TemplateFileName));
            if (File.Exists(templateFileName))
            {
                return GetDocumentStream(templateFileName, ReadProtection.ReadOnly, guidanceItemFileReference, false);
            }
            return null;
        }

        /// <summary>
        /// Return the example for the guidance item.
        /// </summary>
        /// <param name="guidanceItemFileReference"></param>
        /// <returns></returns>
        public Stream GetReadOnlyExampleStream(GuidanceItemFileReference guidanceItemFileReference)
        {
            string exampleFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItemFileReference.GuidanceType.ExampleFileName));
            if (File.Exists(exampleFileName))
            {
                return GetDocumentStream(exampleFileName, ReadProtection.ReadOnly, guidanceItemFileReference, false);
            }
            return null;
        }

        #endregion

        /// <summary>
        /// Adapts between Model-oriented method and file-oriented method.
        /// </summary>
        /// <param name="guidanceItem"></param>
        /// <param name="readOnly"></param>
        /// <param name="readProtection"></param>
        /// <param name="showData"></param>
        /// <returns></returns>
        private Stream GetDocumentStream(GuidanceItemFileReference guidanceItem, ReadProtection readProtection, bool showData)
        {
            string templateFileName = string.Empty;
            if (string.IsNullOrEmpty(guidanceItem.FileName))
            {
                return null;
            }
            string guidanceItemFileName = Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, guidanceItem.FileName));
            if ((guidanceItem.GuidanceType != null) &&
                (!string.IsNullOrEmpty(guidanceItem.GuidanceType.TemplateFileName)))
            {
                templateFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItem.GuidanceType.TemplateFileName));
            }
            if (File.Exists(guidanceItemFileName))
            {
                return GetDocumentStream(guidanceItemFileName, readProtection, guidanceItem, showData);
            }
            else if (!string.IsNullOrEmpty(templateFileName) && File.Exists(templateFileName))
            {
                return GetDocumentStream(templateFileName, readProtection, guidanceItem, showData);
            }

            if (readProtection == ReadProtection.ReadWrite)
            {
                return GetDocumentStream(templateFileName, readProtection, guidanceItem, showData);
            }
            return null;
        }

        /// <summary>
        /// Exports the provided Guidance Item as Html
        /// </summary>
        /// <param name="guidanceItem"></param>
        /// <param name="file"></param>
        public void RenderGuidanceItemToHtml(GuidanceItemFileReference guidanceItem, FileInfo file)
        {
            string fullCssPath = string.Empty;
            if (File.Exists(guidanceItem.CSSFile))
            {
                fullCssPath = guidanceItem.CSSFile;
            }
            else
            {
                fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItem.CSSFile));
                if (!File.Exists(fullCssPath))
                {
                    fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName));
                }
            }

            using (FileStream stream = new FileStream(file.FullName, FileMode.Create, FileAccess.ReadWrite))
            using (StreamWriter streamWriter = new StreamWriter(stream, Encoding.UTF8))
            {
                streamWriter.WriteLine("<html>");
                streamWriter.WriteLine("<head>");
                streamWriter.WriteLine("<style type=\"text/css\">");
                streamWriter.WriteLine(File.ReadAllText(fullCssPath));
                streamWriter.WriteLine("</style>");
                streamWriter.WriteLine("</head>");
                WriteDocumentBody(guidanceItem.FileName, ReadProtection.ReadOnly, guidanceItem, true, streamWriter);
                streamWriter.WriteLine("</html>");

                streamWriter.Flush();
            }
        }

        /// <summary>
        /// Creates the HTML in a stream for later reading.
        /// </summary>
        /// <param name="guidanceItemFileName"></param>
        /// <param name="guidanceItemCssFileName"></param>
        /// <param name="readProtection"></param>
        /// <param name="guidanceItem"></param>
        /// <param name="showData"></param>
        /// <returns></returns>
        private Stream GetDocumentStream(string guidanceItemFileName, ReadProtection readProtection, GuidanceItemFileReference guidanceItem, bool showData)
        {
            string hrefCssFormat = string.Empty;
            if (File.Exists(guidanceItem.CSSFile))
            {
                string fullCssPath = guidanceItem.CSSFile;
                hrefCssFormat = fullCssPath.Replace('\\', '/');
            }
            else
            {
                string fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItem.CSSFile));
                if (File.Exists(fullCssPath))
                {
                    hrefCssFormat = fullCssPath.Replace('\\', '/');
                }
                else
                {
                    fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName));
                    hrefCssFormat = fullCssPath.Replace('\\', '/');
                }
            }

            return WriteDocumentStream(guidanceItemFileName, readProtection, guidanceItem, showData, hrefCssFormat);
        }

        private Stream WriteDocumentStream(string guidanceItemFileName, ReadProtection readProtection, GuidanceItemFileReference guidanceItem, bool showData, string hrefCssFormat)
        {
            MemoryStream memoryStream = new MemoryStream();
            StreamWriter streamWriter = new StreamWriter(memoryStream, Encoding.UTF32);

            
            streamWriter.WriteLine("<html>");

            if (File.Exists(hrefCssFormat))
            {
                streamWriter.WriteLine("<style type=\"text/css\">");
                streamWriter.WriteLine(File.ReadAllText(hrefCssFormat));
                streamWriter.WriteLine("</style>");
            }
            else
            {
                streamWriter.WriteLine(string.Format("<link href=\"{0}\" rel=\"stylesheet\" type=\"text/css\" />", guidanceItem.CSSFile));
            }
            WriteDocumentBody(guidanceItemFileName, readProtection, guidanceItem, showData, streamWriter);
            streamWriter.WriteLine("</html>");

            streamWriter.Flush();
            memoryStream.Position = 0;

            return memoryStream;
        }

        private void WriteDocumentBody(string guidanceItemFileName, ReadProtection readProtection, GuidanceItemFileReference guidanceItem, bool showData, StreamWriter streamWriter)
        {
            if (readProtection == ReadProtection.ReadOnly)
            {
                streamWriter.WriteLine("<body id=\"bodyId\">");
            }
            else
            {
                streamWriter.WriteLine("<script>");
                using (StreamReader innerCode =
                    new StreamReader(Assembly.GetExecutingAssembly().GetManifestResourceStream("GuidanceExplorer.innerCode.js")))
                {
                    streamWriter.Write(innerCode.ReadToEnd());
                }
                streamWriter.WriteLine("</script>");
                streamWriter.WriteLine("<body id=\"bodyId\" contentEditable=\"true\" onselectstart=\"handleEvent();\" onkeyup=\"handleEvent();\" onmouseup=\"handleEvent();\" ondblclick=\"handleEvent();\" onblur=\"handleEvent();\" onfocus=\"handleEvent();\">");
            }

            if (showData)
            {
                string link = guidanceItem.Title;//Links lenght is up to 40 chars
                if (link.Length > 40)
                {
                    link = link.Substring(0, 39);
                }
                streamWriter.WriteLine("<h1 class=\"title\"><a name='" + link.Replace(" ", "_") + "'>" + guidanceItem.Title + "</a></h1>\n");
            }

            if (File.Exists(guidanceItemFileName))
            {
                //Places the HTML for the document.
                streamWriter.WriteLine(GetDocumentText(guidanceItemFileName));
            }

            //Show attributes
            if (showData)
            {
                streamWriter.WriteLine("\n\n<br><hr><br>\n");
                streamWriter.WriteLine("<h1>Attributes</h1>\n");
                foreach (string att in guidanceItem.CustomData.Keys)
                {
                    streamWriter.WriteLine("<li>" + guidanceItem.GuidanceType.Fields.GetDisplayName(att) + ": " + guidanceItem[att] + "</li><br>");
                }
            }

            streamWriter.WriteLine("</body>");
        }

        /// <summary>
        /// Returns the document HTML content as string.
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to convert.</param>
        /// <returns>A string with the HTML content.</returns>
        private string GetDocumentText(string fileName)
        {
            using (FileStream fileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read))
            using (XmlReader reader = XmlTextReader.Create(fileStream))
            {
                while (reader.Read())
                {
                    if (SchemaHelper.IsGuidanceItemNode(reader, SchemaHelper.GuidanceItem.Content))
                    {
                        return reader.ReadElementContentAsString();
                    }
                }
            }
            return null;
        }

        /// <summary>
        /// Exports the selected items to Word with an index page
        /// </summary>
        /// <param name="itemsContainer"></param>
        /// <param name="file"></param>
        public void RenderGuidanceItemsToWordFileInfoWithIndex(IList<GuidanceItemFileReference> itemsContainer, FileInfo file)
        {
            string extension = ".doc";
            using (StreamWriter outFile = new StreamWriter(file.FullName, false, Encoding.UTF8))
            {
                outFile.WriteLine("<html>");

                string cssFile = Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName);
                if (File.Exists(cssFile))
                {
                    outFile.WriteLine("<head>");
                    outFile.WriteLine("<style type=\"text/css\">");
                    outFile.WriteLine(File.ReadAllText(cssFile));
                    outFile.WriteLine("</style>\n");
                    //outFile.WriteLine("</head>");
                }

                outFile.WriteLine("<body>");
                outFile.WriteLine(BuildIndexofTitles(itemsContainer, extension, null));
                foreach (GuidanceItem item in itemsContainer)
                {
                    GuidanceItemFileReference gifr = item as GuidanceItemFileReference;
                    if (gifr != null)
                    {
                        WriteDocumentBody(gifr.FileName, ReadProtection.ReadOnly, gifr, true, outFile);
                        outFile.WriteLine("<br>");
                        outFile.WriteLine("<br>");
                    }
                }
                outFile.WriteLine("</body>");
                outFile.WriteLine("</html>");
            }
        }

        /// <summary>
        /// Exports the selected items to Html with an index page
        /// </summary>
        /// <param name="itemsContainer"></param>
        /// <param name="file"></param>
        public void RenderGuidanceItemsToHtmlFileInfoWithIndex(IList<GuidanceItemFileReference> itemsContainer, FileInfo file)
        {
            string directory = file.DirectoryName;
            string extension = ".htm";
            using (StreamWriter outFile = new StreamWriter(file.FullName, false, Encoding.UTF8))
            {
                directory += "\\" + Path.GetFileNameWithoutExtension(file.FullName) + "_files";
                outFile.WriteLine("<html>");

                string cssFile = Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName);
                if (File.Exists(cssFile))
                {
                    outFile.WriteLine("<head>");
                    outFile.WriteLine("<style type=\"text/css\">");
                    outFile.WriteLine(File.ReadAllText(cssFile));
                    outFile.WriteLine("</style>\n");
                    outFile.WriteLine("</head>");
                }

                outFile.WriteLine("<body>");
                outFile.Write(BuildIndexofTitles(itemsContainer, extension, directory));
                outFile.WriteLine("</body>");
                outFile.WriteLine("</html>");
            }

            Directory.CreateDirectory(directory);

            foreach (GuidanceItem item in itemsContainer)
            {
                GuidanceItemFileReference gifr = item as GuidanceItemFileReference;
                if (gifr != null)
                {
                    string newPath = Path.Combine(directory, Path.GetFileNameWithoutExtension(gifr.FileName) + extension);
                    string oldCss = gifr.CSSFile;
                    string newCss = Path.Combine(directory, Path.GetFileName(gifr.CSSFile));
                    if (!File.Exists(newCss))
                    {
                        File.Copy(GetValidCssFile(gifr), newCss, true);
                    }
                    gifr.CSSFile = newCss;
                    using (StreamWriter itemFile = new StreamWriter(newPath, false, Encoding.UTF8))
                    {
                        using (StreamReader reader = new StreamReader(WriteDocumentStream(gifr.FileName, ReadProtection.ReadOnly, gifr, true, Path.GetFileName(gifr.CSSFile))))
                        {
                            itemFile.Write(reader.ReadToEnd());
                        }
                    }

                    gifr.CSSFile = oldCss;
                }
            }
        }

        private static string GetValidCssFile(GuidanceItemFileReference item)
        {
            string cssFile = item.CSSFile;

            if (!File.Exists(cssFile))
            {
                cssFile = Path.Combine(ModelStorageService.TemplateFolder, item.CSSFile);

                if (!File.Exists(cssFile))
                {
                    cssFile = Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName);
                }
            }

            return cssFile;
        }

        private string BuildIndexofTitles(IList<GuidanceItemFileReference> itemList, string fileExtension, string directory)
        {
            StringBuilder indexString = new StringBuilder();
            SortedList technologyList = new SortedList();
            SortedList topicList;
            SortedList typeList;
            SortedList categoryList;
            SortedList titleList;
            //When building an Index document, use the CSS of the first Guideline item in the list

            foreach (GuidanceItem guidanceItem in itemList)
            {
                GuidanceItemFileReference item = guidanceItem as GuidanceItemFileReference;

                if (item != null)
                {
                    string fileName = Path.GetFileNameWithoutExtension(item.FileName);
                    if (!technologyList.ContainsKey(item["technology"])) //TODO: Find a better way to do this
                    {
                        topicList = new SortedList();
                        typeList = new SortedList();
                        categoryList = new SortedList();
                        titleList = new SortedList();
                        titleList.Add(fileName, item.Title);
                        categoryList.Add(item["category"], titleList); //TODO: Find a better way to do this
                        typeList.Add(item.Type, categoryList);
                        topicList.Add(item["topic"], typeList); //TODO: Find a better way to do this
                        technologyList.Add(item["technology"], topicList); //TODO: Find a better way to do this
                    }
                    else
                    {
                        topicList = (SortedList)technologyList[item["technology"]]; //TODO: Find a better way to do this
                        if (!topicList.ContainsKey(item["topic"])) //TODO: Find a better way to do this
                        {
                            typeList = new SortedList();
                            categoryList = new SortedList();
                            titleList = new SortedList();
                            titleList.Add(fileName, item.Title);
                            categoryList.Add(item["category"], titleList); //TODO: Find a better way to do this
                            typeList.Add(item.Type, categoryList);
                            topicList.Add(item["topic"], typeList); //TODO: Find a better way to do this
                        }
                        else
                        {
                            typeList = (SortedList)topicList[item["topic"]]; //TODO: Find a better way to do this
                            if (!typeList.ContainsKey(item.Type))
                            {
                                categoryList = new SortedList();
                                titleList = new SortedList();
                                titleList.Add(fileName, item.Title);
                                categoryList.Add(item["category"], titleList); //TODO: Find a better way to do this
                                typeList.Add(item.Type, categoryList);
                            }
                            else
                            {
                                categoryList = (SortedList)typeList[item.Type];
                                if (!categoryList.ContainsKey(item["category"]))
                                {
                                    titleList = new SortedList();
                                    titleList.Add(fileName, item.Title);
                                    categoryList.Add(item["category"], titleList);
                                }
                                else
                                {
                                    titleList = (SortedList)categoryList[item["category"]];
                                    if (!titleList.ContainsKey(fileName))
                                    {
                                        titleList.Add(fileName, item.Title);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            indexString.Append("<h1 class=\"title\">Guidance Index</h1>\n");

            for (int technologyListIndex = 0; technologyListIndex < technologyList.Count; technologyListIndex++)
            {
                SortedList topicList1 = (SortedList)technologyList.GetByIndex(technologyListIndex);
                for (int topicListIndex = 0; topicListIndex < topicList1.Count; topicListIndex++)
                {
                    SortedList typeList1 = (SortedList)topicList1.GetByIndex(topicListIndex);
                    indexString.Append("<h1>" + technologyList.GetKey(technologyListIndex) + " " + topicList1.GetKey(topicListIndex) + "</h1>");
                    int index = typeList1.IndexOfKey("Design");
                    if (index >= 0)
                    {
                        ExportType(typeList1, index, indexString, directory, fileExtension);
                    }

                    index = typeList1.IndexOfKey("Implementation");
                    if (index >= 0)
                    {
                        ExportType(typeList1, index, indexString, directory, fileExtension);
                    }

                    index = typeList1.IndexOfKey("Deployment");
                    if (index >= 0)
                    {
                        ExportType(typeList1, index, indexString, directory, fileExtension);
                    }

                    for (int typeListIndex = 0; typeListIndex < typeList1.Count; typeListIndex++)
                    {
                        string key = (string)typeList1.GetKey(typeListIndex);
                        if (string.Compare(key, "Design") != 0 && string.Compare(key, "Deployment") != 0 &&
                            string.Compare(key, "Implementation") != 0) //TODO: resources
                        {
                            ExportType(typeList1, typeListIndex, indexString, directory, fileExtension);
                        }
                    }
                }
            }

            return indexString.ToString();
        }

        private void ExportType(SortedList typeList1, int typeListIndex, StringBuilder indexString,
            string directory, string fileExtension)
        {
            //TODO: this method is a little complicated now. Maybe, there should be a ChecklistCollection, that inherits form ItemCollection
            SortedList categoryList1 = (SortedList)typeList1.GetByIndex(typeListIndex);
            indexString.Append("<h2>" + typeList1.GetKey(typeListIndex) + "</h2>");
            for (int categoryListIndex = 0; categoryListIndex < categoryList1.Count; categoryListIndex++)
            {
                SortedList titleList1 = (SortedList)categoryList1.GetByIndex(categoryListIndex);
                indexString.Append("<h3>" + categoryList1.GetKey(categoryListIndex) + "</h3>");
                indexString.Append("<ul>\n");

                for (int titleListIndex = 0; titleListIndex < titleList1.Count; titleListIndex++)
                {
                    string filename = (string)titleList1.GetKey(titleListIndex);
                    string title = (string)titleList1.GetByIndex(titleListIndex);
                    indexString.Append("<li>");
                    if (directory != null)
                    {
                        indexString.Append("<a href = 'file://" + Path.Combine(directory, filename) + fileExtension + "' style='font-size: 12px'>" + title + "</a>\n");
                    }
                    else
                    {
                        string link = title;//Links lenght is up to 40 chars
                        if (link.Length > 40)
                        {
                            link = link.Substring(0, 39);
                        }
                        indexString.Append("<a href = '#" + link.Replace(" ", "_") + "' style='font-size: 12px'>" + title + "</a>\n");
                    }
                    indexString.Append("</li>");
                }

                indexString.Append("</ul>\n");
            }
            indexString.Append("<hr>");
        }
    }
}

