//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;
using System.Windows.Forms;
using System.Web.Services.Protocols;
using System.Net;

using Microsoft.Practices.CompositeUI;

using online = GuidanceExplorer.OnlineGuidance;
using model = GuidanceExplorer.Model;
using GuidanceExplorer.Controls;
using System.Runtime.InteropServices;

namespace GuidanceExplorer.Services
{
    enum UploadConfirmation { None, Yes, No }

    [Service(typeof(IOnlineStorageService))]
    public class OnlineStorageService : IOnlineStorageService
    {
        /// <summary>
        /// Whether the synchronization is happening or not.
        /// </summary>
        private static bool IsSynchronizing = false;

        /// <summary>
        /// Credentials in the configuration file.
        /// </summary>
        private GuidanceExplorer.OnlineGuidance.Authentication _credentials;

        #region IOnlineStorageService Members

        /// <summary>
        /// Return a list of available libraries in the online storage.
        /// </summary>
        /// <returns></returns>
        public model.LibraryCollection GetAvailableLibraries(AvailableLibrariesMode allowCreation)
        {
            model.LibraryCollection libCollection = model.LibraryCollection.New();

            online.OnlineStorage storage = CreateProxy(true);

            //Get the navigation tree
            online.NavigationTree navTree = storage.GetNavigationTree(1); //Only the first level is required.

            //Convert online model to the offline model
            if (navTree.Libraries.Library != null)
            {
                foreach (online.Library library in navTree.Libraries.Library)
                {
                    //Create a new library.
                    model.Library newLibrary = model.Library.New();
                    newLibrary.WebSuscribed = true;
                    newLibrary.ReadProtection = (library.readProtection ? model.ReadProtection.ReadOnly : model.ReadProtection.ReadWrite);
                    newLibrary.Caption = library.caption;
                    newLibrary.Name = library.id.Split(':')[1];
                    newLibrary.SourceFileInfo = new FileInfo(Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, newLibrary.Name + ".xml")));

                    //Process the views
                    if (library.Views != null)
                    {
                        foreach (online.View view in library.Views)
                        {
                            //Guidance type nodes are ignored.
                            if (view.id.Contains("guidanceType"))
                            {
                                continue;
                            }

                            //Views within folders are ignored.
                            if (!string.IsNullOrEmpty(view.parentFolder))
                            {
                                continue;
                            }

                            //Create the view.
                            model.View newView = CreateModelView(view);
                            newView.Container = newLibrary;

                            //Add the view to the library.
                            newLibrary.Views.Add(newView.Caption, newView);

                            //Get the contents of the view.
                            PopulateLocalView(storage, view, newView);
                        }
                    }

                    //Process the views
                    if (library.Folders != null)
                    {
                        foreach (online.Folder folder in library.Folders)
                        {
                            model.Folder newFolder = model.Folder.New();
                            newFolder.Caption = folder.caption;
                            newFolder.Container = newLibrary;
                            newFolder.ParentFolder = null;

                            ProcessFolderContents(newLibrary, folder, newFolder);

                            //Add the folder to the library
                            newLibrary.Folders.Add(newFolder.Caption, newFolder);
                        }
                    }
                    //Add the library to the guidance.
                    libCollection.Add(newLibrary.Name, newLibrary);
                }
            }

            if (allowCreation == AvailableLibrariesMode.AllowCreation)
            {
                //Add a placeholder for a new library creation.
                model.Library newLibrary = model.Library.New();
                newLibrary.WebSuscribed = true;
                newLibrary.ReadProtection = model.ReadProtection.ReadWrite;
                newLibrary.Caption = "<create a new library>";
                newLibrary.Name = GuidanceExplorer.Commands.NewLibrary;
                newLibrary.SourceFileInfo = new FileInfo(Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, newLibrary.Name + ".xml")));

                //Add the library to the guidance.
                libCollection.Add(newLibrary.Name, newLibrary);
            }

            return libCollection;
        }

        private void ProcessFolderContents(model.Library library, online.Folder folder, model.Folder newFolder)
        {
            //Convert the child folders
            if (folder.Folders != null)
            {
                foreach (online.Folder childFolder in folder.Folders)
                {
                    model.Folder newChildFolder = model.Folder.New();
                    newChildFolder.Caption = childFolder.caption;
                    newChildFolder.Container = null;
                    newChildFolder.ParentFolder = newFolder;

                    newFolder.Folders.Add(newChildFolder.Caption, newChildFolder);

                    ProcessFolderContents(library, childFolder, newChildFolder);
                }
            }

            //Convert the child views
            if (folder.Views != null)
            {
                foreach (online.View childView in folder.Views)
                {
                    model.View newChildView = CreateModelView(childView);
                    newChildView.ParentFolder = newFolder;

                    newFolder.Views.Add(newChildView.Caption, newChildView);
                }
            }
        }

        private void PopulateLocalView(online.OnlineStorage storage, online.View view, model.View newView)
        {
            //Get the view contents.
            online.ListView listView = storage.GetListViewForNodeId(view.id, string.Empty, null, 0, 0);
            if (listView.Content != null && listView.Content.GuidanceItem != null)
            {
                //Create the guidance items for the view.
                foreach (online.GuidanceItem item in listView.Content.GuidanceItem)
                {
                    //If the item is already in the view, then ignore.
                    if (newView.Items.ContainsKey(item.id))
                    {
                        continue;
                    }

                    model.GuidanceItem guidanceItem = model.GuidanceItem.New();
                    guidanceItem.CSSFile = ModelStorageService.DefaultCSSFileName;
                    guidanceItem.DateTime = item.lastUpdate;
                    guidanceItem.GuidanceType = model.GuidanceType.New(item.guidanceTypeCaption);
                    guidanceItem.Id = item.id;
                    guidanceItem.Title = item.title;

                    newView.Items.Add(guidanceItem.Id, guidanceItem);
                }
            }

            // Iterate all the items from the 
            model.GuidanceItemCollection itemsToDelete = model.GuidanceItemCollection.New();
            foreach (model.GuidanceItem item in newView.Items.Values)
            {
                bool found = false;
                if (listView.Content != null && listView.Content.GuidanceItem != null)
                {
                    foreach (online.GuidanceItem onlineItem in listView.Content.GuidanceItem)
                    {
                        if (String.Equals(item.Id, onlineItem.id, StringComparison.InvariantCultureIgnoreCase))
                        {
                            found = true;
                            break;
                        }
                    }
                }

                // Check if the item is not in the local view
                if (!found)
                {
                    itemsToDelete.Add(item.Id, item);
                }
            }

            //Delete the items that was not found locally
            foreach (model.GuidanceItem item in itemsToDelete.Values)
            {
                newView.Items.Remove(item.Id);
            }

        }

        /// <summary>
        /// Creates an online view instance from the online view.
        /// </summary>
        /// <param name="library"></param>
        /// <param name="view"></param>
        /// <returns></returns>
        private model.View CreateModelView(online.View view)
        {
            model.View newView = model.View.New();
            newView.Id = view.id.Split(':')[1];
            newView.Author = view.creator;
            newView.Caption = view.caption;
            return newView;
        }

        /// <summary>
        /// Obtains the guidance items from the online storage to add each item to the library and creates 
        /// all the local files for the guidance library.
        /// </summary>
        /// <param name="modelStorage"></param>
        /// <param name="library"></param>
        public void AddGuidanceItemsFileReferences(Services.IModelStorageService modelStorage, model.Library library)
        {
            online.OnlineStorage storage = CreateProxy(true);

            //Process the items
            online.ListView listView = storage.GetListViewForNodeId(string.Format("library:{0}", library.Name), string.Empty, null, 0, 0);
            foreach (online.GuidanceItem guidanceItem in listView.Content.GuidanceItem)
            {
                //Create the new item
                model.GuidanceItemFileReference gifr = SetGuidanceItemFileReference(library, guidanceItem, model.GuidanceItemFileReference.New());

                //Create the library folder
                if (!Directory.Exists(Path.GetDirectoryName(gifr.FileName)))
                {
                    Directory.CreateDirectory(Path.GetDirectoryName(gifr.FileName));
                }

                //Create a local file
                modelStorage.RenderGuidanceItemToFileInfo(gifr, new FileInfo(gifr.FileName), storage.GetGuidanceItemContent(guidanceItem.id));

                //Add the item to the library
                library.Items.Add(gifr.Id, gifr);
            }
        }

        /// <summary>
        /// Creates a GuidanceItemFileReference from a GuidanceItem online definition.
        /// </summary>
        /// <param name="library"></param>
        /// <param name="guidanceItem"></param>
        /// <returns></returns>
        public static model.GuidanceItemFileReference SetGuidanceItemFileReference(model.Library library, online.GuidanceItem guidanceItem, GuidanceExplorer.Model.GuidanceItemFileReference gifr)
        {
            gifr.Id = guidanceItem.id;
            gifr.Title = guidanceItem.title;
            gifr.Library = library;
            gifr.CSSFile = ModelStorageService.DefaultCSSFileName;
            string tempFileName = Path.Combine(guidanceItem.guidanceTypeCaption, Path.ChangeExtension(gifr.Id, "xml"));
            tempFileName = Path.Combine(library.Caption, tempFileName);
            gifr.FileName = Path.Combine(library.SourceFileInfo.DirectoryName, tempFileName);
            gifr.DateTime = guidanceItem.lastUpdate;
            gifr.GuidanceType = model.GuidanceType.New(guidanceItem.guidanceTypeCaption);

            //Copy all the custom data
            if (guidanceItem.AnyAttr != null)
            {
                foreach (XmlAttribute attribute in guidanceItem.AnyAttr)
                {
                    gifr.CustomData[attribute.Name] = attribute.Value;
                }
            }
            return gifr;
        }

        /// <summary>
        /// Publishes the guidance item to the online storage.
        /// </summary>
        /// <param name="guidanceItem"></param>
        public bool PublishGuidanceItem(model.Library library, model.GuidanceItem guidanceItem, string content, GuidanceItemOperation operation, bool noWarning)
        {
            online.OnlineStorage storage = CreateProxy(false);

            //Check if the item already exists
            online.GuidanceItem newItem = storage.GetGuidanceItemById(guidanceItem.Id);

            //Call the online storage.
            try
            {
                //Create the item for the online storage.
                newItem = CreateOnlineGuidanceItem(library, guidanceItem);
                if (operation == GuidanceItemOperation.Create)
                {
                    storage.CreateGuidanceItem(newItem, content);
                }
                else if (operation == GuidanceItemOperation.Update)
                {
                    storage.UpdateGuidanceItem(newItem, content);
                }
            }
            catch (SoapException ex)
            {
                if (ex.Code.Name == "StorageError")
                {
                    if (!noWarning)
                    {
                        MessageBox.Show(ex.Detail.InnerText);
                    }
                }
                else if (ex.Message.IndexOf("Authorization") != -1)
                {
                    MessageBox.Show("You are not authorized to update the online store, please provide the correct credentials in the configuration file.", Messages.GUIDANCE_EXPLORER, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                else
                {
                    MessageBox.Show(ex.Message);
                    return false;
                }
            }
            return true;
        }

        public online.GuidanceItem CreateOnlineGuidanceItem(model.Library library, model.GuidanceItem guidanceItem)
        {
            online.OnlineStorage storage = CreateProxy(false);

            online.GuidanceItem newItem = new online.GuidanceItem();
            newItem.creatorCaption = "DiegoG"; //Environment.UserName;
            newItem.guidanceTypeCaption = guidanceItem.GuidanceType.Id;
            newItem.id = guidanceItem.Id;
            newItem.lastUpdate = guidanceItem.DateTime;
            newItem.library = library.Name;
            newItem.title = guidanceItem.Title;

            //Get the guidance type.
            online.GuidanceType guidanceType = storage.GetGuidanceTypeByName(guidanceItem.GuidanceType.Id);
            if (guidanceType == null)
            {
                throw new Exception("The guidance type can't be found in the online storage.");
            }

            //Get the schema.
            online.Schema schema = storage.GetSchemaById(guidanceType.schemaId);
            if (schema == null)
            {
                throw new Exception("The schema for the guidance can't be found in the online storage.");
            }

            //Create the custom fields.
            List<XmlAttribute> attributes = new List<XmlAttribute>();
            foreach (online.Field field in schema.Fields)
            {
                XmlAttribute attribute = new XmlDocument().CreateAttribute(field.name);
                attribute.Value = guidanceItem[field.name];
                attributes.Add(attribute);
            }
            newItem.AnyAttr = attributes.ToArray();

            return newItem;
        }

        /// <summary>
        /// Performs a synchronization of the local storage and the online storage.
        /// </summary>
        /// <param name="modelService"></param>
        /// <param name="htmlRender"></param>
        public void Syncronize(Services.IModelStorageService modelService, IHtmlRenderService htmlRender, bool showMessages, DoWorkEventArgs eventArgs)
        {
            if (IsSynchronizing)
            {
                MessageBox.Show("Synchronization has already been started", "Online Store", MessageBoxButtons.OK, MessageBoxIcon.Information);
                if (eventArgs != null)
                {
                    eventArgs.Cancel = true;
                }
                return;
            }

            IsSynchronizing = true;
            try
            {
                model.Guidance guidance = modelService.Guidance;

                online.OnlineStorage storage = CreateProxy(true);

                int localAdded = 0;
                int onlineAdded = 0;
                int localDeleted = 0;
                //int onlineDeleted = 0;
                int localUpdated = 0;
                int onlineUpdated = 0;
                int notChanged = 0;

                foreach (model.Library library in guidance.Libraries.Values)
                {
                    //Ignore the libraries that are not web suscribed.
                    if (!library.WebSuscribed)
                    {
                        continue;
                    }

                    //Get library index.
                    online.LibraryIndex libIndex = null;
                    try
                    {
                        libIndex = storage.GetLibraryIndex(string.Format("library:{0}", library.Name));
                    }
                    catch
                    {
                        if (showMessages)
                        {
                            if (eventArgs != null)
                            {
                                eventArgs.Cancel = true;
                            }

                            MessageBox.Show("The synchronization can't connect to the online store.", "Online Storage");
                            return;
                        }
                    }

                    //If there is no index, then skip the library.
                    if (libIndex == null)
                    {
                        continue;
                    }

                    //Get the online library.
                    online.Library onlineLib = storage.GetLibraryById(library.Name);

                    //The library was not in the store.
                    if (onlineLib == null)
                    {
                        continue;
                    }

                    model.ReadProtection oldReadProtection = library.ReadProtection;
                    try
                    {
                        library.ReadProtection = GuidanceExplorer.Model.ReadProtection.ReadOnly;

                        //Update the name of the library
                        library.Caption = onlineLib.caption;

                        //Syncronize the guidance items.
                        SynchronizeLibraryGuidanceItems(modelService, htmlRender, storage, library, libIndex, ref localAdded, ref onlineAdded, ref localDeleted, ref localUpdated, ref onlineUpdated, ref notChanged); //TODO: hmm automatic refactor :)

                        //Synchronize the views.
                        SynchronizeLibraryViews(modelService, htmlRender, storage, library, onlineLib);

                        //If the library is read-only, there must be no folders with no content.
                        if (model.ReadProtection.ReadOnly == library.ReadProtection)
                        {
                            model.FolderCollection folders = model.FolderCollection.New();
                            foreach (model.Folder folder in library.Folders.Values)
                            {
                                if (folder.Folders.Count == 0 && folder.Views.Count == 0)
                                {
                                    folders.Add(folder.Caption, folder);
                                }
                                ProcessChildFolders(folder, folders);
                            }

                            //Remove all the folders.
                            foreach (model.Folder folder in folders.Values)
                            {
                                folder.Remove();
                            }
                        }
                    }
                    finally
                    {
                        library.ReadProtection = oldReadProtection;
                    }
                }
            }
            catch { }
            finally
            {
                IsSynchronizing = false;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="folder"></param>
        /// <returns></returns>
        private void ProcessChildFolders(model.Folder parentFolder, model.FolderCollection folders)
        {
            foreach (model.Folder folder in parentFolder.Folders.Values)
            {
                if (folder.Folders.Count == 0 && folder.Views.Count == 0)
                {
                    folders.Add(folder.Caption, folder);
                }
                ProcessChildFolders(folder, folders);
            }
        }

        /// <summary>
        /// Synchronize the views.
        /// </summary>
        /// <param name="modelService"></param>
        /// <param name="storage"></param>
        /// <param name="library"></param>
        private void SynchronizeLibraryViews(IModelStorageService modelService, IHtmlRenderService htmlRender, online.OnlineStorage storage, model.Library library, online.Library onlineLibrary)
        {
            //Find all the views in the remote that are not local.
            if (null != onlineLibrary.Views)
            {
                foreach (online.View onlineView in onlineLibrary.Views)
                {
                    model.View modelView = null;
                    bool found = false;
                    foreach (model.View tempView in library.AllViews.Values)
                    {
                        if (String.Equals(tempView.Id, onlineView.id.Split(':')[1], StringComparison.InvariantCultureIgnoreCase))
                        {
                            modelView = tempView;
                            found = true;
                            break;
                        }
                    }

                    if (!found)
                    {
                        //Create the view.
                        model.View newView = CreateModelView(onlineView);
                        if (string.IsNullOrEmpty(onlineView.parentFolder))
                        {
                            newView.Container = library;
                            //Add the view to the library.
                            library.Views.Add(newView.Caption, newView);
                        }
                        else
                        {
                            bool root = true;
                            string[] allFolders = onlineView.parentFolder.Split('/');
                            model.Folder parentFolder = null;
                            model.Folder currentFolder = null;
                            foreach (string folderName in allFolders)
                            {
                                currentFolder = null;
                                //Find the root fodler or the next nesting level.
                                if (root)
                                {
                                    root = false;
                                    if (library.Folders.ContainsKey(folderName))
                                    {
                                        currentFolder = parentFolder = library.Folders[folderName];
                                    }
                                }
                                else
                                {
                                    if (parentFolder.Folders.ContainsKey(folderName))
                                    {
                                        currentFolder = parentFolder.Folders[folderName];
                                    }
                                }

                                //If the folder does not exist, then create it.
                                if (currentFolder == null)
                                {
                                    currentFolder = model.Folder.New();
                                    currentFolder.Caption = folderName;
                                    if (parentFolder == null)
                                    {
                                        currentFolder.Container = library;
                                        library.Folders.Add(currentFolder.Caption, currentFolder);
                                    }
                                    else
                                    {
                                        currentFolder.ParentFolder = parentFolder;
                                        parentFolder.Folders.Add(currentFolder.Caption, currentFolder);
                                    }
                                }

                                //Set the previous parent
                                parentFolder = currentFolder;
                            }

                            //Set the view to the folder
                            newView.ParentFolder = currentFolder;
                            currentFolder.Views.Add(newView.Caption, newView);
                        }

                        //Populate the local view.
                        PopulateLocalView(storage, onlineView, newView);
                    }
                    else if (found)
                    {
                        if (onlineView.delete)
                        {
                            modelView.Remove();
                        }
                        else
                        {
                            if (onlineView.caption != modelView.Caption)
                            {
                                if (modelView.Container != null)
                                {
                                    modelView.Container.Views.Remove(modelView.Caption);
                                    modelView.Container.Views.Add(onlineView.caption, modelView);
                                }
                                else
                                {
                                    modelView.ParentFolder.Views.Remove(modelView.Caption);
                                    modelView.ParentFolder.Views.Add(onlineView.caption, modelView);
                                }
                                modelView.Caption = onlineView.caption;
                            }

                            //Synchronize view contents. Since the view was found on local and online store.
                            PopulateLocalView(storage, onlineView, modelView);
                        }
                    }
                }
            }

            //Find all the views in the local that are not remote.
            foreach (model.View modelView in library.AllViews.Values)
            {
                bool found = false;
                online.View onlineView = null;
                if (onlineLibrary.Views != null)
                {
                    foreach (online.View tempView in onlineLibrary.Views)
                    {
                        if (String.Equals(modelView.Id, tempView.id.Split(':')[1], StringComparison.InvariantCultureIgnoreCase))
                        {
                            onlineView = tempView;
                            found = true;
                            break;
                        }
                    }
                }

                if (found)
                {
                    if (onlineView.delete)
                    {
                        modelView.Remove();
                    }
                    else
                    {
                        if (onlineView.caption != modelView.Caption)
                        {
                            if (modelView.Container != null)
                            {
                                modelView.Container.Views.Remove(modelView.Caption);
                                modelView.Container.Views.Add(onlineView.caption, modelView);
                            }
                            else
                            {
                                modelView.ParentFolder.Views.Remove(modelView.Caption);
                                modelView.ParentFolder.Views.Add(onlineView.caption, modelView);
                            }
                            modelView.Caption = onlineView.caption;
                        }

                        // The view exist and the previous pass has synchronized all the view contents.
                    }
                }
                else if (!found)
                {
                    //A view has been added locally, remove it.
                    modelView.Remove();
                }
            }
        }

        private void PublishViewContent(IModelStorageService modelService, IHtmlRenderService htmlRender, model.Library library, model.View modelView, string viewId)
        {
            model.GuidanceItemFileReferenceCollection gifrs = model.GuidanceItemFileReferenceCollection.New();
            foreach (model.GuidanceItem guidanceItem in modelView.Items)
            {
                model.GuidanceItemFileReference gifr = guidanceItem as model.GuidanceItemFileReference;
                if (gifr != null)
                {
                    gifrs.Add(gifr.Id, gifr);
                }
                else if (guidanceItem.ReferencedItem != null)
                {
                    gifrs.Add(guidanceItem.ReferencedItem.Id, guidanceItem.ReferencedItem);
                }
                else
                {
                    //Find the reference.
                    modelService.Guidance.GetGuidanceItemFileReference(guidanceItem);

                    gifrs.Add(guidanceItem.ReferencedItem.Id, guidanceItem.ReferencedItem);
                }
            }

            //If there is no current item.
            if (gifrs.Count != 0)
            {
                //Publish the selected guidance items
                foreach (model.GuidanceItemFileReference gifr in gifrs)
                {
                    //Add the item to the library.
                    using (StreamReader reader = new StreamReader(htmlRender.GetRawDocumentStream(gifr)))
                    {
                        if (!PublishGuidanceItem(library, gifr, reader.ReadToEnd(), GuidanceItemOperation.Create, true))
                        {
                            break;
                        }
                    }
                }

                //Add the item to the view.
                AddGuidanceItemToView(new Guid(viewId), gifrs);
            }
        }

        /// <summary>
        /// Publish the guidance items for an entire library.
        /// </summary>
        /// <param name="modelService"></param>
        /// <param name="htmlRender"></param>
        /// <param name="storage"></param>
        /// <param name="library"></param>
        /// <param name="libIndex"></param>
        /// <param name="localAdded"></param>
        /// <param name="onlineAdded"></param>
        /// <param name="localDeleted"></param>
        /// <param name="localUpdated"></param>
        /// <param name="onlineUpdated"></param>
        /// <param name="notChanged"></param>
        private void SynchronizeLibraryGuidanceItems(Services.IModelStorageService modelService, IHtmlRenderService htmlRender, online.OnlineStorage storage, model.Library library, online.LibraryIndex libIndex, ref int localAdded, ref int onlineAdded, ref int localDeleted, ref int localUpdated, ref int onlineUpdated, ref int notChanged)
        {
            UploadConfirmation uploadConfirmation = UploadConfirmation.None;

            //Check each remote item date time with local last updated.
            foreach (online.GuidanceItemIndex itemIndex in libIndex.GuidanceItems)
            {
                model.GuidanceItemFileReference gifr = null;
                if (library.Items.ContainsKey(itemIndex.id))
                {
                    gifr = library.Items[itemIndex.id] as model.GuidanceItemFileReference;
                }
                if (gifr == null && !itemIndex.deleted)
                {
                    //There is a new item in the online library.

                    //Get the online guidance  item definition
                    online.GuidanceItem guidanceItem = storage.GetGuidanceItemById(itemIndex.id);

                    //Create a local guidance item
                    gifr = SetGuidanceItemFileReference(library, guidanceItem, model.GuidanceItemFileReference.New());

                    //Create the library folder
                    if (!Directory.Exists(Path.GetDirectoryName(gifr.FileName)))
                    {
                        Directory.CreateDirectory(Path.GetDirectoryName(gifr.FileName));
                    }

                    //Create a local file
                    modelService.RenderGuidanceItemToFileInfo(gifr, new FileInfo(gifr.FileName), storage.GetGuidanceItemContent(guidanceItem.id));

                    //Add the item to the library.
                    library.Items.Add(gifr.Id, gifr);

                    //Update the count.
                    localAdded++;
                }
                else if (gifr != null)
                {
                    //The item already exist locally, compare date time.
                    if (itemIndex.deleted)
                    {
                        //Remove the guidance item from the model.
                        modelService.DeleteGuidanceItem(gifr, library);

                        //Remove guidance across the library
                        modelService.RemoveGuidanceItemFromLibrary(library, gifr);

                        //Remove the pointers from the user views.
                        foreach (model.View view in modelService.Guidance.UserViews.Views.Values)
                        {
                            view.Items.Remove(gifr.Id);
                        }

                        //Remove guidance across 
                        modelService.RemoveGuidanceItemFromAllViews(modelService.Guidance.UserViews.Folders, gifr);

                        //Update the count.
                        localDeleted++;
                    }
                    else if (gifr.DateTime == itemIndex.lastUpdate)
                    {
                        //Both items have the same date.

                        //Update the count.
                        notChanged++;
                    }
                    else if (gifr.DateTime < itemIndex.lastUpdate)
                    {
                        //Local item is older.

                        //Get the online guidance  item definition
                        online.GuidanceItem guidanceItem = storage.GetGuidanceItemById(itemIndex.id);

                        //Create a local file
                        modelService.RenderGuidanceItemToFileInfo(gifr, new FileInfo(gifr.FileName), storage.GetGuidanceItemContent(guidanceItem.id));

                        //Create a local guidance item
                        gifr = SetGuidanceItemFileReference(library, guidanceItem, gifr);

                        //Update the count.
                        localUpdated++;
                    }
                    else if (gifr.DateTime > itemIndex.lastUpdate)
                    {
                        //Online item is older.

                        if (uploadConfirmation == UploadConfirmation.None)
                        {
                            if (MessageBox.Show("Do you want to upload item updates to the online library?", "Online Storage", MessageBoxButtons.YesNo) == DialogResult.Yes)
                            {
                                uploadConfirmation = UploadConfirmation.Yes;
                            }
                            else
                            {
                                uploadConfirmation = UploadConfirmation.No;
                            }
                        }

                        if (uploadConfirmation == UploadConfirmation.Yes)
                        {
                            using (StreamReader reader = new StreamReader(htmlRender.GetRawDocumentStream(gifr)))
                            {
                                if (!PublishGuidanceItem(library, gifr, reader.ReadToEnd(), GuidanceItemOperation.Update, false))
                                {
                                    break;
                                }
                            }

                            //Update the count.
                            onlineUpdated++;
                        }
                    }
                }
            }

            //Check if there is a new item in the local store.
            foreach (model.GuidanceItem guidanceItem in library.Items.Values)
            {
                bool found = false;
                foreach (online.GuidanceItemIndex itemIndex in libIndex.GuidanceItems)
                {
                    if (string.Equals(guidanceItem.Id, itemIndex.id, StringComparison.InvariantCultureIgnoreCase))
                    {
                        found = true;
                        break;
                    }
                }
                if (!found)
                {
                    //There is a new item in the local store

                    if (uploadConfirmation == UploadConfirmation.None)
                    {
                        if (MessageBox.Show("Do you want to upload item updates to the online library?", "Online Storage", MessageBoxButtons.YesNo) == DialogResult.Yes)
                        {
                            uploadConfirmation = UploadConfirmation.Yes;
                        }
                        else
                        {
                            uploadConfirmation = UploadConfirmation.No;
                        }
                    }

                    if (uploadConfirmation == UploadConfirmation.Yes)
                    {
                        model.GuidanceItemFileReference gifr = guidanceItem as model.GuidanceItemFileReference;
                        if (gifr == null)
                        {
                            continue;
                        }

                        using (StreamReader reader = new StreamReader(htmlRender.GetRawDocumentStream(gifr)))
                        {
                            if (!PublishGuidanceItem(library, gifr, reader.ReadToEnd(), GuidanceItemOperation.Create, false))
                            {
                                break;
                            }
                        }
                    }

                    //Update the count.
                    onlineAdded++;
                }
            }
        }

        /// <summary>
        /// Creates a view in the online storage for the given library.
        /// </summary>
        /// <param name="suscribedLibrary"></param>
        /// <param name="view"></param>
        /// <returns></returns>
        public string CreateOnlineView(model.Library library, ViewNode viewNode)
        {
            online.OnlineStorage storage = CreateProxy(false);
            try
            {
                StringBuilder sb = new StringBuilder();
                bool first = true;
                for (TreeNode tempNode = viewNode.Parent; !(tempNode is LibraryNode); tempNode = tempNode.Parent)
                {
                    if (first)
                    {
                        first = false;
                    }
                    else
                    {
                        sb.Insert(0, "/");
                    }
                    sb.Insert(0, tempNode.Text);
                }
                string folderName = sb.ToString();

                online.View newView = new online.View();
                newView.caption = viewNode.ModelInstance.Caption;
                newView.creator = string.Empty;
                newView.creatorCaption = "DiegoG";//Environment.UserName;
                newView.delete = false;
                newView.id = viewNode.ModelInstance.Id;
                newView.library = library.Name;
                newView.parentFolder = folderName;

                storage.CreateView(newView);

                return newView.id;
            }
            catch (SoapException ex)
            {
                if (ex.Code.Name == "StorageError")
                {
                    MessageBox.Show(ex.Detail.InnerText);
                }
                else if (ex.Message.IndexOf("Authorization") != -1)
                {
                    MessageBox.Show("You are not authorized to update the online store, please provide the correct credentials in the configuration file.", Messages.GUIDANCE_EXPLORER, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show(ex.Message);
                }
                throw;
            }
        }

        /// <summary>
        /// Add an item to the view.
        /// </summary>
        /// <param name="viewId"></param>
        /// <param name="gifr"></param>
        public void AddGuidanceItemToView(Guid viewId, model.GuidanceItemFileReferenceCollection guidanceItems)
        {
            online.OnlineStorage storage = CreateProxy(false);

            //Create the online representation of the guidance items.
            online.GuidanceItem[] newGuidanceItems = new online.GuidanceItem[guidanceItems.Count];
            int guidanceIdx = 0;
            foreach (model.GuidanceItemFileReference guidanceItem in guidanceItems)
            {
                newGuidanceItems[guidanceIdx] = new online.GuidanceItem();
                newGuidanceItems[guidanceIdx].id = guidanceItem.Id;
                guidanceIdx++;
            }

            try
            {
                //Add the items to the storage.
                storage.AddGuidanceItemsToView(viewId.ToString(), newGuidanceItems);
            }
            catch (SoapException ex)
            {
                if (ex.Code.Name == "StorageError")
                {
                }
                else
                {
                    throw;
                }
            }
        }

        /// <summary>
        /// Creates a library in the online storage.
        /// </summary>
        /// <param name="library"></param>
        public void CreateLibrary(model.Library library)
        {
            online.OnlineStorage storage = CreateProxy(false);

            online.Library newLibrary = new online.Library();
            newLibrary.caption = library.Caption;
            newLibrary.delete = false;
            newLibrary.id = Guid.NewGuid().ToString();
            newLibrary.readProtection = library.ReadProtection == GuidanceExplorer.Model.ReadProtection.ReadOnly;

            try
            {
                storage.CreateLibrary(newLibrary);

                library.Name = newLibrary.id;
                library.WebSuscribed = true;
            }
            catch (SoapException ex)
            {
                if (string.IsNullOrEmpty(ex.Detail.InnerText))
                {
                    throw new Exception(ex.Message);
                }
                else
                {
                    throw new Exception(ex.Detail.InnerText);
                }
            }
        }

        public string GetViewIdByCaption(model.Library library, string viewCaption)
        {
            online.OnlineStorage storage = CreateProxy(true);

            //Get the online library.
            online.Library onlineLib = storage.GetLibraryById(library.Name);

            if (onlineLib.Views != null)
            {
                foreach (online.View onlineView in onlineLib.Views)
                {
                    if (viewCaption == onlineView.caption)
                    {
                        return onlineView.id.Split(':')[1];
                    }
                }
            }

            return null;
        }

        public bool RemoveGuidanceItem(model.GuidanceItemFileReference guidanceItem)
        {
            online.OnlineStorage storage = CreateProxy(false);

            online.GuidanceItem newItem = new online.GuidanceItem();
            newItem.id = guidanceItem.Id;
            newItem.delete = true;

            //Call the online storage.
            try
            {
                storage.UpdateGuidanceItem(newItem, string.Empty);
            }
            catch (SoapException ex)
            {
                if (ex.Code.Name == "StorageError")
                {
                    MessageBox.Show(ex.Detail.InnerText);
                }
                else if (ex.Message.IndexOf("Authorization") != -1)
                {
                    MessageBox.Show("You are not authorized to update the online store, please provide the correct credentials in the configuration file.", Messages.GUIDANCE_EXPLORER, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                else
                {
                    MessageBox.Show(ex.Message);
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Remove a view in the online store based on the model definition.
        /// </summary>
        /// <param name="view"></param>
        public void RemoveView(model.View view)
        {
            online.OnlineStorage storage = CreateProxy(false);

            model.Library library = view.GetContainerLibrary();
            if (library == null)
            {
                MessageBox.Show("The view is not a library view.");
                return;
            }

            string viewId = GetViewIdByCaption(library, view.Caption);

            online.View onlineView = new online.View();
            onlineView.id = viewId;
            onlineView.delete = true;

            try
            {
                storage.UpdateView(onlineView);
            }
            catch (SoapException ex)
            {
                if (ex.Code.Name == "StorageError")
                {
                    MessageBox.Show(ex.Detail.InnerText);
                }
                else if (ex.Message.IndexOf("Authorization") != -1)
                {
                    MessageBox.Show("You are not authorized to update the online store, please provide the correct credentials in the configuration file.", Messages.GUIDANCE_EXPLORER, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        #endregion

        /// <summary>
        /// If configured, it sets the WebProxy settings
        /// </summary>
        /// <param name="storage"></param>
        private online.OnlineStorage CreateProxy(bool readOnly)
        {
            online.OnlineStorage storage = new online.OnlineStorage();
            string proxyUrl = global::GuidanceExplorer.Properties.Settings.Default.GuidanceExplorer_OnlineGuidance_OnlineStorage_WebProxy;
            if (!string.IsNullOrEmpty(proxyUrl))
            {
                WebProxy proxy = new WebProxy(proxyUrl);
                proxy.Credentials = CredentialCache.DefaultCredentials;
                storage.Proxy = proxy;
            }

            if (!readOnly)
            {
                if (_credentials == null)
                {
                    _credentials = new GuidanceExplorer.OnlineGuidance.Authentication();
                    _credentials.User = global::GuidanceExplorer.Properties.Settings.Default.GuidanceExplorer_OnlineGuidance_OnlineStorage_UserName;
                    _credentials.Password = global::GuidanceExplorer.Properties.Settings.Default.GuidanceExplorer_OnlineGuidance_OnlineStorage_Password;
                }
                storage.AuthenticationValue = _credentials;
            }

            return storage;
        }
    }
}
