//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using GuidanceExplorer.Model;
using GuidanceExplorer.Controls;
using System.IO;
using System.Drawing;
using Microsoft.Practices.CompositeUI;
using GuidanceExplorer.Views;

namespace GuidanceExplorer.Services
{
    [Service(typeof(IWinFormsRenderService))]
    public class WinFormsRenderService : IWinFormsRenderService
    {
        /// <summary>
        /// Render an instance of the Guidance model into a TreeView control.
        /// </summary>
        /// <param name="guidance">The instance in the model.</param>
        /// <param name="treeView">The tree view instance to render to.</param>
        public void RenderGuidanceToTreeView(Guidance guidance, TreeView treeView)
        {
            //Create two root nodes: My Views and Libraries.
            MyViewsNode myViewsNode = MyViewsNode.New();
            LibrariesNode librariesNode = LibrariesNode.New();

            if (guidance.ClientFlavour == ClientFlavour.Offline)
            {
                //Set the model instance to the my views node.
                myViewsNode.ModelInstance = guidance.UserViews;

                //Add nodes to the TreeView.
                treeView.Nodes.Add(myViewsNode);
                treeView.Nodes.Add(librariesNode);

                //Populate My Views node.
                RenderUserViewsToMyViews(treeView, guidance.UserViews, myViewsNode, guidance);

                //Populate Libraries node.
                RenderLibraryCollectionToLibrariesNode(treeView, guidance, guidance.Libraries, librariesNode);

                if (treeView.Nodes[0].Text != "What's New")
                {
                    SearchView searchView = SearchView.New();
                    searchView.Author = "diegog";
                    searchView.Description = "";
                    searchView.ContainerScope = guidance;
                    searchView.RootView = true;
                    searchView.Caption = "What's New";
                    searchView.CreationDate = DateTime.Now;
                    searchView.Criteria = new TopSearchCriteria(new List<Criteria>(), 25);
                    searchView.Columns = ColumnCollection.DefaultColumns(GuidanceType.Get(GuidanceType.DefaultTypeName));
                    foreach (Column col in searchView.Columns.Values)
                    {
                        col.Sort = SortOrder.None;
                    }

                    SearchViewNode searchViewNode = SearchViewNode.New();
                    searchViewNode.Text = searchView.Caption;
                    searchViewNode.ModelInstance = searchView;
                    searchView.ContainerScope = guidance;

                    guidance.UserViews.Views.Add(searchView.Caption, searchView);

                    treeView.Nodes.Insert(0, searchViewNode);
                }

                //Expand the libraries node.
                librariesNode.Expand();

                //Expand my views node.
                myViewsNode.Expand();

                //Set the libraries node.
                treeView.SelectedNode = librariesNode;
            }
            else if (guidance.ClientFlavour == ClientFlavour.AuthoringToolkit)
            {
                AuthoringToolkitNode authoringToolkitNode = AuthoringToolkitNode.New();

                //Add nodes to the TreeView.
                treeView.Nodes.Add(authoringToolkitNode);

                //Set the libraries node.
                treeView.SelectedNode = authoringToolkitNode;
            }

            //Set the focus to the treeView so the selection is visible.
            treeView.Select();

        }

        /// <summary>
        /// Renders the UserViews in the MyViews node in the TreeView
        /// </summary>
        /// <param name="views">The user views and folders in the model.</param>
        /// <param name="myViewsNode">The node where the user views must be placed.</param>
        private void RenderUserViewsToMyViews(TreeView treeView, UserViews views, MyViewsNode myViewsNode, Guidance guidance)
        {
            //Iterate through all the folders.
            foreach (Folder folder in views.Folders.Values)
            {
                //Create a new folder node.
                FolderNode folderNode = FolderNode.New();

                //Add the node to the my views node.
                myViewsNode.Nodes.Add(folderNode);

                //Render the Folder to the FolderNode
                RenderFolderToFolderNode(treeView, folder, folderNode, guidance);
            }

            //Iterate through all the views.
            foreach (GuidanceExplorer.Model.View view in views.Views.Values)
            {
                SearchView searchView = view as SearchView;
                if (searchView != null)
                {
                    //Create the search view node.
                    SearchViewNode searchViewNode = SearchViewNode.New();
                    searchViewNode.Text = searchView.Caption;
                    searchViewNode.ModelInstance = searchView;
                    searchView.ContainerScope = guidance;

                    if (view.RootView)
                    {
                        treeView.Nodes.Insert(0, searchViewNode);
                    }
                    else
                    {
                        myViewsNode.Nodes.Insert(0, searchViewNode);
                    }
                }
                else
                {
                    //Create a new view node.
                    ViewNode viewNode = ViewNode.New();

                    //Set the node properties
                    viewNode.Text = view.Caption;
                    viewNode.ModelInstance = view;

                    //Add the node to the my views node.
                    if (view.RootView)
                    {
                        treeView.Nodes.Insert(0, viewNode);
                    }
                    else
                    {
                        myViewsNode.Nodes.Insert(0, viewNode);
                    }
                }
            }
        }

        /// <summary>
        /// Renders a collection of libraries into a LibrariesNode in the TreeView.
        /// </summary>
        /// <param name="libraries"></param>
        /// <param name="librariesNode"></param>
        private void RenderLibraryCollectionToLibrariesNode(TreeView treeView, Guidance guidance, LibraryCollection libraries, LibrariesNode librariesNode)
        {
            //Iterate throug all the libraries.
            foreach (KeyValuePair<string, Library> library in libraries)
            {
                RenderLibraryToLibrariesNode(treeView, guidance, library.Value, librariesNode);
            }
        }

        /// <summary>
        /// Renders a library into a LibraryNode in the TreeView.
        /// </summary>
        /// <param name="library"></param>
        /// <param name="libraryNode"></param>
        public void RenderLibraryToLibrariesNode(TreeView treeView, Guidance guidance, Library library, LibrariesNode librariesNode)
        {
            //Create a new library node.
            LibraryNode libraryNode = LibraryNode.New();

            //Add the node to the libraries node.
            librariesNode.Nodes.Add(libraryNode);

            //Render the library to the library node.
            RenderLibraryToLibraryNode(treeView, guidance, library, libraryNode);
        }

        /// <summary>
        /// Renders a Library into a LibraryNode instance.
        /// </summary>
        /// <param name="library">The library instance to render.</param>
        /// <param name="libraryNode">The library node to render to.</param>
        private void RenderLibraryToLibraryNode(TreeView treeView, Guidance guidance, Library library, LibraryNode libraryNode)
        {
            //Set the node properties
            libraryNode.Text = library.Caption;
            libraryNode.ModelInstance = library;

            //Iterate through all the folders.
            foreach (Folder folder in library.Folders.Values)
            {
                //Create a new folder node.
                FolderNode folderNode = FolderNode.New();

                //Add the node to the my views node.
                libraryNode.Nodes.Add(folderNode);

                //Render the Folder to the FolderNode
                RenderFolderToFolderNode(treeView, folder, folderNode, guidance);
            }

            //Iterate through all the views.
            foreach (GuidanceExplorer.Model.View view in library.Views.Values)
            {
                SearchView searchView = view as SearchView;
                if (searchView == null)
                {
                    //Create a new view node.
                    ViewNode viewNode = ViewNode.New();

                    //Set the node properties
                    viewNode.Text = view.Caption;
                    viewNode.ModelInstance = view;

                    //Add the node to the my views node.
                    libraryNode.Nodes.Add(viewNode);
                }
                else
                {
                    //Create a new search view node.
                    SearchViewNode searchViewNode = SearchViewNode.New();

                    //Process the search view context.
                    //switch (searchView.ReadedScope)
                    //{
                    //    case SearchViewScope.Guidance:
                    //        searchView.ContainerScope = guidance;
                    //        break;
                    //    case SearchViewScope.Library:
                    //        searchView.ContainerScope = library;
                    //        break;
                    //    case SearchViewScope.Folder:
                    //        searchView.ContainerScope = library.FindFolderByCaption(searchView.ReadedScopeName);
                    //        break;
                    //    case SearchViewScope.View:
                    //        searchView.ContainerScope = library.FindViewByCaption(searchView.ReadedScopeName);
                    //        break;
                    //}

                    searchView.ContainerScope = guidance;

                    //Set the node properties
                    searchViewNode.Text = searchView.Caption;
                    searchViewNode.ModelInstance = searchView;

                    //Add the node to the my views node.
                    libraryNode.Nodes.Add(searchViewNode);
                }
            }

            //Create all the guidance type nodes.
            foreach (GuidanceType guidanceType in GuidanceType.AllGuidanceTypes)
            {
                //Guidance type with display name as empty, means not visible in the UI.
                if (!String.IsNullOrEmpty(guidanceType.DisplayName))
                {
                    RefreshGuidanceTypeNodes(libraryNode, guidanceType);
                }
            }

        }

        /// <summary>
        /// Refresh the GuidanceType nodes that belong to the library
        /// </summary>
        /// <param name="libraryNode">LibraryNode to refresh</param>
        /// <param name="guidanceType">GuidanceType updated</param>
        public void RefreshGuidanceTypeNodes(LibraryNode libraryNode, GuidanceType guidanceType)
        {
            int itemsCount = 0;

            if (string.IsNullOrEmpty(guidanceType.NodeDisplayName))
            {
                itemsCount = libraryNode.ModelInstance.GuidanceItemCountInLibrary(guidanceType);
            }
            else
            {
                itemsCount = libraryNode.ModelInstance.GuidanceItemCountInLibrary(guidanceType.NodeDisplayName);
            }

            if (itemsCount > 0)
            {
                if (!libraryNode.HasGuidanceTypeNode(guidanceType))
                {
                    GuidanceTypeNode guidanceTypeNode = GuidanceTypeNode.New();
                    guidanceTypeNode.Text = guidanceType.ListDisplayName;
                    guidanceTypeNode.ModelInstance = guidanceType;
                    libraryNode.AddGuidanceTypeNode(guidanceTypeNode);

                    //Set the icon to the guidance type and guidance type node.
                    string fullImageFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceType.ImageFileName));
                    if (File.Exists(fullImageFileName))
                    {
                        using (FileStream fileStream = new FileStream(fullImageFileName, FileMode.Open, FileAccess.Read))
                        {
                            Image nodeImage = Image.FromStream(fileStream);
                            int imageIndex = libraryNode.TreeView.ImageList.Images.Add(nodeImage, Color.Magenta);
                            guidanceTypeNode.ImageIndex = guidanceTypeNode.SelectedImageIndex = imageIndex;
                        }
                    }

                    //Expand the LibraryNode
                    if (!libraryNode.IsExpanded)
                    {
                        libraryNode.Expand();
                    }
                }
            }
            else if (libraryNode.HasGuidanceTypeNode(guidanceType))
            {
                libraryNode.RemoveGuidanceTypeNode(guidanceType);
            }
        }

        /// <summary>
        /// Renders a Folder into a FolderNode.
        /// </summary>
        /// <param name="folder">The folder to render.</param>
        /// <param name="folderNode">The folder node to render to.</param>
        private void RenderFolderToFolderNode(TreeView treeView, Folder folder, FolderNode folderNode, Guidance guidance)
        {
            //Set the node properties
            folderNode.Text = folder.Caption;
            folderNode.ModelInstance = folder;

            //Render the child folders.
            RenderFolderCollectionToFolderNode(treeView, folder.Folders as FolderCollection, folderNode, guidance);

            //Iterate through all the views.
            foreach (GuidanceExplorer.Model.View view in folder.Views.Values)
            {
                SearchView searchView = view as SearchView;
                if (searchView != null)
                {
                    //Create a new search view node.
                    SearchViewNode searchViewNode = SearchViewNode.New();

                    searchView.ContainerScope = guidance;

                    //Set the node properties
                    searchViewNode.Text = searchView.Caption;
                    searchViewNode.ModelInstance = searchView;

                    //Add the node to the my views node.
                    if (view.RootView)
                    {
                        treeView.Nodes.Insert(0, searchViewNode);
                    }
                    else
                    {
                        folderNode.Nodes.Insert(0, searchViewNode);
                    }
                }
                else
                {
                    //Create a new view node.
                    ViewNode viewNode = ViewNode.New();

                    //Set the node properties
                    viewNode.Text = view.Caption;
                    viewNode.ModelInstance = view;

                    //Add the node to the my views node.
                    if (view.RootView)
                    {
                        treeView.Nodes.Insert(0, viewNode);
                    }
                    else
                    {
                        folderNode.Nodes.Insert(0, viewNode);
                    }
                }
            }
        }

        /// <summary>
        /// Renders the Folder into the folderNode.
        /// </summary>
        /// <param name="folders">The list of folders to render.</param>
        /// <param name="parentFolderNode">The parent node where the folder will be created.</param>
        private void RenderFolderCollectionToFolderNode(TreeView treeView, FolderCollection folders, FolderNode parentFolderNode, Guidance guidance)
        {
            //Iterate through all the folders
            foreach (Folder folder in folders.Values)
            {
                //Create the folder node
                FolderNode folderNode = FolderNode.New();

                //Add the folder node to the parent node
                parentFolderNode.Nodes.Add(folderNode);

                //Render folder to the node.
                RenderFolderToFolderNode(treeView, folder, folderNode, guidance);
            }
        }

        /// <summary>
        /// Populates the given ComboBox with the items' information
        /// </summary>
        /// <param name="combo"></param>
        /// <param name="field"></param>
        /// <param name="items"></param>
        public void PopulateCombo(ComboBox combo, Field field, GuidanceItemCollection items)
        {
            List<string> values = new List<string>();

            foreach (GuidanceItem item in items)
            {
                string value = item[field.Name] ?? string.Empty;
                if (!values.Contains(value))
                {
                    values.Add(value);
                }
            }

            values.Sort();

            combo.Items.Clear();
            combo.Items.AddRange(values.ToArray());
        }

        /// <summary>
        /// Gets the Tree Node asociated to the Model Instance
        /// </summary>
        /// <param name="library"></param>
        /// <returns></returns>
        public LibraryNode GetLibraryNodeByLibrary(GuidanceExplorerTreeView treeView, Library library)
        {
            foreach (TreeNode node in treeView.TreeView.Nodes)
            {
                LibrariesNode libsNode = node as LibrariesNode;
                if (libsNode == null)
                {
                    continue;
                }

                foreach (TreeNode libNode in libsNode.Nodes)
                {
                    IModelItem modelNode = libNode as IModelItem;
                    if (modelNode == null)
                    {
                        continue;
                    }

                    LibraryNode lib = modelNode as LibraryNode;
                    if (lib != null && lib.ModelInstance == library)
                    {
                        return lib;
                    }
                }
            }

            return null;
        }

        public LibrariesNode GetLibrariesNode(GuidanceExplorerTreeView treeView)
        {
            foreach (TreeNode node in treeView.TreeView.Nodes)
            {
                LibrariesNode libsNode = node as LibrariesNode;
                if (libsNode != null)
                {
                    return libsNode;
                }
            }
            return null;
        }

    }
}

