using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.CompositeUI;

using GuidanceExplorer.Services;
using GuidanceExplorer.Model;
using GuidanceExplorer.WorkItems;
using GuidanceExplorer.Controls;

using RssToolkit;
using RssToolkit.Rss;
using System.IO;
//using RssToolkit.Web;

namespace GuidanceExplorer.Views
{
    [SmartPart]
    public partial class SuscribeToRss : UserControl
    {
        /// <summary>
        /// The main model service instance.
        /// </summary>
        private IModelStorageService _modelService;

        /// <summary>
        /// The HTML rendering service for the guidance items.
        /// </summary>
        private IHtmlRenderService _htmlRenderService;

        /// <summary>
        /// The WinForm rendering service for the guidance items.
        /// </summary>
        private IWinFormsRenderService _winFormsRenderService;

        /// <summary>
        /// The main work item instance.
        /// </summary>
        private GuidanceExplorerWorkItem _workItem;

        /// <summary>
        /// The latest loaded library.
        /// </summary>
        private Library _tempLibrary;

        /// <summary>
        /// The latest document loaded.
        /// </summary>
        private RssDocument _document;

        public SuscribeToRss()
        {
            InitializeComponent();

            listView.DisableSelectedEvent = false;
        }

        /// <summary>
        /// The main model service instance.
        /// </summary>
        [ServiceDependency]
        public IModelStorageService ModelService
        {
            set { _modelService = value; }
        }

        /// <summary>
        /// The main work item instance.
        /// </summary>
        [ServiceDependency]
        public GuidanceExplorerWorkItem WorkItem
        {
            get { return _workItem; }
            set { _workItem = value; }
        }

        /// <summary>
        /// The HTML rendering service for the guidance items.
        /// </summary>
        [ServiceDependency]
        public IHtmlRenderService HtmlRenderService
        {
            set { _htmlRenderService = value; }
        }

        /// <summary>
        /// The WinForms rendering service for the guidance items.
        /// </summary>
        [ServiceDependency]
        public IWinFormsRenderService WinFormsRenderService
        {
            set { _winFormsRenderService = value; }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            _document = RssDocument.Load(new Uri(txtURL.Text));

            cmbCategory.Items.Clear();
            cmbCategory.Items.Add("All");
            cmbCategory.SelectedIndex = 0;

            if (_document.Channel.Categories == null || _document.Channel.Categories.Count != 0)
            {
                foreach (RssCategory category in _document.Channel.Categories)
                {
                    if (!string.IsNullOrEmpty(category.Text))
                    {
                        cmbCategory.Items.Add(category.Text);
                    }
                }
            }
            else
            {
                foreach (RssItem item in _document.Channel.Items)
                {
                    foreach (RssCategory category in item.Categories)
                    {
                        if (!string.IsNullOrEmpty(category.Text) && !cmbCategory.Items.Contains(category.Text))
                        {
                            cmbCategory.Items.Add(category.Text);
                        }
                    }
                }
            }

            _tempLibrary = Library.New();
            _tempLibrary.HasChanges = true;
            _tempLibrary.Name = _document.Channel.Title;
            _tempLibrary.ReadProtection = ReadProtection.ReadWrite;
            _tempLibrary.RenderAsFile = true;
            _tempLibrary.WebSuscribed = false;
            _tempLibrary.Caption = _document.Channel.Title;
            _tempLibrary.SourceFileInfo = new FileInfo(Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, _tempLibrary.Name + ".xml")));

            foreach (RssItem item in _document.Channel.Items)
            {
                GuidanceItemFileReference guidanceItem = GuidanceItemFileReference.New();
                guidanceItem.Title = item.Title;
                guidanceItem.Id = item.Guid.Text;
                guidanceItem.GuidanceType = GuidanceType.New("technote");
                guidanceItem.Library = _tempLibrary;
                guidanceItem.DateTime = item.PubDateParsed;
                guidanceItem.CustomData["Author"] = item.Author;
                if (item.Source != null)
                {
                    guidanceItem.CustomData["Source"] = item.Source.Text;
                }

                _tempLibrary.Items.Add(guidanceItem.Id, guidanceItem);
            }

            listView.ShowContents(_tempLibrary, _tempLibrary.Columns);
        }

        private void SuscribeToRss_Load(object sender, EventArgs e)
        {
            cmbLibrary.Items.Add("<New Library>");
            foreach (Library library in _modelService.Guidance.Libraries.Values)
            {
                cmbLibrary.Items.Add(library.Caption);
            }
            cmbLibrary.SelectedIndex = 0;

            ParentForm.AcceptButton = btnOk;
            ParentForm.CancelButton = btnCancel;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            ParentForm.Close();
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            if (cmbLibrary.SelectedIndex == 0)
            {
                _modelService.Guidance.Libraries.Add(_tempLibrary.Name, _tempLibrary);

                _tempLibrary.Items.Clear();

                if (!Directory.Exists(Path.Combine(ModelStorageService.LibraryFolder, _tempLibrary.Name)))
                {
                    Directory.CreateDirectory(Path.Combine(ModelStorageService.LibraryFolder, _tempLibrary.Name));
                }

                foreach (RssItem item in _document.Channel.Items)
                {
                    GuidanceItemFileReference guidanceItem = GuidanceItemFileReference.New();
                    guidanceItem.Title = item.Title;
                    guidanceItem.Id = item.Guid.Text;
                    guidanceItem.GuidanceType = GuidanceType.New("technote");
                    guidanceItem.Library = _tempLibrary;
                    guidanceItem.DateTime = item.PubDateParsed;
                    guidanceItem.CustomData["Author"] = item.Author;
                    guidanceItem.FileName = Path.Combine(Path.Combine(ModelStorageService.LibraryFolder, _tempLibrary.Name), Path.ChangeExtension(Guid.NewGuid().ToString(), ".xml"));
                    guidanceItem.CSSFile = ModelStorageService.DefaultCSSFileName;
                    if (item.Source != null)
                    {
                        guidanceItem.CustomData["Source"] = item.Source.Text;
                    }

                    _modelService.RenderGuidanceItemToFileInfo(guidanceItem, new FileInfo(guidanceItem.FileName), item.Description);

                    _tempLibrary.Items.Add(guidanceItem.Id, guidanceItem);
                }

                _winFormsRenderService.RenderLibraryToLibrariesNode(_workItem.TreeView.TreeView, _modelService.Guidance, _tempLibrary, _winFormsRenderService.GetLibrariesNode(_workItem.TreeView));

                ParentForm.Close();
            }
        }
    }
}
