//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.CompositeUI.Collections;
using Microsoft.Practices.CompositeUI.Commands;
using Microsoft.Practices.CompositeUI.EventBroker;
using Microsoft.Practices.CompositeUI.WinForms;

using GuidanceExplorer.Views;
using GuidanceExplorer.Services;
using GuidanceExplorer.Model;
using GuidanceExplorer.Controls;

namespace GuidanceExplorer.WorkItems
{
    public partial class GuidanceExplorerWorkItem : WorkItem
    {
        public static ClientFlavour ClientFlavour = ClientFlavour.Offline;

        [Flags]
        private enum ResetOptions { Visibility = 1, Position = 2, Sort = 4 }

        #region Private members

        /// <summary>
        /// The main model service instance.
        /// </summary>
        private IModelStorageService _modelService;

        private IAsyncLoadLibrariesService _asyncLoad;
        private IWinFormsRenderService _winformRender;
        private IOnlineStorageService _onlineStorage;
        private IHtmlRenderService _htmlService;
        private GuidanceExplorerTreeView _treeView;
        private GuidanceItemListView _listView;
        private GuidanceItemDetailView _detailView;
        private ToolStripDropDownMenu _mainMenu;
        private ToolStripStatusLabel _statusLabel;
        private SearchToolbarView _searchToolbar;

        #endregion

        public GuidanceExplorerTreeView TreeView
        {
            get { return _treeView; }
        }

        protected override void OnTerminating()
        {
            base.OnTerminating();

            try
            {
                FileExit(this, null);
            }
            catch (Exception e) //Ignore errors while exiting
            {
                string s = e.Message;
            }
        }

        protected override void OnRunStarted()
        {
            base.OnRunStarted();

            _modelService = RootWorkItem.Services.Get<IModelStorageService>();
            _asyncLoad = RootWorkItem.Services.Get<IAsyncLoadLibrariesService>();
            _winformRender = RootWorkItem.Services.Get<IWinFormsRenderService>();
            _onlineStorage = RootWorkItem.Services.Get<IOnlineStorageService>();
            _htmlService = RootWorkItem.Services.Get<IHtmlRenderService>();

            _treeView = RootWorkItem.SmartParts.AddNew<GuidanceExplorerTreeView>();
            _listView = RootWorkItem.SmartParts.AddNew<GuidanceItemListView>();
            _detailView = RootWorkItem.SmartParts.AddNew<GuidanceItemDetailView>();
            _searchToolbar = RootWorkItem.SmartParts.AddNew<SearchToolbarView>();

            _modelService.LoadGuidance(Environment.CurrentDirectory, ClientFlavour);

            MenuItemAttribute[] attributes = (MenuItemAttribute[])_modelService.Guidance.GetType().GetCustomAttributes(typeof(MenuItemAttribute), false);
            List<ToolStripMenuItem> rootMenus = CreateRootMenues(attributes, null);

            UpdateMenuStrip(rootMenus);

            IWorkspace workspace = null;

            workspace = Workspaces["detailWorkspace"];
            workspace.Show(_detailView);

            workspace = Workspaces["listWorkspace"];
            workspace.Show(_listView);

            workspace = Workspaces["treeWorkspace"];
            workspace.Show(_treeView);

            workspace = Workspaces["toolbarWorkspace"];
            workspace.Show(_searchToolbar);

            //Find the progress bar in the main form.
            ToolStripProgressBar _progressBar = null;
            foreach (ToolStripProgressBar progressBar in RootWorkItem.SmartParts.FindByType<ToolStripProgressBar>())
            {
                progressBar.Minimum = 0;
                progressBar.Maximum = _modelService.Guidance.Items.Count;
                progressBar.Step = 1;
                _progressBar = progressBar;
            }

            //Find the status bar label in the main form.
            foreach (ToolStripStatusLabel statusLabel in RootWorkItem.SmartParts.FindByType<ToolStripStatusLabel>())
            {
                _statusLabel = statusLabel;
                break;
            }

            //Set the status label reference to the list view.
            _listView.StatusLabel = _statusLabel;

            BackgroundWorker bw2 = new BackgroundWorker();
            bw2.DoWork += delegate(object sender, DoWorkEventArgs eventArgs)
            {
                //After loading the synchronization takes place.
                _onlineStorage.Syncronize(_modelService, _htmlService, false, eventArgs);
            };
            bw2.RunWorkerCompleted += delegate(object sender, RunWorkerCompletedEventArgs e)
            {
                string selectedNode = GetNodePath(_treeView.TreeView.SelectedNode);
                string selectedItem = null;
                GuidanceItem[] items = _listView.GetSelectedGuidanceItems();

                if (items != null && items.Length > 0)
                {
                    selectedItem = items[0].Id;
                }

                //Repaint the guidance in the tree view control
                foreach (Library subscribedLibrary in _modelService.Guidance.Libraries.Values)
                {
                    if (subscribedLibrary.WebSuscribed)
                    {
                        //Remove the previous library node.
                        LibraryNode libNode = _winformRender.GetLibraryNodeByLibrary(_treeView, subscribedLibrary);
                        libNode.Remove();

                        //Add the new one.
                        _winformRender.RenderLibraryToLibrariesNode(_treeView.TreeView, _modelService.Guidance, subscribedLibrary, _winformRender.GetLibrariesNode(_treeView));
                    }
                }

                SelectNode(selectedNode);
                if (selectedItem != null)
                {
                    _listView.SelectItem(selectedItem);
                }
            };

            //Load all the items asynchronously and render the tree view.
            BackgroundWorker bw = new BackgroundWorker();
            bw.WorkerReportsProgress = true;
            bw.DoWork += delegate(object sender, DoWorkEventArgs eventArgs)
            {
                //Load all the items asynchronously.
                _asyncLoad.ProcessGuidance(_modelService.Guidance, bw);
            };
            bw.ProgressChanged += delegate(object sender, ProgressChangedEventArgs e)
            {
                _progressBar.PerformStep();
            };
            bw.RunWorkerCompleted += delegate(object sender, RunWorkerCompletedEventArgs e)
            {
                //Render the tree.
                _winformRender.RenderGuidanceToTreeView(_modelService.Guidance, _treeView.TreeView);

                //Finish the progress bar.
                _progressBar.PerformStep();

                //Set the text in the status label.
                _statusLabel.Text = string.Format("{0} items", _progressBar.Maximum);

                //When the loading operation has finished, remove the progress bar
                _progressBar.Visible = false;

                //Set the initial menu options (to initialize the shortcuts)
                foreach (ToolStripMenuItem menuItem in rootMenus)
                {
                    PopulateRootMenu(menuItem);
                }

                bw2.RunWorkerAsync();
            };
            bw.RunWorkerAsync();
        }

        /// <summary>
        /// Update the root main menu with the provided items.
        /// </summary>
        /// <param name="rootMenus"></param>
        private void UpdateMenuStrip(List<ToolStripMenuItem> rootMenus)
        {
            //Get the root menu.
            UIExtensionSite mainMenu = RootWorkItem.UIExtensionSites["MainMenu"];

            foreach (ToolStripMenuItem menuItem in rootMenus)
            {
                RootWorkItem.UIExtensionSites["MainMenu"].Add(menuItem);
                if (_mainMenu == null)
                {
                    _mainMenu = menuItem.DropDown.TopLevelControl as ToolStripDropDownMenu;
                }
            }
        }

        /// <summary>
        /// Create the root menus for the type provided.
        /// </summary>
        /// <param name="typeWithMenuItemAttributes"></param>
        /// <param name="modelItem"></param>
        /// <returns></returns>
        private List<ToolStripMenuItem> CreateRootMenues(MenuItemAttribute[] menuItemAttributes, IModelItem[] modelItem)
        {
            SortedDictionary<int, MenuItemAttribute> sortedList = new SortedDictionary<int, MenuItemAttribute>();

            //Obtain the attributes and add them to a new list that will be used to sort them.
            foreach (MenuItemAttribute tempAttrib in menuItemAttributes)
            {
                sortedList.Add(tempAttrib.Position, tempAttrib);
            }

            //Keep the root menus to add them later.
            List<ToolStripMenuItem> rootMenus = new List<ToolStripMenuItem>();

            //Get all the attributes of the root guidance.
            foreach (MenuItemAttribute attrib in sortedList.Values)
            {
                //Check the condition for this menu item.
                bool showMenuCondition = false;
                if (attrib.Condition != null)
                {
                    showMenuCondition = attrib.Condition.ShowMenuItem(attrib, modelItem != null ? modelItem[0] : null);
                }
                else
                {
                    showMenuCondition = true;
                }

                //If the attribute condition returns false, then ignore this menu.
                if (!showMenuCondition)
                {
                    continue;
                }

                //Split the menu item hierarchy.
                string[] menuItems = attrib.MenuItemName.Split('\\');

                ToolStripMenuItem menuItem = null;
                //Iterate through each menu item.
                for (int idx = 0; idx < menuItems.Length; idx++)
                {
                    string newMenuName = menuItems[idx];
                    if (menuItem == null)
                    {
                        //Check if there is already a root for this menu.
                        foreach (ToolStripMenuItem tempMenu in rootMenus)
                        {
                            if (tempMenu.Text == newMenuName)
                            {
                                menuItem = tempMenu;
                                break;
                            }
                        }

                        //If the root has not been created
                        if (menuItem == null)
                        {
                            //Create a new menu item.
                            menuItem = new ToolStripMenuItem(newMenuName);
                            LinkMenuItemAttribute linkAtt = attrib as LinkMenuItemAttribute;
                            if (linkAtt != null)
                            {
                                menuItem.Alignment = linkAtt.Alignment;
                                menuItem.ForeColor = linkAtt.ForeColor;
                                menuItem.Font = linkAtt.TextFont;
                            }

                            //Call the method that will populate the menu when it's opened.
                            menuItem.DropDownOpening += new EventHandler(
                                delegate(object sender, EventArgs e)
                                {
                                    PopulateRootMenu(sender as ToolStripDropDownItem);
                                }
                            );

                            //Add the menu to the root list.
                            rootMenus.Add(menuItem);
                        }

                        //Set the command to the menu.
                        if (idx == (menuItems.Length - 1) && //In the last item, create the command.
                            (!string.IsNullOrEmpty(attrib.CommandName))) //There is a command name.
                        {
                            AddCommandInvoker(attrib, menuItem);
                        }
                    }
                    else if (newMenuName == "-")
                    {
                        //If there is a separator, then create it without validating previous existence.
                        menuItem.DropDownItems.Add(new ToolStripSeparator());
                    }
                    else if (menuItem.DropDownItems[newMenuName] != null)
                    {
                        ToolStripMenuItem parent = menuItem;
                        //Check if there is already a menu with this name.
                        menuItem = menuItem.DropDownItems[newMenuName] as ToolStripMenuItem;

                        if (attrib.MenuItemsProvider == null)
                        {
                            //Add the command if corresponds.
                            if (idx == (menuItems.Length - 1) && //In the last item, create the command.
                                (!string.IsNullOrEmpty(attrib.CommandName))) //There is a command name.
                            {
                                AddCommandInvoker(attrib, menuItem);
                            }
                        }
                        else
                        {
                            //Get the child menu items
                            ProcessChildMenuItemAttribute(menuItem, attrib, modelItem);
                            if (!menuItem.HasDropDownItems)
                            {
                                parent.DropDownItems.Remove(menuItem);
                            }
                        }
                    }
                    else
                    {
                        //Create a new item.
                        if (modelItem != null && modelItem.Length > 1 && attrib.MultipleSelectionMenuItemName != null &&
                            attrib.MultipleSelectionMenuItemName.Length > 0)
                        {
                            newMenuName = attrib.MultipleSelectionMenuItemName;
                        }

                        GuidanceTypeMenuItemAttribute guidanceTypeAtt = attrib as GuidanceTypeMenuItemAttribute;
                        LinkMenuItemAttribute linkAtt = attrib as LinkMenuItemAttribute;
                        if (guidanceTypeAtt != null)
                        {
                            guidanceTypeAtt.ProcessMenuItemName(modelItem != null ? modelItem[0] : null);
                            newMenuName = guidanceTypeAtt.MenuItemName.Substring(guidanceTypeAtt.MenuItemName.LastIndexOf("\\") + 1, guidanceTypeAtt.MenuItemName.Length - guidanceTypeAtt.MenuItemName.LastIndexOf("\\") - 1);
                        }

                        ToolStripMenuItem childItem = new ToolStripMenuItem(newMenuName);
                        ToolStripMenuItem parent = menuItem;
                        childItem.Name = newMenuName;
                        childItem.ShortcutKeys = attrib.ShortCutKeys;

                        if (linkAtt != null)
                        {
                            childItem.Alignment = linkAtt.Alignment;
                            childItem.ForeColor = linkAtt.ForeColor;
                            childItem.Font = linkAtt.TextFont;
                        }

                        menuItem.DropDownItems.Add(childItem);
                        menuItem = childItem;

                        if (attrib.MenuItemsProvider == null)
                        {
                            if (idx == (menuItems.Length - 1) && //In the last item, create the command.
                                (!string.IsNullOrEmpty(attrib.CommandName))) //There is a command name.
                            {
                                AddCommandInvoker(attrib, menuItem);
                            }
                        }
                        else
                        {
                            ProcessChildMenuItemAttribute(menuItem, attrib, modelItem);
                            if (!menuItem.HasDropDownItems)
                            {
                                parent.DropDownItems.Remove(menuItem);
                            }
                            else if (guidanceTypeAtt != null)
                            {
                                parent.DropDownItems.Remove(menuItem);
                                parent.DropDownItems.AddRange(menuItem.DropDownItems);
                            }
                        }
                    }
                } //for (int idx = 0; idx < menuItems.Length; idx++)
            } //foreach (MenuItemAttribute attrib in sortedList.Values)

            return rootMenus;
        }

        /// <summary>
        /// Process the child menu items defined in the menu item attribute.
        /// </summary>
        /// <param name="parentMenuItem"></param>
        /// <param name="parentAttribute"></param>
        /// <param name="modelItem"></param>
        private void ProcessChildMenuItemAttribute(ToolStripMenuItem parentMenuItem, MenuItemAttribute parentAttribute, IModelItem[] modelItem)
        {
            IModelItem tempModelItem = (modelItem != null && modelItem.Length != 0) ? modelItem[0] : null;
            //Get the child menu items
            MenuItemAttribute[] childMenuItemAttributes = parentAttribute.MenuItemsProvider.GetSubMenuItems(parentAttribute, tempModelItem);
            foreach (MenuItemAttribute childMenuItemAttribute in childMenuItemAttributes)
            {
                //Split the menu item hierarchy.
                string[] menuItems = childMenuItemAttribute.MenuItemName.Split('\\');

                ToolStripMenuItem menuItem = null;

                //Iterate through each menu item.
                for (int idx = 0; idx < menuItems.Length; idx++)
                {
                    string newMenuName = menuItems[idx];
                    if (menuItem == null)
                    {
                        //Check if there is already a root for this menu.
                        foreach (ToolStripMenuItem tempMenu in parentMenuItem.DropDownItems)
                        {
                            if (tempMenu.Text == newMenuName)
                            {
                                menuItem = tempMenu;
                                break;
                            }
                        }

                        //If the root has not been created
                        if (menuItem == null)
                        {
                            ToolStripMenuItem childItem = new ToolStripMenuItem(newMenuName);
                            childItem.Name = newMenuName;
                            string guidanceItemTypeCommand = childMenuItemAttribute.CommandName;
                            parentMenuItem.DropDownItems.Add(childItem);
                            menuItem = childItem;

                            if (idx == (menuItems.Length - 1) && //In the last item, create the command.
                                (!string.IsNullOrEmpty(childMenuItemAttribute.CommandName))) //There is a command name.
                            {
                                AddCommandInvoker(childMenuItemAttribute, menuItem);
                                childItem.Click += delegate(object sender, EventArgs eventArgs)
                                {
                                    NewGuidanceItem(this, new GuidanceTypeEventArgs(GuidanceType.New(guidanceItemTypeCommand)));
                                };
                            }
                        }
                    }
                    else if (newMenuName == "-")
                    {
                        //If there is a separator, then create it without validating previous existence.
                        menuItem.DropDownItems.Add(new ToolStripSeparator());
                    }
                    else if (menuItem.DropDownItems[newMenuName] != null)
                    {
                        //Check if there is already a menu with this name.
                        menuItem = menuItem.DropDownItems[newMenuName] as ToolStripMenuItem;

                        //Add the command if corresponds.
                        if (idx == (menuItems.Length - 1) && //In the last item, create the command.
                            (!string.IsNullOrEmpty(childMenuItemAttribute.CommandName))) //There is a command name.
                        {
                            AddCommandInvoker(childMenuItemAttribute, menuItem);
                        }
                    }
                    else
                    {
                        //Create a new item.
                        ToolStripMenuItem childItem = new ToolStripMenuItem(newMenuName);
                        childItem.Name = newMenuName;
                        string guidanceItemTypeCommand = childMenuItemAttribute.CommandName;
                        menuItem.DropDownItems.Add(childItem);
                        menuItem = childItem;

                        if (idx == (menuItems.Length - 1) && //In the last item, create the command.
                            (!string.IsNullOrEmpty(childMenuItemAttribute.CommandName))) //There is a command name.
                        {
                            childItem.Click += delegate(object sender, EventArgs eventArgs)
                            {
                                NewGuidanceItem(this, new GuidanceTypeEventArgs(GuidanceType.New(guidanceItemTypeCommand)));
                            };
                            AddCommandInvoker(childMenuItemAttribute, menuItem);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Add a command to the workitem.
        /// </summary>
        /// <param name="attrib"></param>
        /// <param name="menuItem"></param>
        private void AddCommandInvoker(MenuItemAttribute attrib, ToolStripMenuItem menuItem)
        {
            Command command = RootWorkItem.Commands[attrib.CommandName];
            if (command == null)
            {
                command = RootWorkItem.Commands.AddNew<Command>(attrib.CommandName);
            }
            command.AddInvoker(menuItem, "Click");
        }

        /// <summary>
        /// Compute the menu items for the provided root menu item.
        /// </summary>
        /// <param name="sender"></param>
        private void PopulateRootMenu(ToolStripDropDownItem firstRootMenu)
        {
            //Add menu items for the current node selected in the view.
            TreeNode treeNode = _treeView.GetCurrentNode();
            if (treeNode == null)
            {
                return;
            }

            MenuItemAttribute[] attributes = (MenuItemAttribute[])treeNode.GetType().GetCustomAttributes(typeof(MenuItemAttribute), false);
            List<ToolStripMenuItem> treeNodeRoots = CreateRootMenues(attributes, new IModelItem[] { treeNode as IModelItem }); //TODO: check this. TreeNode should not be a modelItem

            //Update the menu.
            firstRootMenu.DropDown.Items.Clear();

            foreach (ToolStripMenuItem menuItem in treeNodeRoots)
            {
                if (menuItem.Text == firstRootMenu.Text)
                {
                    List<ToolStripItem> subMenus = new List<ToolStripItem>();
                    foreach (ToolStripItem childItem in menuItem.DropDown.Items)
                    {
                        subMenus.Add(childItem);
                    }
                    firstRootMenu.DropDown.Items.AddRange(subMenus.ToArray());
                    break;
                }
            }
        }

        public void RefreshItem(GuidanceItem item)
        {
            _listView.RefreshItems(item);
        }

        /// <summary>
        /// Return the currently selected library.
        /// </summary>
        /// <returns></returns>
        private Library GetCurrentLibrary()
        {
            Library library = null;
            TreeNode currentTreeNode = _treeView.GetCurrentNode();
            IModelItem modelItem = currentTreeNode as IModelItem;
            if (modelItem == null)
            {
                //Library can be only detected from a model item.
                return library;
            }

            LibraryNode libraryNode = modelItem as LibraryNode;
            FolderNode folderNode = modelItem as FolderNode;
            ViewNode viewNode = modelItem as ViewNode;
            GuidanceTypeNode guidanceTypeNode = modelItem as GuidanceTypeNode;
            if (libraryNode != null)
            {
                library = libraryNode.ModelInstance;
            }
            else if (folderNode != null)
            {
                library = folderNode.ModelInstance.GetContainerLibrary();
            }
            else if (viewNode != null)
            {
                library = viewNode.ModelInstance.GetContainerLibrary();
            }
            else if (guidanceTypeNode != null)
            {
                libraryNode = guidanceTypeNode.Parent as LibraryNode;
                library = libraryNode.ModelInstance;
            }
            else
            {
                //Unknown IModelItem
                throw new NotSupportedException();
            }
            return library;
        }

        public void DeleteCurrentNode()
        {
            DeleteNode(_treeView.GetCurrentNode(), true);
        }

        public void DeleteNode(TreeNode node, bool showConfirmation)
        {
            if (node is MyViewsNode)
            {
                //Do nothing
            }
            else if (node is LibrariesNode)
            {
                //Do nothing
            }
            else if (node is LibraryNode)
            {
                if (!showConfirmation || MessageBox.Show(Messages.LIBRARY_DELETION_CONFIRMATION, Messages.WARNING,
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    LibraryNodeDeleted(this, new ModelNodeEventArgs<Library>(node as LibraryNode));
                }
            }
            else if (node is FolderNode)
            {
                if (!showConfirmation || MessageBox.Show(Messages.FOLDER_DELETION_CONFIRMATION, Messages.WARNING,
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    FolderNodeDeleted(this, new ModelNodeEventArgs<Folder>(node as FolderNode));
                }
            }
            else if (node is ViewNode)
            {
                if (!showConfirmation || MessageBox.Show(Messages.VIEW_DELETION_CONFIRMATION, Messages.WARNING,
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    ViewNodeDeleted(this, new ModelNodeEventArgs<GuidanceExplorer.Model.View>(node as ViewNode));
                }
            }
            else if (node is SearchViewNode)
            {
                if (!showConfirmation || MessageBox.Show(Messages.SEARCH_VIEW_DELETION_CONFIRMATION, Messages.WARNING,
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                {
                    SearchViewNodeDeleted(this, new ModelNodeEventArgs<SearchView>(node as SearchViewNode));
                }
            }
            else if (node is AuthoringToolkitNode)
            {
                //Do nothing
            }
            else if (node is CreateViewNode)
            {
                //Do nothing
            }
            else if (node is GuidanceTypeNode)
            {
                //Do nothing
            }
            else
            {
                throw new Exception("internal error: Invalid node selected"); //TODO resources.
            }
        }

        /// <summary>
        /// Apply changes to the current columns taking the information from the default column configuration.
        /// </summary>
        /// <param name="columns"></param>
        /// <param name="defaultColumns"></param>
        private void ResetColumns(ColumnCollection columns, ColumnCollection defaultColumns, ResetOptions options)
        {
            foreach (KeyValuePair<string, Column> col in columns)
            {
                if (defaultColumns.ContainsKey(col.Key))
                {
                    if ((options & ResetOptions.Visibility) == ResetOptions.Visibility)
                    {
                        col.Value.Visible = defaultColumns[col.Key].Visible;
                    }
                    if ((options & ResetOptions.Position) == ResetOptions.Position)
                    {
                        col.Value.Position = defaultColumns[col.Key].OriginalPosition;
                    }
                    if ((options & ResetOptions.Sort) == ResetOptions.Sort)
                    {
                        col.Value.Sort = defaultColumns[col.Key].Sort;
                    }
                }
                else
                {
                    if ((options & ResetOptions.Visibility) == ResetOptions.Visibility)
                    {
                        col.Value.Visible = false;
                    }
                    if ((options & ResetOptions.Position) == ResetOptions.Position)
                    {
                        col.Value.Position = col.Value.OriginalPosition;
                    }
                    if ((options & ResetOptions.Sort) == ResetOptions.Sort)
                    {
                        col.Value.Sort = SortOrder.None;
                    }
                }
            }
        }

        /// <summary>
        /// In the current list, perform a full text search and show the contents.
        /// </summary>
        /// <param name="searchCriteria"></param>
        public bool PerformFullTextSearch(string searchCriteria)
        {
            SearchView searchView = SearchView.New();

            //Detect the current context.
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode != null)
            {
                LibraryNode libraryNode = modelNode as LibraryNode;
                FolderNode folderNode = modelNode as FolderNode;
                ViewNode viewNode = modelNode as ViewNode;
                GuidanceTypeNode typeNode = modelNode as GuidanceTypeNode;
                MyViewsNode myViewsNode = modelNode as MyViewsNode;
                SearchViewNode selectedSearchNode = modelNode as SearchViewNode;

                if (selectedSearchNode != null)
                {
                    searchView.ContainerScope = selectedSearchNode.ModelInstance;
                }
                else if (viewNode != null)
                {
                    searchView.ContainerScope = viewNode.ModelInstance;
                }
                else if (folderNode != null)
                {
                    searchView.ContainerScope = folderNode.ModelInstance;
                }
                else if (libraryNode != null)
                {
                    searchView.ContainerScope = libraryNode.ModelInstance;
                }
                else if (typeNode != null)
                {
                    LibraryNode parentLibraryNode = typeNode.Parent as LibraryNode;
                    GuidanceItemCollection guidanceItems = GuidanceItemCollection.New();
                    foreach (Library library in _modelService.Guidance.Libraries.Values)
                    {
                        if (library == parentLibraryNode.ModelInstance)
                        {
                            foreach (GuidanceItem guidanceItem in library.Items.Values)
                            {
                                GuidanceItemFileReference gifr = guidanceItem as GuidanceItemFileReference;
                                if (gifr != null && //If it is a file reference
                                    gifr.GuidanceType == typeNode.ModelInstance && //Its guidance type is the same
                                    !guidanceItems.ContainsKey(gifr.Id)) //And the item has not been added before.
                                {
                                    guidanceItems.Add(gifr.Id, gifr);
                                }
                            }
                            searchView.ContainerScope = guidanceItems;
                            break;
                        }
                    }
                }
                else if (myViewsNode != null)
                {
                    searchView.ContainerScope = myViewsNode.ModelInstance;
                }
                else
                {
                    //Unknown model node.
                    throw new NotImplementedException();
                }
            }
            else
            {
                if (treeNode is LibrariesNode)
                {
                    searchView.ContainerScope = _modelService.Guidance;
                }
                else if (treeNode is CreateViewNode)
                {
                    //Do nothing
                    return false;
                }
                else
                {
                    //Can't search in a non model node.
                    throw new NotImplementedException();
                }
            }

            if (searchView.ContainerScope == null)
            {
                //Container was not be set.
                throw new NotImplementedException();
            }

            //Set the criteria.
            searchView.Criteria = new FullTextCriteria(_modelService.Guidance, searchCriteria);

            //Set the search view title, author and dat.
            searchView.Caption = Messages.NEW_SEARCH_VIEW;
            searchView.Author = Environment.UserName;
            searchView.CreationDate = DateTime.Today;

            //Keep the search view created.
            _searchToolbar.LastSearchView = searchView;

            //Show the contents of the search view.
            _listView.ShowContents(searchView, searchView.Columns);

            return true;
        }

        /// <summary>
        /// Clear the control and the current list view.
        /// </summary>
        public void ClearFullTextSearch()
        {
            //Clean the search text box.
            _searchToolbar.FullTextSearch = string.Empty;
            _searchToolbar.LastSearchView = null;

            //Detect the current context.
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode != null)
            {
                LibraryNode libraryNode = modelNode as LibraryNode;
                FolderNode folderNode = modelNode as FolderNode;
                ViewNode viewNode = modelNode as ViewNode;
                MyViewsNode myViewsNode = modelNode as MyViewsNode;
                GuidanceTypeNode typeNode = modelNode as GuidanceTypeNode;
                SearchViewNode searchNode = modelNode as SearchViewNode;
                if (viewNode != null)
                {
                    ViewNodeSelected(this, new ModelNodeEventArgs<GuidanceExplorer.Model.View>(viewNode));
                }
                else if (folderNode != null)
                {
                    FolderNodeSelected(this, new ModelNodeEventArgs<Folder>(folderNode));
                }
                else if (libraryNode != null)
                {
                    LibraryNodeSelected(this, new ModelNodeEventArgs<Library>(libraryNode));
                }
                else if (typeNode != null)
                {
                    GuidanceTypeNodeSelected(this, new ModelNodeEventArgs<GuidanceType>(typeNode));
                }
                else if (myViewsNode != null)
                {
                    MyViewsSelected(this, new ModelNodeEventArgs<UserViews>(myViewsNode));
                }
                else if (searchNode != null)
                {
                    SearchViewNodeSelected(this, new ModelNodeEventArgs<SearchView>(searchNode));
                }
                else
                {
                    //Unknown model node.
                    throw new NotImplementedException();
                }
            }
            else if (treeNode is LibrariesNode)
            {
                LibrariesNodeSelected(this, null);
            }
            else
            {
                //Can't search in a non model node.
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// Selects the given node
        /// </summary>
        /// <param name="node"></param>
        public void SelectNode(TreeNode node)
        {
            _treeView.TreeView.SelectedNode = node;
        }

        /// <summary>
        /// Return the MyViews node
        /// </summary>
        /// <returns></returns>
        private MyViewsNode GetMyViewsNode()
        {
            foreach (TreeNode node in _treeView.TreeView.Nodes)
            {
                MyViewsNode myViews = node as MyViewsNode;
                if (myViews != null)
                {
                    return myViews;
                }
            }

            return null;
        }


        /// <summary>
        /// Gets the model node that contains the provided model instance
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="modelInstance"></param>
        /// <returns></returns>
        public ModelNode<T> GetNodeByModelInstance<T>(T modelInstance)
            where T : ILibraryNodes
        {
            if (modelInstance.ParentFolder == null)
            {
                LibraryNode libNode = _winformRender.GetLibraryNodeByLibrary(_treeView, modelInstance.GetContainerLibrary());
                MyViewsNode myViews = GetMyViewsNode();
                if (libNode != null)
                {
                    foreach (TreeNode node in libNode.Nodes)
                    {
                        IModelItem modelNode = node as IModelItem;
                        if (modelNode != null)
                        {
                            ModelNode<T> tNode = modelNode as ModelNode<T>;
                            if (tNode != null && tNode.ModelInstance.Equals(modelInstance))
                            {
                                return tNode;
                            }
                        }
                    }
                }
                else if (myViews != null)
                {
                    foreach (TreeNode node in myViews.Nodes)
                    {
                        IModelItem modelNode = node as IModelItem;
                        if (modelNode != null)
                        {
                            ModelNode<T> tNode = modelNode as ModelNode<T>;
                            if (tNode != null && tNode.ModelInstance.Equals(modelInstance))
                            {
                                return tNode;
                            }
                        }
                    }
                }

                return null;
            }
            else
            {
                ModelNode<Folder> parentNode = GetNodeByModelInstance<Folder>(modelInstance.ParentFolder);
                foreach (TreeNode node in parentNode.Nodes)
                {
                    IModelItem modelNode = node as IModelItem;
                    if (modelNode != null)
                    {
                        ModelNode<T> tNode = modelNode as ModelNode<T>;
                        if (tNode != null && tNode.ModelInstance.Equals(modelInstance))
                        {
                            return tNode;
                        }
                    }
                }

                return null;
            }
        }

        /// <summary>
        /// Save the latest search view created.
        /// </summary>
        /// <param name="p"></param>
        public void SaveFullTextSearchView(SearchView searchView)
        {
            LibrariesBrowser browser = new LibrariesBrowser();
            browser.NodeTypes = GuidanceNodeTypes.SearchViewLocation;
            browser.Guidance = _modelService.Guidance;
            browser.Text = Messages.SELECT_LOCATION;
            if (browser.ShowDialog() == DialogResult.OK)
            {
                //Create the search view node.
                SearchViewNode searchViewNode = SearchViewNode.New();
                searchViewNode.Text = searchView.Caption;

                GuidanceTypeNode typeNode = _treeView.GetCurrentNode() as GuidanceTypeNode;
                LibraryNode libraryNode = _treeView.GetCurrentNode() as LibraryNode;
                if (typeNode != null)
                {
                    List<Criteria> criterias = new List<Criteria>(3);
                    FieldSearchCriteria fieldCriteria = new FieldSearchCriteria(GuidanceType.GetAllFields()[GuidanceItem.TypeField], typeNode.ModelInstance.DisplayName, SearchTerms.Equals, _modelService.Guidance);
                    FieldSearchCriteria libraryCriteria = new FieldSearchCriteria(GuidanceType.GetAllFields()[GuidanceItem.SourceField], ((LibraryNode)typeNode.Parent).ModelInstance.Caption, SearchTerms.Equals, _modelService.Guidance);

                    criterias.Add(searchView.Criteria);
                    criterias.Add(fieldCriteria);
                    criterias.Add(libraryCriteria);

                    AndSearchCriteria andCriteria = new AndSearchCriteria(criterias);
                    searchView.Criteria = andCriteria;
                }
                else if (libraryNode != null)
                {
                    List<Criteria> criterias = new List<Criteria>(2);
                    FieldSearchCriteria libraryCriteria = new FieldSearchCriteria(GuidanceType.GetAllFields()[GuidanceItem.SourceField], libraryNode.ModelInstance.Caption, SearchTerms.Equals, _modelService.Guidance);

                    criterias.Add(searchView.Criteria);
                    criterias.Add(libraryCriteria);

                    AndSearchCriteria andCriteria = new AndSearchCriteria(criterias);
                    searchView.Criteria = andCriteria;
                }

                searchViewNode.ModelInstance = searchView;


                //Detect the current context.
                TreeNode treeNode = browser.SelectedNode;
                IModelItem modelNode = treeNode as IModelItem;
                if (modelNode != null)
                {
                    libraryNode = modelNode as LibraryNode;
                    FolderNode folderNode = modelNode as FolderNode;
                    MyViewsNode myViewsNode = modelNode as MyViewsNode;

                    if (folderNode != null)
                    {
                        folderNode = (FolderNode)GetNodeByModelInstance<Folder>(folderNode.ModelInstance);
                        //Create a unique caption
                        int i = 0;
                        while (folderNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                        {
                            searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                        }

                        searchView.ContainerScope = _modelService.Guidance;
                        searchView.ParentFolder = folderNode.ModelInstance;
                        folderNode.ModelInstance.Views.Add(searchView.Caption, searchView);
                        //It's added as a sibling to the folder.
                        folderNode.Nodes.Add(searchViewNode);

                        if (folderNode.ModelInstance.GetContainerLibrary() != null)
                        {
                            folderNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                        }
                    }
                    else if (libraryNode != null)
                    {
                        libraryNode = _winformRender.GetLibraryNodeByLibrary(_treeView, libraryNode.ModelInstance);
                        //Create a unique caption
                        int i = 0;
                        while (libraryNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                        {
                            searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                        }

                        searchView.Container = libraryNode.ModelInstance;
                        searchView.ContainerScope = _modelService.Guidance;

                        //Add as a child to the library.
                        libraryNode.ModelInstance.Views.Add(searchView.Caption, searchView);
                        libraryNode.ModelInstance.HasChanges = true;

                        //Add the search view to the tree node.
                        libraryNode.Nodes.Add(searchViewNode);
                    }
                    else if (myViewsNode != null)
                    {
                        myViewsNode = GetMyViewsNode();
                        //Create a unique caption
                        int i = 0;
                        while (myViewsNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                        {
                            searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                        }

                        searchView.ContainerScope = _modelService.Guidance;
                        searchView.Container = myViewsNode.ModelInstance;

                        //Add as a child to the library.
                        myViewsNode.ModelInstance.Views.Add(searchView.Caption, searchView);

                        //Add the search view to the tree node.
                        myViewsNode.Nodes.Add(searchViewNode);
                    }
                    else
                    {
                        //Unknown model node.
                        throw new NotImplementedException();
                    }
                }
                else
                {
                    //Can't search in a non model node.
                    throw new NotImplementedException();
                }
                _treeView.TreeView.SelectedNode = searchViewNode;
                searchViewNode.BeginEdit();
            }
        }
    }
}
