//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.CompositeUI.SmartParts;

using GuidanceExplorer.Services;
using GuidanceExplorer.WorkItems;
using Microsoft.Practices.CompositeUI.EventBroker;
using GuidanceExplorer.Controls;
using GuidanceExplorer.Model;

namespace GuidanceExplorer.Views
{
    [SmartPart]
    public partial class GuidanceExplorerTreeView : UserControl
    {
        struct DragDropOperation
        {
            public object Item;
            public object From;
            public TreeNode To;
            public int PreviousPosition;
        }

        private IModelStorageService _modelService;

        [EventPublication("event://MyViewsSelected", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<UserViews>> MyViewsSelected;

        [EventPublication("event://GlobalViewsSelected", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<GlobalViews>> GlobalViewsSelected;

        [EventPublication("event://LibrariesNodeSelected", PublicationScope.Global)]
        public event EventHandler<EventArgs> LibrariesNodeSelected;

        [EventPublication("event://LibraryNodeSelected", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<Library>> LibraryNodeSelected;

        [EventPublication("event://LibraryNodeDeleted", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<Library>> LibraryNodeDeleted;

        [EventPublication("event://ViewNodeSelected", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<GuidanceExplorer.Model.View>> ViewNodeSelected;

        [EventPublication("event://SearchViewNodeSelected", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<SearchView>> SearchViewNodeSelected;

        [EventPublication("event://ViewNodeDeleted", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<GuidanceExplorer.Model.View>> ViewNodeDeleted;

        [EventPublication("event://SearchViewNodeDeleted", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<SearchView>> SearchViewNodeDeleted;

        [EventPublication("event://FolderNodeSelected", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<Folder>> FolderNodeSelected;

        [EventPublication("event://FolderNodeDeleted", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<Folder>> FolderNodeDeleted;

        [EventPublication("event://GuidanceTypeNodeSelected", PublicationScope.Global)]
        public event EventHandler<ModelNodeEventArgs<GuidanceType>> GuidanceTypeNodeSelected;

        [EventPublication("event://AuthoringToolkitNodeSelected", PublicationScope.Global)]
        public event EventHandler<EventArgs> AuthoringToolkitNodeSelected;

        [EventPublication("event://OpeningContextMenuTreeView", PublicationScope.Global)]
        public event EventHandler<CancelEventArgs> OpeningContextMenuTreeView;

        [EventPublication("event://DropViewToLibrary", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<GuidanceExplorer.Model.View, Library>> DropViewToLibrary;

        [EventPublication("event://DropViewToUserViews", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<GuidanceExplorer.Model.View, UserViews>> DropViewToUserViews;

        [EventPublication("event://DropViewToGlobalViews", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<GuidanceExplorer.Model.View, GlobalViews>> DropViewToGlobalViews;
        
        [EventPublication("event://DropViewToFolder", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<GuidanceExplorer.Model.View, Folder>> DropViewToFolder;
        
        [EventPublication("event://DropSearchViewToLibrary", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<SearchView, Library>> DropSearchViewToLibrary;

        [EventPublication("event://DropSearchViewToUserViews", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<SearchView, UserViews>> DropSearchViewToUserViews;

        [EventPublication("event://DropSearchViewToGlobalViews", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<SearchView, GlobalViews>> DropSearchViewToGlobalViews;

        [EventPublication("event://DropSearchViewToFolder", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<SearchView, Folder>> DropSearchViewToFolder;
        
        [EventPublication("event://DropFolderToLibrary", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<Folder, Library>> DropFolderToLibrary;

        [EventPublication("event://DropFolderToUserViews", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<Folder, UserViews>> DropFolderToUserViews;

        [EventPublication("event://DropFolderToGlobalViews", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<Folder, GlobalViews>> DropFolderToGlobalViews;

        [EventPublication("event://DropFolderToFolder", PublicationScope.Global)]
        public event EventHandler<DropOperationEventArgs<Folder, Folder>> DropFolderToFolder;

        [ServiceDependency]
        public IModelStorageService ModelService
        {
            set { _modelService = value; }
        }

        private GuidanceExplorerWorkItem _workItem;

        private TreeNode _oldNode;

        private bool afterEditMessageShown = false;

        private Stack<DragDropOperation> _operations;

        private TreeNode _tempFrom;

        private TreeNode _lastSelectedNode;

        private GlobalViewsNode _globalViewsNode;

        [ServiceDependency]
        public GuidanceExplorerWorkItem WorkItem
        {
            set { _workItem = value; }
        }

        /// <summary>
        /// Public default constructor.
        /// </summary>
        public GuidanceExplorerTreeView()
        {
            InitializeComponent();
            _oldNode = null;
            _operations = new Stack<DragDropOperation>();
        }

        /// <summary>
        /// The TreeView instance for this control.
        /// </summary>
        public TreeView TreeView
        {
            get { return this.treeView; }
        }

        public bool EnableEdit
        {
            set
            {
                treeView.LabelEdit = value;
            }
        }

        void treeView_BeforeSelect(object sender, System.Windows.Forms.TreeViewCancelEventArgs e)
        {
            _lastSelectedNode = treeView.SelectedNode;
        }

        public TreeNode LastSelectedNode
        {
            get { return _lastSelectedNode; }
        }

        /// <summary>
        /// Occurs when a node is being selected.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            _tempFrom = e.Node;

            if (e.Node is MyViewsNode)
            {
                if (MyViewsSelected != null)
                {
                    MyViewsSelected(this, new ModelNodeEventArgs<UserViews>(e.Node as MyViewsNode));
                }
            }
            else if (e.Node is GlobalViewsNode)
            {
                if (GlobalViewsSelected != null)
                {
                    GlobalViewsSelected(this, new ModelNodeEventArgs<GlobalViews>(e.Node as GlobalViewsNode));
                }
            }
            else if (e.Node is LibrariesNode)
            {
                if (LibrariesNodeSelected != null)
                {
                    LibrariesNodeSelected(this, EventArgs.Empty);
                }
            }
            else if (e.Node is LibraryNode)
            {
                if (LibraryNodeSelected != null)
                {
                    LibraryNodeSelected(this, new ModelNodeEventArgs<Library>(e.Node as LibraryNode));
                }
            }
            else if (e.Node is FolderNode)
            {
                if (FolderNodeSelected != null)
                {
                    FolderNodeSelected(this, new ModelNodeEventArgs<Folder>(e.Node as FolderNode));
                }
            }
            else if (e.Node is SearchViewNode)
            {
                if (SearchViewNodeSelected != null)
                {
                    SearchViewNodeSelected(this, new ModelNodeEventArgs<SearchView>(e.Node as SearchViewNode));
                }
            }
            else if (e.Node is ViewNode)
            {
                if (ViewNodeSelected != null)
                {
                    ViewNodeSelected(this, new ModelNodeEventArgs<GuidanceExplorer.Model.View>(e.Node as ViewNode));
                }
            }
            else if (e.Node is AuthoringToolkitNode)
            {
                if (AuthoringToolkitNodeSelected != null)
                {
                    AuthoringToolkitNodeSelected(this, EventArgs.Empty);
                }
            }
            else if (e.Node is CreateViewNode)
            {
                //Do nothing
            }
            else if (e.Node is GuidanceTypeNode)
            {
                if (GuidanceTypeNodeSelected != null)
                {
                    GuidanceTypeNodeSelected(this, new ModelNodeEventArgs<GuidanceType>(e.Node as GuidanceTypeNode));
                }
            }
            else
            {
                throw new Exception("internal error: Invalid node selected"); //TODO resources.
            }
        }

        /// <summary>
        /// Occurs when the Label is about to be edited.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeView_BeforeLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            IModelItem modelNode = e.Node as IModelItem;
            //Only Model nodes are editable.
            if (modelNode != null)
            {
                if (!modelNode.IsEditable)
                {
                    //Cancel if the node is not editable.
                    e.CancelEdit = true;
                }
            }
            else
            {
                //Allow editing the create view node.
                CreateViewNode createViewNode = e.Node as CreateViewNode;
                if (createViewNode != null)
                {
                    e.CancelEdit = false;
                }
                else
                {
                    e.CancelEdit = true;
                }
            }
        }

        /// <summary>
        /// Occurs when the node has finished the edition mode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeView_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            //Seems a bug in WinForms :)
            string newLabel = e.Label ?? e.Node.Text;

            //Make sure the name isn't blank
            if (newLabel.Length == 0)
            {

                if (!afterEditMessageShown)
                {
                    MessageBox.Show(Messages.NODE_NAME_EMPTY, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                e.CancelEdit = true;
                e.Node.BeginEdit();
                afterEditMessageShown = false;
                return;
            }

            //Check if the name exists
            TreeNodeCollection parentCollection = e.Node.Parent == null ? e.Node.TreeView.Nodes : e.Node.Parent.Nodes;

            foreach (TreeNode node in parentCollection)
            {
                if (node != e.Node && node.Text == newLabel)
                {
                    if (!afterEditMessageShown)
                    {
                        MessageBox.Show(Messages.NODE_NAME_EXISTS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    e.CancelEdit = true;
                    e.Node.BeginEdit();
                    afterEditMessageShown = false;
                    return;
                }
            }

            afterEditMessageShown = false;

            IModelItem modelNode = e.Node as IModelItem;
            //Only Model nodes are editable.
            if (modelNode != null)
            {
                LibraryNode libraryNode = modelNode as LibraryNode;
                ViewNode viewNode = modelNode as ViewNode;
                SearchViewNode searchViewNode = modelNode as SearchViewNode;
                FolderNode folderNode = modelNode as FolderNode;
                if (libraryNode != null)
                {
                    libraryNode.ModelInstance.Caption = newLabel;

                    libraryNode.ModelInstance.HasChanges = true;
                    //update the store immediately for library rename
                    if (libraryNode.ModelInstance.WebSubscribed)
                    {
                        OnlineStorageService onlineStorageService = new OnlineStorageService();
                        onlineStorageService.CreateLibrary(libraryNode.ModelInstance, true);
                    }
                }
                else if (viewNode != null)
                {
                    if (viewNode.ModelInstance.Container != null)
                    {
                        viewNode.ModelInstance.Container.Views.Remove(viewNode.ModelInstance.Caption);
                        viewNode.ModelInstance.Caption = newLabel;
                        viewNode.ModelInstance.LastUpdate = DateTime.UtcNow;
                        viewNode.ModelInstance.Container.Views.Add(newLabel, viewNode.ModelInstance);
                    }
                    else
                    {
                        viewNode.ModelInstance.ParentFolder.Views.Remove(viewNode.ModelInstance.Caption);
                        viewNode.ModelInstance.Caption = newLabel;
                        viewNode.ModelInstance.LastUpdate = DateTime.UtcNow;
                        viewNode.ModelInstance.ParentFolder.Views.Add(newLabel, viewNode.ModelInstance);
                    }

                    if (viewNode.ModelInstance.GetContainerLibrary() != null)
                    {
                        viewNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                    }
                }
                else if (searchViewNode != null)
                {
                    if (searchViewNode.ModelInstance.Container != null)
                    {
                        searchViewNode.ModelInstance.Container.Views.Remove(searchViewNode.ModelInstance.Caption);
                        searchViewNode.ModelInstance.Caption = newLabel;
                        searchViewNode.ModelInstance.LastUpdate = DateTime.UtcNow;
                        searchViewNode.ModelInstance.Container.Views.Add(newLabel, searchViewNode.ModelInstance);
                    }
                    else
                    {
                        searchViewNode.ModelInstance.ParentFolder.Views.Remove(searchViewNode.ModelInstance.Caption);
                        searchViewNode.ModelInstance.Caption = newLabel;
                        searchViewNode.ModelInstance.LastUpdate = DateTime.UtcNow;
                        searchViewNode.ModelInstance.ParentFolder.Views.Add(newLabel, searchViewNode.ModelInstance);
                    }

                    if (searchViewNode.ModelInstance.GetContainerLibrary() != null)
                    {
                        searchViewNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                    }
                }
                else if (folderNode != null)
                {
                    if (folderNode.ModelInstance.Container != null)
                    {
                        folderNode.ModelInstance.Container.Folders.Remove(folderNode.ModelInstance.Caption);
                        folderNode.ModelInstance.Caption = newLabel;
                        folderNode.ModelInstance.UpdateDateTime();
                        folderNode.ModelInstance.Container.Folders.Add(newLabel, folderNode.ModelInstance);
                    }
                    else
                    {
                        folderNode.ModelInstance.ParentFolder.Folders.Remove(folderNode.ModelInstance.Caption);
                        folderNode.ModelInstance.Caption = newLabel;
                        folderNode.ModelInstance.UpdateDateTime();
                        folderNode.ModelInstance.ParentFolder.Folders.Add(newLabel, folderNode.ModelInstance);
                    }

                    if (folderNode.ModelInstance.GetContainerLibrary() != null)
                    {
                        folderNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                    }
                }
                else if (modelNode is MyViewsNode)
                {
                    //Ignore.
                }
                else
                {
                    throw new NotSupportedException();
                }
                e.CancelEdit = false;
            }
            else
            {
                //It's a not model node.
                CreateViewNode createViewNode = e.Node as CreateViewNode;
                if (createViewNode != null)
                {
                    //The Create View node can be edited, for creating new views.

                    //Update the model.
                    GuidanceExplorer.Model.View newView = GuidanceExplorer.Model.View.New();
                    newView.Caption = e.Label;
                    newView.Author = Environment.UserName;
                    newView.Container = _modelService.Guidance.UserViews;
                    newView.LastUpdate = DateTime.Today;

                    //Add the view to the UserViews.
                    _modelService.Guidance.UserViews.Views.Add(newView.Caption, newView);

                    //Update the TreeView.
                    MyViewsNode myViewsNode = createViewNode.Parent as MyViewsNode;
                    if (myViewsNode != null)
                    {
                        ViewNode viewNode = ViewNode.New();
                        viewNode.ModelInstance = newView;
                        viewNode.Text = newView.Caption;
                        myViewsNode.Nodes.Add(viewNode);

                        treeView.SelectedNode = viewNode;
                    }
                    else
                    {
                        throw new NotSupportedException();
                    }

                    //It's cancelled but a new node has been created.
                    e.CancelEdit = true;
                }
                else
                {
                    e.CancelEdit = true;
                }
            }
        }

        /// <summary>
        /// Occurs when a key is pressed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeView_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F2)
            {
                if (treeView.SelectedNode != null && treeView.LabelEdit )
                {
                    treeView.SelectedNode.BeginEdit();
                }
            }
            else if (e.KeyCode == Keys.Delete)
            {
                if (treeView.SelectedNode is MyViewsNode)
                {
                    //Do nothing
                }
                else if (treeView.SelectedNode is LibrariesNode)
                {
                    //Do nothing
                }
                else if (treeView.SelectedNode is LibraryNode)
                {
                    if (LibraryNodeDeleted != null)
                    {
                        LibraryNode libraryNode = treeView.SelectedNode as LibraryNode;

                        if (libraryNode != null)
                        {
                            if (libraryNode.ModelInstance.ReadProtection == ReadProtection.ReadOnly)
                            {
                                MessageBox.Show("Read-only Libraries can't be deleted", Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }
                            else if (MessageBox.Show(Messages.LIBRARY_DELETION_CONFIRMATION, Messages.WARNING,
                                MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                            {
                                LibraryNodeDeleted(this, new ModelNodeEventArgs<Library>(libraryNode));
                            }
                        }
                    }
                }
                else if (treeView.SelectedNode is FolderNode)
                {
                    if (FolderNodeDeleted != null)
                    {
                        FolderNode folderNode = treeView.SelectedNode as FolderNode;
                        Library containingLibrary = folderNode != null ? folderNode.ModelInstance.GetContainerLibrary() : null;

                        if (containingLibrary != null && containingLibrary.ReadProtection == ReadProtection.ReadOnly)
                        {
                            MessageBox.Show(Messages.CANNOT_DELETE_NODES_FROM_READ_ONLY_LIBRARIES, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else if (MessageBox.Show(Messages.FOLDER_DELETION_CONFIRMATION, Messages.WARNING,
                            MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                        {
                            FolderNodeDeleted(this, new ModelNodeEventArgs<Folder>(folderNode));
                        }
                    }
                }
                else if (treeView.SelectedNode is ViewNode)
                {
                    if (ViewNodeDeleted != null)
                    {
                        ViewNode viewNode = treeView.SelectedNode as ViewNode;
                        Library containingLibrary = viewNode != null ? viewNode.ModelInstance.GetContainerLibrary() : null;

                        if (containingLibrary != null && containingLibrary.ReadProtection == ReadProtection.ReadOnly)
                        {
                            MessageBox.Show(Messages.CANNOT_DELETE_NODES_FROM_READ_ONLY_LIBRARIES, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else if (MessageBox.Show(Messages.VIEW_DELETION_CONFIRMATION, Messages.WARNING,
                            MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                        {
                            ViewNodeDeleted(this, new ModelNodeEventArgs<GuidanceExplorer.Model.View>(viewNode));
                        }
                    }
                }
                else if (treeView.SelectedNode is SearchViewNode)
                {
                    if (SearchViewNodeDeleted != null)
                    {
                        SearchViewNode searchViewNode = treeView.SelectedNode as SearchViewNode;
                        
                        if (!searchViewNode.IsEditable)
                        {
                            MessageBox.Show(Messages.CANNOT_DELETE_NODES_FROM_READ_ONLY_LIBRARIES, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else if (MessageBox.Show(Messages.SEARCH_VIEW_DELETION_CONFIRMATION, Messages.WARNING,
                            MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                        {
                            SearchViewNodeDeleted(this, new ModelNodeEventArgs<SearchView>(searchViewNode));
                        }
                    }
                }
                else if (treeView.SelectedNode is AuthoringToolkitNode)
                {
                    //Do nothing
                }
                else if (treeView.SelectedNode is CreateViewNode)
                {
                    //Do nothing
                }
                else if (treeView.SelectedNode is GuidanceTypeNode)
                {
                    //Do nothing
                }
                else
                {
                    throw new Exception("internal error: Invalid node selected"); //TODO resources.
                }
            }
            else
            {
                //Unhadled key
            }
        }

        /// <summary>
        /// Occurs when the drag operation starts.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeView_ItemDrag(object sender, ItemDragEventArgs e)
        {
            IModelItem modelNode = e.Item as IModelItem;
            //Only Model nodes are dragable.
            if (modelNode != null)
            {
                ViewNode viewNode = modelNode as ViewNode;
                FolderNode folderNode = modelNode as FolderNode;
                GuidanceItem guidanceItem = modelNode as GuidanceItem;
                SearchViewNode searchView = modelNode as SearchViewNode;

                if (viewNode != null)
                {
                    treeView.DoDragDrop(viewNode, DragDropEffects.Move);
                }
                else if (folderNode != null)
                {
                    treeView.DoDragDrop(folderNode, DragDropEffects.Move);
                }
                else if (searchView != null)
                {
                    treeView.DoDragDrop(searchView, DragDropEffects.Move);
                }
                else if (modelNode is GuidanceTypeNode)
                {
                    //Do nothing.
                }
                else if (modelNode is MyViewsNode)
                {
                    //Do nothing.
                }
                else if (modelNode is LibraryNode)
                {
                    //Do nothing
                }
                else
                {
                    throw new NotSupportedException();
                }
            }
        }

        /// <summary>
        /// Called when a drag operation occurs over the tree view.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeView_DragOver(object sender, DragEventArgs e)
        {
            //Disable by default.
            e.Effect = DragDropEffects.None;

            //Get the node being draged over.
            Point point = treeView.PointToClient(new Point(e.X, e.Y));
            TreeNode hoverNode = treeView.GetNodeAt(point);

            //Check if there is a model node.
            IModelItem modelNode = hoverNode as IModelItem;
            if (modelNode != null)
            {
                if (e.Data.GetDataPresent(typeof(ViewNode)))
                {
                    ViewNode draggedViewNode = e.Data.GetData(typeof(ViewNode)) as ViewNode;
                    if (modelNode.CanDrop(draggedViewNode))
                    {
                        e.Effect = e.AllowedEffect;
                    }
                }
                else if (e.Data.GetDataPresent(typeof(FolderNode)))
                {
                    FolderNode draggedFolderNode = e.Data.GetData(typeof(FolderNode)) as FolderNode;
                    if (modelNode.CanDrop(draggedFolderNode))
                    {
                        e.Effect = e.AllowedEffect;
                    }
                }
                else if (e.Data.GetDataPresent(typeof(SearchViewNode)))
                {
                    SearchViewNode draggedSearchNode = e.Data.GetData(typeof(SearchViewNode)) as SearchViewNode;
                    if (modelNode.CanDrop(draggedSearchNode))
                    {
                        e.Effect = e.AllowedEffect;
                    }
                }
                else if (e.Data.GetDataPresent(typeof(GuidanceItem)))
                {
                    GuidanceItem draggedGuidanceItem = e.Data.GetData(typeof(GuidanceItem)) as GuidanceItem;
                    if (modelNode.CanDrop(draggedGuidanceItem))
                    {
                        e.Effect = e.AllowedEffect;
                    }
                }
                else if (e.Data.GetDataPresent(typeof(GuidanceItemCollection)))
                {
                    GuidanceItemCollection draggedGuidanceItem = e.Data.GetData(typeof(GuidanceItemCollection)) as GuidanceItemCollection;
                    if (modelNode.CanDrop(draggedGuidanceItem))
                    {
                        e.Effect = e.AllowedEffect;
                    }
                }
                else if (e.Data.GetDataPresent(typeof(GuidanceItemFileReference)))
                {
                    GuidanceItemFileReference draggedGuidanceItem = e.Data.GetData(typeof(GuidanceItemFileReference)) as GuidanceItemFileReference;
                    if (modelNode.CanDrop(draggedGuidanceItem))
                    {
                        e.Effect = e.AllowedEffect;
                    }
                }
                else
                {
                    throw new NotSupportedException();
                }
            }
            else
            {
                //Drag over a non-model node.
                MyViewsNode myViewsNode = hoverNode as MyViewsNode;
                if (myViewsNode != null)
                {
                    if (e.Data.GetDataPresent(typeof(ViewNode)))
                    {
                        ViewNode sourceViewNode = e.Data.GetData(typeof(ViewNode)) as ViewNode;
                        if (myViewsNode.CanDrop(sourceViewNode))
                        {
                            e.Effect = e.AllowedEffect;
                        }
                    }
                    else if (e.Data.GetDataPresent(typeof(FolderNode)))
                    {
                        FolderNode sourceFolderNode = e.Data.GetData(typeof(FolderNode)) as FolderNode;
                        if (myViewsNode.CanDrop(sourceFolderNode))
                        {
                            e.Effect = e.AllowedEffect;
                        }
                    }
                    else if (e.Data.GetDataPresent(typeof(SearchViewNode)))
                    {
                        SearchViewNode draggedSearchNode = e.Data.GetData(typeof(SearchViewNode)) as SearchViewNode;
                        if (modelNode.CanDrop(draggedSearchNode))
                        {
                            e.Effect = e.AllowedEffect;
                        }
                    }
                    else
                    {
                        throw new NotSupportedException();
                    }

                }
                else
                {
                    //Unknown non-model node.
                }
            }

            //Set the new and old nodes backcolor
            if ((_oldNode != null) && (_oldNode != hoverNode))
            {
                _oldNode.BackColor = SystemColors.Window;
            }

            if (e.Effect != DragDropEffects.None)
            {
                hoverNode.BackColor = SystemColors.Highlight;
                _oldNode = hoverNode;
            }
        }

        /// <summary>
        /// Occurs when the node is being drop.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeView_DragDrop(object sender, DragEventArgs e)
        {
            //Reset the node's backcolor
            _oldNode.BackColor = SystemColors.Window;

            //Get the node where the drop operation was detected.
            Point point = treeView.PointToClient(new Point(e.X, e.Y));
            TreeNode dropNode = treeView.GetNodeAt(point);
            IModelItem modelNode = dropNode as IModelItem;

            DragDropOperation op = new DragDropOperation();
            op.From = _tempFrom;
            if (modelNode != null)
            {
                ViewNode dropViewNode = dropNode as ViewNode;
                FolderNode dropFolderNode = dropNode as FolderNode;
                MyViewsNode dropMyViewsNode = dropNode as MyViewsNode;
                GlobalViewsNode dropGlobalViewsNode = dropNode as GlobalViewsNode;
                LibraryNode dropLibraryNode = dropNode as LibraryNode;

                //View node is being dropped
                if (e.Data.GetDataPresent(typeof(ViewNode)))
                {
                    ViewNode dragViewNode = e.Data.GetData(typeof(ViewNode)) as ViewNode;
                    if (modelNode.CanDrop(dragViewNode))
                    {
                        //view dropped to a folder
                        if (dropFolderNode != null)
                        {
                            if (DropViewToFolder != null)
                            {
                                op.Item = dragViewNode;
                                op.PreviousPosition = dragViewNode.Index;
                                op.From = dragViewNode.Parent;
                                op.To = dropFolderNode;
                                _operations.Push(op);

                                DropOperationEventArgs<GuidanceExplorer.Model.View, Folder> eventArgs =
                                    new DropOperationEventArgs<GuidanceExplorer.Model.View, Folder>(dragViewNode, dropFolderNode);
                                DropViewToFolder(this, eventArgs);
                            }
                        }
                        //view dropped to MyViews node
                        else if (dropMyViewsNode != null)
                        {
                            if (DropViewToUserViews != null)
                            {
                                op.Item = dragViewNode;
                                op.PreviousPosition = dragViewNode.Index;
                                op.From = dragViewNode.Parent;
                                op.To = dropMyViewsNode;
                                _operations.Push(op);

                                DropOperationEventArgs<GuidanceExplorer.Model.View, UserViews> eventArgs =
                                    new DropOperationEventArgs<GuidanceExplorer.Model.View, UserViews>(dragViewNode, dropMyViewsNode);
                                DropViewToUserViews(this, eventArgs);
                            }
                        }
                        //view dropped to GlobalViews node
                        else if (dropGlobalViewsNode != null)
                        {
                            if (DropViewToGlobalViews != null)
                            {
                                op.Item = dragViewNode;
                                op.PreviousPosition = dragViewNode.Index;
                                op.From = dragViewNode.Parent;
                                op.To = dropMyViewsNode;
                                _operations.Push(op);

                                DropOperationEventArgs<GuidanceExplorer.Model.View, GlobalViews> eventArgs =
                                    new DropOperationEventArgs<GuidanceExplorer.Model.View, GlobalViews>(dragViewNode, dropGlobalViewsNode);
                                DropViewToGlobalViews(this, eventArgs);
                            }
                        }
                        //view dropped to libraries node
                        else if (dropLibraryNode != null)
                        {
                            if (DropViewToLibrary != null)
                            {
                                op.Item = dragViewNode;
                                op.PreviousPosition = dragViewNode.Index;
                                op.From = dragViewNode.Parent;
                                op.To = dropLibraryNode;
                                _operations.Push(op);

                                DropOperationEventArgs<GuidanceExplorer.Model.View, Library> eventArgs =
                                    new DropOperationEventArgs<GuidanceExplorer.Model.View, Library>(dragViewNode, dropLibraryNode);
                                DropViewToLibrary(this, eventArgs);
                            }
                        }
                        else
                        {
                            throw new NotSupportedException();
                        }
                        return;
                    }
                    else
                    {
                        //Can't drop here.
                        throw new NotSupportedException();
                    }
                }
                //Search View node is being dropped
                else if (e.Data.GetDataPresent(typeof(SearchViewNode)))
                {
                    SearchViewNode dragSearchNode = e.Data.GetData(typeof(SearchViewNode)) as SearchViewNode;
                    if (modelNode.CanDrop(dragSearchNode))
                    {
                        //Search View dropped into a folder
                        if (dropFolderNode != null)
                        {
                            if (DropSearchViewToFolder != null)
                            {
                                op.Item = dragSearchNode;
                                op.PreviousPosition = dragSearchNode.Index;
                                op.From = dragSearchNode.Parent;
                                op.To = dropFolderNode;
                                _operations.Push(op);

                                DropOperationEventArgs<SearchView, Folder> eventArgs =
                                    new DropOperationEventArgs<SearchView, Folder>(dragSearchNode, dropFolderNode);
                                DropSearchViewToFolder(this, eventArgs);
                            }
                        }
                        //Search View dropped into My Views
                        else if (dropMyViewsNode != null)
                        {
                            if (DropSearchViewToUserViews != null)
                            {
                                op.Item = dragSearchNode;
                                op.PreviousPosition = dragSearchNode.Index;
                                op.From = dragSearchNode.Parent;
                                op.To = dropMyViewsNode;
                                _operations.Push(op);

                                DropOperationEventArgs<SearchView, UserViews> eventArgs =
                                    new DropOperationEventArgs<SearchView, UserViews>(dragSearchNode, dropMyViewsNode);
                                DropSearchViewToUserViews(this, eventArgs);
                            }
                        }
                        //Search view dropped into GlobalViews
                        else if (dropGlobalViewsNode != null)
                        {
                            if (DropSearchViewToGlobalViews != null)
                            {
                                op.Item = dragSearchNode;
                                op.PreviousPosition = dragSearchNode.Index;
                                op.From = dragSearchNode.Parent;
                                op.To = dropMyViewsNode;
                                _operations.Push(op);

                                DropOperationEventArgs<SearchView, GlobalViews> eventArgs =
                                    new DropOperationEventArgs<SearchView, GlobalViews>(dragSearchNode, dropGlobalViewsNode);
                                DropSearchViewToGlobalViews(this, eventArgs);
                            }
                        }
                        //Search view dropped into library node
                        else if (dropLibraryNode != null)
                        {
                            if (DropSearchViewToLibrary != null)
                            {
                                op.Item = dragSearchNode;
                                op.PreviousPosition = dragSearchNode.Index;
                                op.From = dragSearchNode.Parent;
                                op.To = dropLibraryNode;
                                _operations.Push(op);

                                DropOperationEventArgs<SearchView, Library> eventArgs =
                                    new DropOperationEventArgs<SearchView, Library>(dragSearchNode, dropLibraryNode);
                                DropSearchViewToLibrary(this, eventArgs);
                            }
                        }
                        else
                        {
                            throw new NotSupportedException();
                        }
                        return;
                    }
                    else
                    {
                        //Can't drop here.
                        throw new NotSupportedException();
                    }
                }
                //Folder node is being dropped
                else if (e.Data.GetDataPresent(typeof(FolderNode)))
                {
                    FolderNode dragFolderNode = e.Data.GetData(typeof(FolderNode)) as FolderNode;
                    if (modelNode.CanDrop(dragFolderNode))
                    {
                        //Folder node dropped to another folder
                        if (dropFolderNode != null)
                        {
                            if (DropFolderToFolder != null)
                            {
                                op.Item = dragFolderNode;
                                op.PreviousPosition = dragFolderNode.Index;
                                op.From = dragFolderNode.Parent;
                                op.To = dropFolderNode;
                                _operations.Push(op);

                                DropOperationEventArgs<Folder, Folder> eventArgs =
                                    new DropOperationEventArgs<Folder, Folder>(dragFolderNode, dropFolderNode);
                                DropFolderToFolder(this, eventArgs);
                            }
                        }
                        //Folder node dropped to MyViews node
                        else if (dropMyViewsNode != null)
                        {
                            if (DropFolderToUserViews != null)
                            {
                                op.Item = dragFolderNode;
                                op.PreviousPosition = dragFolderNode.Index;
                                op.From = dragFolderNode.Parent;
                                op.To = dropMyViewsNode;
                                _operations.Push(op);

                                DropOperationEventArgs<Folder, UserViews> eventArgs =
                                    new DropOperationEventArgs<Folder, UserViews>(dragFolderNode, dropMyViewsNode);
                                DropFolderToUserViews(this, eventArgs);
                            }
                        }
                        //Folder node dropped to GlobalViews node
                        else if (dropGlobalViewsNode != null)
                        {
                            if (dropGlobalViewsNode != null)
                            {
                                op.Item = dragFolderNode;
                                op.PreviousPosition = dragFolderNode.Index;
                                op.From = dragFolderNode.Parent;
                                op.To = dropMyViewsNode;
                                _operations.Push(op);

                                DropOperationEventArgs<Folder, GlobalViews> eventArgs =
                                    new DropOperationEventArgs<Folder, GlobalViews>(dragFolderNode, dropGlobalViewsNode);
                                DropFolderToGlobalViews(this, eventArgs);
                            }
                        }
                        //folder node dropped to library node
                        else if (dropLibraryNode != null)
                        {
                            if (DropFolderToLibrary != null)
                            {
                                op.Item = dragFolderNode;
                                op.PreviousPosition = dragFolderNode.Index;
                                op.From = dragFolderNode.Parent;
                                op.To = dropLibraryNode;
                                _operations.Push(op);

                                DropOperationEventArgs<Folder, Library> eventArgs =
                                    new DropOperationEventArgs<Folder, Library>(dragFolderNode, dropLibraryNode);
                                DropFolderToLibrary(this, eventArgs);
                            }
                        }
                        else
                        {
                            throw new NotSupportedException();
                        }
                        return;
                    }
                    else
                    {
                        //Can't drop here.
                        throw new NotSupportedException();
                    }
                }
                else if (e.Data.GetDataPresent(typeof(GuidanceItem)) || e.Data.GetDataPresent(typeof(GuidanceItemCollection)) || e.Data.GetDataPresent(typeof(GuidanceItemFileReference)))
                {
                    GuidanceItem draggedGuidanceItem = e.Data.GetData(typeof(GuidanceItem)) as GuidanceItem;
                    GuidanceItemFileReference draggedGuidanceItemFileReference = e.Data.GetData(typeof(GuidanceItemFileReference)) as GuidanceItemFileReference;
                    GuidanceItemCollection draggedGuidanceItemCollection = e.Data.GetData(typeof(GuidanceItemCollection)) as GuidanceItemCollection;

                    op.To = dropViewNode;
                    op.PreviousPosition = -1;

                    //Guidance item is being dropped
                    if (draggedGuidanceItem != null || draggedGuidanceItemFileReference != null)
                    {
                        if (draggedGuidanceItem == null && draggedGuidanceItemFileReference != null)
                        {
                            draggedGuidanceItem = draggedGuidanceItemFileReference;
                        }

                        if (modelNode.CanDrop(draggedGuidanceItem))
                        {
                            if (dropViewNode != null)
                            {
                                if (!dropViewNode.ModelInstance.Items.ContainsKey(draggedGuidanceItem.Id))
                                {
                                    _modelService.AddGuidanceItemToView(dropViewNode.ModelInstance, draggedGuidanceItem);
                                    GuidanceItemCollection items = GuidanceItemCollection.New();
                                    items.Add(draggedGuidanceItem.Id, draggedGuidanceItem);
                                    op.Item = items;
                                    _operations.Push(op);
                                    if (dropViewNode.ModelInstance.GetContainerLibrary() != null)
                                    {
                                        dropViewNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                                    }
                                }
                                else
                                {
                                    //Item is already in the collection.
                                }
                                return;
                            }
                            else
                            {
                                //Guidance Items can only be droped in a view.
                                throw new NotSupportedException();
                            }
                        }
                        else
                        {
                            //Can't drop here.
                        }
                    }
                    //Set of guidance items is being dropped
                    else if (draggedGuidanceItemCollection != null)
                    {
                        if (modelNode.CanDrop(draggedGuidanceItemCollection))
                        {
                            if (dropViewNode != null)
                            {
                                foreach (GuidanceItem guidanceItem in draggedGuidanceItemCollection.Values)
                                {
                                    if (!dropViewNode.ModelInstance.Items.ContainsKey(guidanceItem.Id))
                                    {
                                        _modelService.AddGuidanceItemToView(dropViewNode.ModelInstance, guidanceItem);
                                    }
                                    else
                                    {
                                        //Item is already in the collection.
                                    }
                                }

                                op.Item = draggedGuidanceItemCollection;
                                _operations.Push(op);
                                return;
                            }
                            else
                            {
                                //Guidance Items can only be droped in a view.
                                throw new NotSupportedException();
                            }
                        }
                        else
                        {
                            //Can't drop here.
                        }
                    }
                    else
                    {
                        //Unknown element.
                    }
                }
                else
                {
                    //Dragging an unknown node.
                    throw new NotImplementedException();
                }
            }
            else
            {
                //Drop into a not expected node.
                throw new NotImplementedException();
            }
        }

        /// <summary>
        /// Performs the undo drag/drop operations
        /// </summary>
        /// <returns>How many drag/drop operations are left</returns>
        public int UndoDragDrop()
        {
            if (_operations.Count > 0)
            {
                DragDropOperation op = _operations.Pop();
                GuidanceItemCollection items = op.Item as GuidanceItemCollection;
                TreeNode node = op.Item as TreeNode;

                if (items != null)
                {
                    ViewNode viewNode = (ViewNode)op.To;
                    foreach (GuidanceItem selected in items.Values)
                    {
                        if (viewNode.ModelInstance.Items.ContainsKey(selected.Id))
                        {
                            viewNode.ModelInstance.Items.Remove(selected.Id);
                        }
                    }

                    if (treeView.SelectedNode == viewNode)
                    {
                        _workItem.ViewNodeSelected(this, new ModelNodeEventArgs<GuidanceExplorer.Model.View>(viewNode));
                    }
                    else
                    {
                        treeView.SelectedNode = viewNode;
                    }
                }
                else if (node != null)
                {
                    IModelItem treeNode = op.From as IModelItem;

                    if (op.To.Nodes.Contains(node))
                    {
                        _workItem.DeleteNode(node, false);
                    }

                    if (treeNode != null)
                    {
                        FolderNode folderNodeDragged = node as FolderNode;
                        ViewNode viewNodeDragged = node as ViewNode;
                        SearchViewNode searchNodeDragged = node as SearchViewNode;


                        if (folderNodeDragged != null)
                        {
                            _workItem.AddFolder(folderNodeDragged.ModelInstance, folderNodeDragged, treeNode);
                        }
                        else if (searchNodeDragged != null)
                        {
                            _workItem.AddSearchView(searchNodeDragged, searchNodeDragged.ModelInstance, treeNode);
                        }
                        else if (viewNodeDragged != null)
                        {
                            _workItem.AddView(viewNodeDragged.ModelInstance, viewNodeDragged, treeNode);
                        }
                    }
                }
            }

            return _operations.Count;
        }

        /// <summary>
        /// Return the node currently selected.
        /// </summary>
        /// <returns></returns>
        public TreeNode GetCurrentNode()
        {
            return treeView.SelectedNode;
        }

        /// <summary>
        /// Called when the context menu is being opened
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void contextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            if (OpeningContextMenuTreeView != null)
            {
                OpeningContextMenuTreeView(sender, e);
            }
        }

        /// <summary>
        /// Return the node name when it is added to the Tree View
        /// </summary>
        /// <param name="parent">Parent node</param>
        /// <param name="name">New node name</param>
        /// <returns></returns>
        public string GetNewLibraryName(LibraryCollection libraryCollection, TreeNode parent, string name)
        {
            string folderName = name;
            int foldersCount = 0;

            if (NodeExists(parent.Nodes, folderName))
            {
                foldersCount = GetNodesCount(parent.Nodes, folderName);

                while (NodeExists(parent.Nodes, folderName))
                {
                    folderName = name + foldersCount.ToString();
                    foldersCount++;
                }
            }

            while (libraryCollection.ContainsKey(folderName))
            {
                folderName = name + foldersCount.ToString();
                foldersCount++;
            }

            return folderName;
        }

        /// <summary>
        /// Return the node name when it is added to the Tree View
        /// </summary>
        /// <param name="parent">Parent node</param>
        /// <param name="name">New node name</param>
        /// <returns></returns>
        public string GetNewNodeName(TreeNode parent, string name)
        {
            string folderName = name;
            int foldersCount = 0;

            if (NodeExists(parent.Nodes, folderName))
            {
                foldersCount = GetNodesCount(parent.Nodes, folderName);

                while (NodeExists(parent.Nodes, folderName))
                {
                    folderName = name + foldersCount.ToString();
                    foldersCount++;
                }
            }

            return folderName;
        }

        /// <summary>
        /// Access the global views node.
        /// </summary>
        /// <returns></returns>
        public GlobalViewsNode GlobalViewsNode
        {
            get
            {
                return _globalViewsNode;
            }
            set
            {
                _globalViewsNode = value;
            }
        }

        /// <summary>
        /// Returns true if there's a node with the provided name
        /// </summary>
        /// <param name="nodes"></param>
        /// <param name="folderName"></param>
        /// <returns></returns>
        private bool NodeExists(TreeNodeCollection nodes, string nodeName)
        {
            foreach (TreeNode node in nodes)
            {
                if (string.Compare(node.Text, nodeName) == 0)
                {
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Returns how many nodes with the provided name are in the node collection
        /// </summary>
        /// <param name="nodes"></param>
        /// <param name="nodeName"></param>
        /// <returns></returns>
        private int GetNodesCount(TreeNodeCollection nodes, string nodeName)
        {
            int count = 0;
            Regex regex = new Regex("(" + nodeName + "([0-9]){0,})", RegexOptions.IgnoreCase);

            foreach (TreeNode t in nodes)
            {
                if (t.Text.StartsWith(nodeName, StringComparison.CurrentCultureIgnoreCase) &&
                    regex.Match(t.Text).Success)
                {
                    count++;
                }
            }
            return count;
        }

        /// <summary>
        /// Gets/Sets whether the Tree View header is visible or not
        /// </summary>
        public bool IsHeaderVisible
        {
            set { header.Visible = value; }
            get { return header.Visible; }
        }

        private void treeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            treeView.SelectedNode = e.Node;
        }

        private void treeView_DragLeave(object sender, EventArgs e)
        {
            if (_oldNode != null)
            {
                _oldNode.BackColor = SystemColors.Window;
            }
        }

        private void treeView_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                IModelItem modelNode = e.Node as IModelItem;
                if (modelNode != null)
                {
                    SearchViewNode searchNode = modelNode as SearchViewNode;
                    ViewNode viewNode = modelNode as ViewNode;
                    FolderNode folderNode = modelNode as FolderNode;
                    LibraryNode libraryNode = modelNode as LibraryNode;
                    MyViewsNode myViewsNode = modelNode as MyViewsNode;

                    if (searchNode != null)
                    {
                        if (searchNode.IsEditable)
                        {
                            _workItem.NewSearchView(this, null);
                        }
                    }
                    else if (viewNode != null)
                    {
                        //Do nothing
                    }
                    else if (folderNode != null)
                    {
                        //Do nothing
                    }
                    else if (libraryNode != null)
                    {
                        //Do nothing
                    }
                    else if (myViewsNode != null)
                    {
                        //Do nothing
                    }
                }
            }
        }

        private void treeView_MouseDown(object sender, MouseEventArgs e)
        {
            if (treeView.GetNodeAt(e.X, e.Y) == null && treeView.SelectedNode != null)
            {
                if (treeView.SelectedNode.IsEditing)
                {
                    afterEditMessageShown = true;
                }
            }
        }
    }
}
