//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.CompositeUI.EventBroker;
using Microsoft.Practices.CompositeUI;

using GuidanceExplorer.Model;
using GuidanceExplorer.Controls;
using GuidanceExplorer.Services;
using GuidanceExplorer.WorkItems;

namespace GuidanceExplorer.Views
{
    [SmartPart]    
    public partial class SearchCriteriaControl : UserControl
    {
        /// <summary>
        /// The main work item instance.
        /// </summary>
        private GuidanceExplorerWorkItem _workItem;
        private int nextControlY;
        private int controlCount;
        private IModelStorageService _modelService;
        private IModelItem _node;

        public SearchCriteriaControl(IModelItem node)
        {
            InitializeComponent();
            _node = node;
        }

        [ServiceDependency]
        public IModelStorageService ModelService
        {
            set { _modelService = value; }
        }

        /// <summary>
        /// The main work item instance.
        /// </summary>
        [ServiceDependency]
        public GuidanceExplorerWorkItem WorkItem
        {
            set { _workItem = value; }
        }


        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            this.Dock = DockStyle.Fill;

            FieldSearchEntry criteria = new FieldSearchEntry(_modelService.Guidance);
            criteria.Add.Click += new EventHandler(Add_Click);
            criteria.Remove.Click += new EventHandler(Remove_Click);
            grpCriterias.Controls.Add(criteria);
            criteria.Location = new Point(8, 8);
            criteria.Remove.Enabled = false;
            nextControlY = criteria.Location.Y + criteria.Size.Height;
            controlCount = 1;

            cmbOperation.SelectedIndex = 0;

            SearchViewNode searchNode = _node as SearchViewNode;
            LibraryNode libraryNode = _node as LibraryNode;
            FolderNode folderNode = _node as FolderNode;

            if (searchNode != null)
            {
                CreateControlsFromSearchView(searchNode.ModelInstance);
            }
            else if (folderNode != null)
            {
                //Adds the default search criteria to filter library items
                Library library = folderNode.ModelInstance.GetContainerLibrary();
                if (library != null)
                {
                    FieldSearchCriteria libCriteria = new FieldSearchCriteria(GuidanceType.GetAllFields()[GuidanceItem.SourceField], library.Caption, SearchTerms.Equals, _modelService.Guidance);
                    AddCriteria(libCriteria);
                }
            }
            else if (libraryNode != null)
            {
                //Adds the default search criteria to filter library items
                FieldSearchCriteria libCriteria = new FieldSearchCriteria(GuidanceType.GetAllFields()[GuidanceItem.SourceField], libraryNode.ModelInstance.Caption, SearchTerms.Equals, _modelService.Guidance);
                AddCriteria(libCriteria);
            }
        }

        private void CreateControlsFromSearchView(SearchView searchView)
        {
            int controlHeight = 0;
            foreach (Control control in grpCriterias.Controls)
            {
                FieldSearchEntry c = control as FieldSearchEntry;

                if (c != null)
                {
                    nextControlY = c.Location.Y;
                    controlHeight = c.Height;
                }
            }
            AndSearchCriteria andCriteria = searchView.Criteria as AndSearchCriteria;
            OrSearchCriteria orCriteria = searchView.Criteria as OrSearchCriteria;
            FullTextCriteria fullText = searchView.Criteria as FullTextCriteria;
            Criteria[] criterias = null;

            if (fullText != null)
            {
                nextControlY += controlHeight;
                txtSearchFor.Text = fullText.Text;
                return;
            }
            else if (andCriteria != null)
            {
                cmbOperation.SelectedIndex = 0;
                criterias = andCriteria.Criterias;
            }
            else if (orCriteria != null)
            {
                cmbOperation.SelectedIndex = 1;
                criterias = orCriteria.Criterias;
            }
            else
            {
                nextControlY += controlHeight;
            }

            if (criterias != null)
            {
                grpCriterias.Controls.Clear();

                controlCount = 0;
                foreach (Criteria c in criterias)
                {
                    FullTextCriteria fullTextCriteria = c as FullTextCriteria;
                    if (fullTextCriteria != null)
                    {
                        txtSearchFor.Text = fullTextCriteria.Text;
                    }
                    else
                    {
                        AndSearchCriteria innerAndCriteria = c as AndSearchCriteria;
                        OrSearchCriteria innerOrCriteria = c as OrSearchCriteria;
                        Criteria[] innerCriterias = null;

                        if (innerAndCriteria != null)
                        {
                            cmbOperation.SelectedIndex = 0;
                            innerCriterias = innerAndCriteria.Criterias;
                        }
                        else if (innerOrCriteria != null)
                        {
                            cmbOperation.SelectedIndex = 1;
                            innerCriterias = innerOrCriteria.Criterias;
                        }

                        if (innerCriterias != null)
                        {
                            foreach (Criteria childInnerCriteria in innerCriterias)
                            {
                                AddCriteria(childInnerCriteria);
                            }
                        }
                        else
                        {
                            AddCriteria(c);
                        }
                    }
                }
            }

            if (grpCriterias.Controls.Count == 0)
            {
                FieldSearchEntry criteria = new FieldSearchEntry(_modelService.Guidance);
                criteria.Add.Click += new EventHandler(Add_Click);
                criteria.Remove.Click += new EventHandler(Remove_Click);
                grpCriterias.Controls.Add(criteria);
                criteria.Location = new Point(8, 8);
                criteria.Remove.Enabled = false;
                
                nextControlY = criteria.Location.Y + criteria.Size.Height;
                controlCount = 1;
            }
            else if (grpCriterias.Controls.Count == 1)
            {
                ((FieldSearchEntry)grpCriterias.Controls[0]).Remove.Enabled = false;
            }
        }

        private void AddCriteria(Criteria childInnerCriteria)
        {
            if (childInnerCriteria != null)
            {
                FullTextCriteria fullTextCriteria = childInnerCriteria as FullTextCriteria;
                FieldSearchCriteria fieldSearchCriteria = childInnerCriteria as FieldSearchCriteria;

                if (fullTextCriteria != null)
                {
                    txtSearchFor.Text = fullTextCriteria.Text;
                }
                else if (fieldSearchCriteria != null)
                {
                    FieldSearchEntry criteria = new FieldSearchEntry(fieldSearchCriteria, _modelService.Guidance);
                    criteria.Add.Click += new EventHandler(Add_Click);
                    criteria.Remove.Click += new EventHandler(Remove_Click);
                    criteria.Location = new Point(8, nextControlY);
                    grpCriterias.Controls.Add(criteria);
                    nextControlY = criteria.Location.Y + criteria.Size.Height;
                    controlCount++;

                    if (controlCount > 1)
                    {
                        this.Parent.Height += criteria.Height;
                    }
                }
            }
        }

        void Add_Click(object sender, EventArgs e)
        {
            if (controlCount < 6)
            {
                FieldSearchEntry criteria = new FieldSearchEntry(_modelService.Guidance);
                criteria.Add.Click += new EventHandler(Add_Click);
                criteria.Remove.Click += new EventHandler(Remove_Click);
                criteria.Location = new Point(8, nextControlY);
                grpCriterias.Controls.Add(criteria);
                nextControlY = criteria.Location.Y + criteria.Size.Height;
                controlCount++;


                if (controlCount == 2)
                {
                    foreach (Control control in grpCriterias.Controls)
                    {
                        FieldSearchEntry c = control as FieldSearchEntry;

                        if (c != null)
                        {
                            c.Remove.Enabled = true;
                        }
                    }
                }

                this.Parent.Height += criteria.Height;
            }
        }

        void Remove_Click(object sender, EventArgs e)
        {
            FieldSearchEntry removedControl = (FieldSearchEntry)((ButtonBase)sender).Parent;

            controlCount--;
            nextControlY = 0;
            this.Parent.Height -= removedControl.Height;
            foreach (Control control in grpCriterias.Controls)
            {
                FieldSearchEntry criteria = control as FieldSearchEntry;

                if (criteria != null)
                {
                    if (controlCount == 1)
                    {
                        criteria.Remove.Enabled = false;
                    }

                    if ((criteria.Location.Y + criteria.Size.Height) > nextControlY)
                    {
                        nextControlY = criteria.Location.Y;
                    }

                    if (criteria.Location.Y > removedControl.Location.Y)
                    {
                        criteria.Location = new Point(criteria.Location.X, criteria.Location.Y - removedControl.Size.Height);
                    }
                }
            }

            grpCriterias.Controls.Remove(removedControl);
        }

        /// <summary>
        /// Gets the search criteria represented by the control
        /// </summary>
        public Criteria Criteria
        {
            get
            {
                List<Criteria> criterias = new List<Criteria>(grpCriterias.Controls.Count);
                foreach (Control control in grpCriterias.Controls)
                {
                    FieldSearchEntry criteria = control as FieldSearchEntry;

                    if (criteria != null && criteria.Criteria != null)
                    {
                        criterias.Add(criteria.Criteria);
                    }
                }

                Criteria retCriteria = null;

                if (criterias.Count > 0)
                {
                    if (cmbOperation.SelectedIndex == 0)
                    {
                        retCriteria = new AndSearchCriteria(criterias);
                    }
                    else
                    {
                        retCriteria = new OrSearchCriteria(criterias);
                    }
                }

                if (txtSearchFor.Text != null && txtSearchFor.Text.Length > 0)
                {
                    if (retCriteria == null)
                    {
                        return new FullTextCriteria(_modelService.Guidance, txtSearchFor.Text);
                    }
                    else
                    {
                        List<Criteria> andCrit = new List<Criteria>(2);
                        andCrit.Add(new FullTextCriteria(_modelService.Guidance, txtSearchFor.Text));
                        andCrit.Add(retCriteria);
                        return new AndSearchCriteria(andCrit);
                    }
                }
                else
                {
                    return retCriteria;
                }
            }
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            SearchViewNode searchViewNode = _node as SearchViewNode;
            SearchView searchView = null;

            if (searchViewNode == null)
            {
                searchView = SearchView.New();
                searchView.Criteria = Criteria;
                searchView.Caption = Messages.NEW_SEARCH_VIEW;
                searchView.Author = Environment.UserName;
                searchView.LastUpdate = DateTime.UtcNow;
                searchViewNode = SearchViewNode.New();
                searchViewNode.ModelInstance = searchView;
            }
            else
            {
                searchViewNode.ModelInstance.Criteria = Criteria;
                searchViewNode.ModelInstance.LastUpdate = DateTime.UtcNow;
                Library library = searchViewNode.ModelInstance.GetContainerLibrary();
                if (library != null)
                {
                    library.HasChanges = true;
                }
            }

            _workItem.AddSearchView(searchViewNode, searchView, _node);

            if (ParentForm != null)
            {
                ParentForm.Close();
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (ParentForm != null)
            {
                ParentForm.Close();
            }
        }

        private void SearchCriteriaControl_ParentChanged(object sender, EventArgs e)
        {
            if (ParentForm != null)
            {
                ParentForm.FormBorderStyle = FormBorderStyle.FixedDialog;
            }
        }
    }
}
