//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;

using GuidanceExplorer.Services;

namespace GuidanceExplorer.Model
{
    public class FullTextCriteria : Criteria
    {
        /// <summary>
        /// The text that will be searched.
        /// </summary>
        private string _text;

        /// <summary>
        /// The guidance model reference.
        /// </summary>
        private Guidance _guidance;

        /// <summary>
        /// The RegularExpression used to evaluate the Guidance Items
        /// </summary>
        Regex _regEx;

        /// <summary>
        /// Creates a new full text search.
        /// </summary>
        /// <param name="text"></param>
        public FullTextCriteria(Guidance guidance, string text)
        {
            _guidance = guidance;
            _text = text;
            MakeRegularExpression();
        }

        /// <summary>
        /// Evaluate the criteria in the item.
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public override bool Evaluate(GuidanceItem item)
        {
            GuidanceItemFileReference gifr = item as GuidanceItemFileReference;
            if (null == gifr)
            {
                gifr = _guidance.GetGuidanceItemFileReference(item);
            }

            if (null == gifr)
            {
                return false;
            }
            
            if (null != gifr.Title && 
                _regEx.Match(gifr.Title).Success)
            {
                return true;
            }

            foreach (KeyValuePair<String, String> pair in gifr.CustomData)
            {
                if (null != pair.Value && _regEx.Match(pair.Value).Success)
                {
                    return true;
                }
            }

            HtmlRenderService html = new HtmlRenderService();
            String content = html.GetDocumentText(gifr);

            if (null != content && _regEx.Match(content).Success)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// The text that will be searched.
        /// </summary>
        public string Text
        {
            get { return _text; }
            set { _text = value; }
        }

        /// <summary>
        /// Creates the regular expression
        /// </summary>
        private void MakeRegularExpression()
        {
            StringBuilder regExpressionPattern = new StringBuilder("\\b");
            string pattern = (string)_text.Clone();
            string[] searchValues;
            bool searchQuotes = pattern.Contains("\"");

            if (searchQuotes)
            {
                //this means the search is in quotes so we search on the phrase
                searchValues = pattern.Trim().Split(new string[] { "\"" }, StringSplitOptions.RemoveEmptyEntries);
            }
            else
            {
                // this means the search is not in quotes so we search each word seperately
                searchValues = pattern.Trim().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
            }

            foreach (string val in searchValues)
            {
                regExpressionPattern.Append(val.Trim() + "\\b.*\\b");
            }

            regExpressionPattern.Remove(regExpressionPattern.Length - 5, 4);

            _regEx = new Regex(regExpressionPattern.ToString(), RegexOptions.IgnoreCase | RegexOptions.Compiled | RegexOptions.Singleline);
        }
    }
}

