//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using GuidanceExplorer.Model;
using Microsoft.Practices.CompositeUI;
using System.IO;
using System.Xml;
using GuidanceExplorer.Schemas;
using System.Reflection;
using Microsoft.Security.Application;

namespace GuidanceExplorer.Services
{
    [Service(typeof(IHtmlRenderService))]
    public class HtmlRenderService : IHtmlRenderService
    {
        #region IBuildHtmlForGuidanceItem Members

        public String GetDocumentText(GuidanceItemFileReference guidanceItem)
        {
            Stream stream = GetReadOnlyDocumentStream(guidanceItem);
            string content = null;

            if (null != stream)
            {
                using (stream)
                using (StreamReader reader = new StreamReader(stream))
                {
                    content = reader.ReadToEnd();

                    if (null != content)
                    {
                        content = System.Text.RegularExpressions.Regex.Replace(content, "<(/?[^\\>]+)>", "");
                        content = System.Web.HttpUtility.HtmlDecode(content);
                        content = content.Replace("\n", " ");
                        content = content.Replace("\r", " ");
                    }
                }
            }

            return content;
        }

        /// <summary>
        /// Returns the document HTML content as string.
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to convert</param>
        /// <returns></returns>
        public Stream GetReadOnlyDocumentStream(GuidanceItemFileReference guidanceItem)
        {
            return GetDocumentStream(guidanceItem, ReadProtection.ReadOnly, false, false);
        }

        /// <summary>
        /// Returns the document HTML content as string for showing or exporting
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to convert</param>
        /// <returns></returns>
        public Stream GetDocumentStreamForShowing(GuidanceItemFileReference guidanceItem)
        {
            return GetDocumentStream(guidanceItem, ReadProtection.ReadOnly, true, true);
        }

        /// <summary>
        /// Returns the raw HTML content for the document 
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to process</param>
        /// <returns></returns>
        public Stream GetRawDocumentStream(GuidanceItemFileReference guidanceItem)
        {
            MemoryStream ms = new MemoryStream();
            StreamWriter sw = new StreamWriter(ms);
            sw.Write(GetDocumentText(guidanceItem.FileName, guidanceItem));
            sw.Flush();
            ms.Position = 0;
            return ms;
        }

        /// <summary>
        /// Returns the document HTML content as string.
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to convert</param>
        /// <returns></returns>
        public Stream GetEditableDocumentStream(GuidanceItemFileReference guidanceItem)
        {
            return GetDocumentStream(guidanceItem, ReadProtection.ReadWrite, false, true);
        }

        /// <summary>
        /// Return the editable schema for the guidance item.
        /// </summary>
        /// <param name="guidanceItemFileReference"></param>
        /// <returns></returns>
        public Stream GetSchemaStream(GuidanceItemFileReference guidanceItemFileReference)
        {
            string schemaFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItemFileReference.GuidanceType.SchemaFileName));
            if (File.Exists(schemaFileName))
            {
                return GetDocumentStream(schemaFileName, ReadProtection.ReadWrite, guidanceItemFileReference, false, true);
            }
            return null;
        }

        /// <summary>
        /// Return the template for the guidance item.
        /// </summary>
        /// <param name="guidanceItemFileReference"></param>
        /// <returns></returns>
        public Stream GetReadOnlyTemplateStream(GuidanceItemFileReference guidanceItemFileReference)
        {
            if (!string.IsNullOrEmpty(guidanceItemFileReference.GuidanceType.TemplateFileName))
            {
                string templateFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItemFileReference.GuidanceType.TemplateFileName));
                if (File.Exists(templateFileName))
                {
                    return GetDocumentStream(templateFileName, ReadProtection.ReadOnly, guidanceItemFileReference, false, true);
                }
            }
            return null;
        }

        /// <summary>
        /// Return the example for the guidance item.
        /// </summary>
        /// <param name="guidanceItemFileReference"></param>
        /// <returns></returns>
        public Stream GetReadOnlyExampleStream(GuidanceItemFileReference guidanceItemFileReference)
        {
            if (!string.IsNullOrEmpty(guidanceItemFileReference.GuidanceType.ExampleFileName))
            {
                string exampleFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItemFileReference.GuidanceType.ExampleFileName));
                if (File.Exists(exampleFileName))
                {
                    return GetDocumentStream(exampleFileName, ReadProtection.ReadOnly, guidanceItemFileReference, false, true);
                }
            }
            return null;
        }

        #endregion

        /// <summary>
        /// Adapts between Model-oriented method and file-oriented method.
        /// </summary>
        /// <param name="guidanceItem"></param>
        /// <param name="readOnly"></param>
        /// <param name="readProtection"></param>
        /// <param name="showData">If set to true then show attribute values in the document. True assumes call from details view so it will show missing file text if the file doesn't exist. If this is set to false then don't show attributes. False assumes call from editor so it will show template if the file doesn't exist.</param>
        /// <returns></returns>
        private Stream GetDocumentStream(GuidanceItemFileReference guidanceItem, ReadProtection readProtection, bool showAttributes, bool showStyleSheet)
        {
            string missingFileName = string.Empty;
            Library library = guidanceItem.Library;
            string guidanceItemFileName = string.Empty;

            if (library == null)
            {
                guidanceItemFileName = guidanceItem.FileName;
            }
            else if (library.FileSubscribed)
            {
                guidanceItemFileName = Path.GetFullPath(Path.Combine(library.OriginalSourceFileInfo.DirectoryName, guidanceItem.FileName));
            }
            else
            {
                guidanceItemFileName = Path.GetFullPath(Path.Combine(library.SourceFileInfo.DirectoryName, guidanceItem.FileName));
            }
            
            //get the missing file or schema text
            if ((guidanceItem.GuidanceType != null) && (!string.IsNullOrEmpty(guidanceItem.GuidanceType.MissingFileName)))
            {
                if (showAttributes)
                {
                    missingFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItem.GuidanceType.MissingFileName));
                }
                else
                {
                    missingFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItem.GuidanceType.SchemaFileName));
                }
            }

            //check to see if the file exists first, if not then use the missing file text
            if (File.Exists(guidanceItemFileName))
            {
                return GetDocumentStream(guidanceItemFileName, readProtection, guidanceItem, showAttributes, showStyleSheet);
            }
            else if (!string.IsNullOrEmpty(missingFileName) && File.Exists(missingFileName))
            {
                return GetDocumentStream(missingFileName, readProtection, guidanceItem, showAttributes, showStyleSheet);
            }

            return null;
        }


        /// <summary>
        /// Build path to the css file used for saving items to word 
        /// </summary>
        /// <param name="guidanceItem"></param>
        /// <param name="file"></param>
        public string GetSaveAsWordCSS(string CSSFile)
        {
            string fullCssPath = Path.GetDirectoryName(CSSFile) + "\\" + Path.GetFileNameWithoutExtension(CSSFile) + "SaveAsWord" + Path.GetExtension(CSSFile);
            if (!File.Exists(fullCssPath))
            {
                fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, Path.GetFileName(fullCssPath)));
                if (!File.Exists(fullCssPath))
                {
                    fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName));
                }
            }

            return fullCssPath;
        }

        /// <summary>
        /// Build path to the css file used for saving items to html
        /// </summary>
        /// <param name="guidanceItem"></param>
        /// <param name="file"></param>
        public string GetCSS(string CSSFile)
        {
            string fullCssPath = CSSFile;
            if (!File.Exists(fullCssPath))
            {
                fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, Path.GetFileName(fullCssPath)));
                if (!File.Exists(fullCssPath))
                {
                    fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName));
                }
            }

            return fullCssPath;
        }

        /// <summary>
        /// Exports the provided Guidance Item as Html
        /// </summary>
        /// <param name="guidanceItem"></param>
        /// <param name="file"></param>
        public void RenderGuidanceItemToHtml(GuidanceItemFileReference guidanceItem, FileInfo file, bool wordDoc)
        {
            string fullCssPath;

            if (wordDoc)
            {
                fullCssPath = GetSaveAsWordCSS(guidanceItem.CSSFile);
            }
            else
            {
                fullCssPath = GetCSS(guidanceItem.CSSFile);
            }

            using (FileStream stream = new FileStream(file.FullName, FileMode.Create, FileAccess.ReadWrite))
            using (StreamWriter streamWriter = new StreamWriter(stream, Encoding.UTF8))
            {
                streamWriter.WriteLine("<html>");
                streamWriter.WriteLine("<head>");
                streamWriter.WriteLine("<style type=\"text/css\">");
                streamWriter.WriteLine(File.ReadAllText(fullCssPath));
                streamWriter.WriteLine("</style>");
                streamWriter.WriteLine("</head>");
                WriteDocumentBody(guidanceItem.FileName, ReadProtection.ReadOnly, guidanceItem, true, streamWriter, file.DirectoryName);
                streamWriter.WriteLine("</html>");

                streamWriter.Flush();
            }
        }

        /// <summary>
        /// Creates the HTML in a stream for later reading.
        /// </summary>
        /// <param name="guidanceItemFileName"></param>
        /// <param name="guidanceItemCssFileName"></param>
        /// <param name="readProtection"></param>
        /// <param name="guidanceItem"></param>
        /// <param name="showData"></param>
        /// <returns></returns>
        private Stream GetDocumentStream(string guidanceItemFileName, ReadProtection readProtection, GuidanceItemFileReference guidanceItem, bool showAttributes, bool showStyleSheet)
        {
            string hrefCssFormat = string.Empty;
            if (File.Exists(guidanceItem.CSSFile))
            {
                string fullCssPath = guidanceItem.CSSFile;
                hrefCssFormat = fullCssPath.Replace('\\', '/');
            }
            else
            {
                string fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItem.CSSFile));
                if (File.Exists(fullCssPath))
                {
                    hrefCssFormat = fullCssPath.Replace('\\', '/');
                }
                else
                {
                    fullCssPath = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName));
                    hrefCssFormat = fullCssPath.Replace('\\', '/');
                }
            }

            return WriteDocumentStream(guidanceItemFileName, readProtection, guidanceItem, showAttributes, showStyleSheet, hrefCssFormat, null);
        }

        private Stream WriteDocumentStream(string guidanceItemFileName, ReadProtection readProtection, GuidanceItemFileReference guidanceItem, bool showAttributes, bool showStyleSheet, string hrefCssFormat, string imgDirectory)
        {
            MemoryStream memoryStream = new MemoryStream();
            StreamWriter streamWriter = new StreamWriter(memoryStream, Encoding.UTF32);

            if (showStyleSheet)
            {
                streamWriter.WriteLine("<html>");

                if (File.Exists(hrefCssFormat))
                {
                    streamWriter.WriteLine("<style type=\"text/css\">");
                    streamWriter.WriteLine(File.ReadAllText(hrefCssFormat));
                    streamWriter.WriteLine("</style>");
                }
                else
                {
                    streamWriter.WriteLine(string.Format("<link href=\"{0}\" rel=\"stylesheet\" type=\"text/css\" />", guidanceItem.CSSFile));
                }
            }

            WriteDocumentBody(guidanceItemFileName, readProtection, guidanceItem, showAttributes, streamWriter, imgDirectory);

            if (showStyleSheet)
            {
                streamWriter.WriteLine("</html>");
            }

            streamWriter.Flush();
            memoryStream.Position = 0;

            return memoryStream;
        }

        private void WriteDocumentBody(string guidanceItemFileName, ReadProtection readProtection, GuidanceItemFileReference guidanceItem, bool showAttributes, StreamWriter streamWriter, string imgDirectory)
        {
            if (readProtection != ReadProtection.ReadOnly)
            {
                streamWriter.WriteLine("<script>");
                using (StreamReader innerCode =
                    new StreamReader(Assembly.GetExecutingAssembly().GetManifestResourceStream("GuidanceExplorer.innerCode.js")))
                {
                    streamWriter.Write(innerCode.ReadToEnd());
                }
                streamWriter.WriteLine("</script>");
                streamWriter.WriteLine("<body id=\"bodyId\" contentEditable=\"true\" onselectstart=\"handleEvent();\" onkeyup=\"handleEvent();\" onmouseup=\"handleEvent();\" ondblclick=\"handleEvent();\" onblur=\"handleEvent();\" onfocus=\"handleEvent();\">");
            }

            if (showAttributes)
            {
                string link = guidanceItem.Title;//Links lenght is up to 40 chars
                if (link.Length > 40)
                {
                    link = link.Substring(0, 39);
                }
                streamWriter.WriteLine("<h1 class=\"title\"><a name='" + link.Replace(" ", "_") + "'>" + CleanHTML(guidanceItem.Title) + "</a></h1>\n");
                if (guidanceItem.CustomData.ContainsKey("Author"))
                {
                    streamWriter.WriteLine("<i>" + CleanHTML(guidanceItem.CustomData["Author"]) + "</i><br><br>\n");
                }
            }

            if (File.Exists(guidanceItemFileName))
            {
                //Places the HTML for the document.
                string text = GetDocumentText(guidanceItemFileName, guidanceItem);

                while (text.Contains("|image|"))
                {
                    string oldPath = text.Substring(text.IndexOf("|image|") + 7, text.IndexOf("\"", text.IndexOf("|image|") + 1) - (text.IndexOf("|image|") + 7));
                    string newPath = guidanceItem.GenerateImage(oldPath, imgDirectory, "imgs");
                    guidanceItem.ImagesMapping[oldPath] = newPath;
                    text = text.Replace("|image|" + oldPath, newPath);
                }

                if (showAttributes)
                {
                    streamWriter.WriteLine(CleanHTML(text));
                }
                else
                {
                    streamWriter.WriteLine(text);
                }
            }

            //Show attributes
            //if (showAttributes)
            //{
            //    streamWriter.WriteLine("<h1>Attributes</h1>\n");
            //    foreach (string att in guidanceItem.CustomData.Keys)
            //    {
            //        streamWriter.WriteLine("<li>" + guidanceItem.GuidanceType.Fields.GetDisplayName(att) + ": " + CleanHTML(guidanceItem[att]) + "</li><br>");
            //    }
            //    streamWriter.WriteLine("<li>ID: " + CleanHTML(guidanceItem.Id) + "</li><br>");
            //}
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="encodedTag"></param>
        /// <param name="decodedTag"></param>
        /// <param name="endTag"></param>
        /// <returns></returns>
        private String CleanTag(string encodedString, string tag, bool stripInnerContent)
        {
            StringBuilder cleanedString = new StringBuilder(encodedString);
            string encodedTag = "&#60;" + tag + "&#62;";
            string decodedTag = "<" + tag + ">";
            string encodedOpenTag = "&#60;" + tag;
            string decodedOpenTag = "<" + tag;
            string encodedEndTag = "&#60;&#47;" + tag + "&#62;";
            string decodedEndTag = "</" + tag + ">";

            //take care of tag if it has no inner content
            cleanedString = cleanedString.Replace(encodedTag, decodedTag);
            cleanedString = cleanedString.Replace(encodedTag.ToLower(), decodedTag.ToLower());

            //take care of the end tag if it exists
            cleanedString = cleanedString.Replace(encodedEndTag, decodedEndTag);
            cleanedString = cleanedString.Replace(encodedEndTag.ToLower(), decodedEndTag.ToLower());

            //if a tag has inner content then strip it out
            int openTagIndex = cleanedString.ToString().ToLower().IndexOf(encodedOpenTag.ToLower());
            while (openTagIndex != -1)
            {
                int endTagIndex = cleanedString.ToString().IndexOf("&#62;", openTagIndex);
                if (endTagIndex > openTagIndex)
                {
                    if (stripInnerContent)
                    {
                        //remove everything but the simple tag
                        cleanedString.Replace(cleanedString.ToString().Substring(openTagIndex, endTagIndex - openTagIndex + 5), decodedTag);
                    }
                    else
                    {
                        //replace the encoded tag with a decoded version - including inner content
                        string encodedFullTag = cleanedString.ToString().Substring(openTagIndex, endTagIndex - openTagIndex + 5);
                        string decodedFullTag = "<" + encodedFullTag.Substring(5, encodedFullTag.Length - 10) + ">";
                        cleanedString.Replace(encodedFullTag, decodedFullTag);
                    }
                }
                else
                {
                    break;
                }
                openTagIndex = cleanedString.ToString().ToLower().IndexOf(encodedOpenTag.ToLower());
            }

            return cleanedString.ToString();
        }

        private string RemoveTag(string encodedString, string tag)
        {
            StringBuilder cleanedString = new StringBuilder(encodedString);
            String encodedOpenTag = "&#60;" + tag;
            string encodedEndTag = "&#60;&#47;" + tag;

            int openTagIndex = cleanedString.ToString().ToLower().IndexOf(encodedOpenTag.ToLower());
            while (openTagIndex != -1)
            {
                int endTagIndex = cleanedString.ToString().IndexOf("&#62;", openTagIndex);
                if (endTagIndex > openTagIndex)
                {
                    cleanedString.Replace(cleanedString.ToString().Substring(openTagIndex, endTagIndex - openTagIndex + 5), "");
                }
                else
                {
                    break;
                }
                openTagIndex = cleanedString.ToString().ToLower().IndexOf(encodedOpenTag.ToLower());
            }

            //take care of the end tag if it exists
            openTagIndex = cleanedString.ToString().ToLower().IndexOf(encodedEndTag.ToLower());
            while (openTagIndex != -1)
            {
                int endTagIndex = cleanedString.ToString().IndexOf("&#62;", openTagIndex);
                if (endTagIndex > openTagIndex)
                {
                    cleanedString.Replace(cleanedString.ToString().Substring(openTagIndex, endTagIndex - openTagIndex + 5), "");
                }
                else
                {
                    break;
                }
                openTagIndex = cleanedString.ToString().ToLower().IndexOf(encodedEndTag.ToLower());
            }

            return cleanedString.ToString();
        }

        private string ReplaceImageTags(string encodedString, string tag)
        {
            StringBuilder cleanedString = new StringBuilder(encodedString);
            String openTag = "<" + tag;
            string endTag = "</" + tag;

            int openTagIndex = cleanedString.ToString().ToLower().IndexOf(openTag.ToLower());
            while (openTagIndex != -1)
            {
                int srcTagStartIndex = cleanedString.ToString().ToLower().IndexOf("src=\"", openTagIndex);
                int srcTagEndIndex = cleanedString.ToString().IndexOf("\"", srcTagStartIndex + 5);
                String imgSrc = cleanedString.ToString().Substring(srcTagStartIndex, srcTagEndIndex - srcTagStartIndex + 1);
                int endTagIndex = cleanedString.ToString().IndexOf(">", openTagIndex);
                if (endTagIndex > openTagIndex)
                {
                    cleanedString.Replace(cleanedString.ToString().Substring(openTagIndex, endTagIndex - openTagIndex + 1), "<img " + imgSrc + ">");
                }
                else
                {
                    break;
                }
                openTagIndex = cleanedString.ToString().ToLower().IndexOf(openTag.ToLower());
            }

            //take care of the end tag if it exists
            openTagIndex = cleanedString.ToString().ToLower().IndexOf(endTag.ToLower());
            while (openTagIndex != -1)
            {
                int endTagIndex = cleanedString.ToString().IndexOf(">", openTagIndex);
                if (endTagIndex > openTagIndex)
                {
                    cleanedString.Replace(cleanedString.ToString().Substring(openTagIndex, endTagIndex - openTagIndex + 1), "");
                }
                else
                {
                    break;
                }
                openTagIndex = cleanedString.ToString().ToLower().IndexOf(endTag.ToLower());
            }

            return cleanedString.ToString();
        }

        /// <summary>
        /// Performs the security encoding
        /// </summary>
        /// <param name="encodedString"></param>
        /// <returns></returns>
        string SecureClean(string encodedString)
        {
            encodedString = AntiXss.HtmlEncode(encodedString);

            //replace each good tag with non-encoded equivelant
            encodedString = CleanTag(encodedString, "H1", true);
            encodedString = CleanTag(encodedString, "H2", true);
            encodedString = CleanTag(encodedString, "H3", true);
            encodedString = CleanTag(encodedString, "H4", true);
            encodedString = CleanTag(encodedString, "H5", true);
            encodedString = CleanTag(encodedString, "PRE", true);
            encodedString = CleanTag(encodedString, "P", true);
            encodedString = CleanTag(encodedString, "UL", true);
            encodedString = CleanTag(encodedString, "HR", true);
            encodedString = CleanTag(encodedString, "DIV", true);
            encodedString = CleanTag(encodedString, "SPAN", true);
            encodedString = CleanTag(encodedString, "TABLE", true);
            encodedString = CleanTag(encodedString, "TR", true);
            encodedString = CleanTag(encodedString, "TD", true);
            encodedString = CleanTag(encodedString, "TBODY", true);
            encodedString = CleanTag(encodedString, "TH", true);
            encodedString = CleanTag(encodedString, "LI", true);
            encodedString = CleanTag(encodedString, "STRONG", true);
            encodedString = CleanTag(encodedString, "BR", true);
            encodedString = CleanTag(encodedString, "OL", true);
            encodedString = CleanTag(encodedString, "BLOCKQUOTE", true);
            encodedString = CleanTag(encodedString, "EM", true);

            //tags that have inner content that must remain
            encodedString = CleanTag(encodedString, "IMG", false);
            encodedString = CleanTag(encodedString, "A", false);

            //tags that cause problems and must be done last
            encodedString = CleanTag(encodedString, "B", true);
            encodedString = CleanTag(encodedString, "I", true);
            encodedString = CleanTag(encodedString, "U", true);

            //simple replace
            encodedString = encodedString.Replace("&#13;&#10;", "");
            encodedString = encodedString.Replace("&#160;", " ");
            encodedString = encodedString.Replace("&#61;", "=");
            encodedString = encodedString.Replace("&#34;", "\"");
            encodedString = encodedString.Replace("&#39;", "'");
            encodedString = encodedString.Replace("&#58;", ":");
            encodedString = encodedString.Replace("&#47;", "/");
            encodedString = encodedString.Replace("&#92;", "\\");
            encodedString = encodedString.Replace("&#63;", "?");
            encodedString = encodedString.Replace("&#126;", "~");
            encodedString = encodedString.Replace("&#183;", "");
            encodedString = encodedString.Replace("&#40;", "(");
            encodedString = encodedString.Replace("&#41;", ")");
            encodedString = encodedString.Replace("&#38;", "&");
            encodedString = encodedString.Replace("&#59;", ";");
            encodedString = encodedString.Replace("&#38;nbsp&#59;", "&nbsp;");
            encodedString = encodedString.Replace("&#38;gt&#59;", "&gt;");
            encodedString = encodedString.Replace("&#38;lt&#59;", "&lt;");
            encodedString = encodedString.Replace("&#35;", "#");
            encodedString = encodedString.Replace("&#8211;", "-");

            return encodedString;
        }

        /// <summary>
        /// Replaces original HTML with a clean version with far fewer tags.
        /// </summary>
        /// <param name="origHTML"></param>
        /// <returns></returns>
        public string CleanHTML(string origHTML)
        {
            //string encodedString = System.Web.HttpUtility.HtmlDecode(origHTML);
            string encodedString = origHTML;

            //replace image tags
            encodedString = ReplaceImageTags(encodedString, "V:IMAGEDATA");

            //encodedString = SecureClean(encodedString);

            //remove tags that are unnecessary or will conflict with our stylesheet
            encodedString = RemoveTag(encodedString, "FONT");
            encodedString = RemoveTag(encodedString, "STYLE");

            //remove garbage office tags we get from copy/paste from office apps
            encodedString = RemoveTag(encodedString, "&#63;XML");
            encodedString = RemoveTag(encodedString, "O&#58;");
            encodedString = RemoveTag(encodedString, "V&#58;");
                        
            return encodedString;
            //return System.Web.HttpUtility.HtmlDecode(encodedString);
        }
        
        /// <summary>
        /// Returns the document HTML content as string.
        /// </summary>
        /// <param name="guianceItem">The GuidanceItem to convert.</param>
        /// <returns>A string with the HTML content.</returns>
        private string GetDocumentText(string fileName, GuidanceItemFileReference reference)
        {
            try
            {
                using (FileStream fileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read))
                using (XmlReader reader = XmlTextReader.Create(fileStream))
                {
                    while (reader.Read())
                    {
                        if (SchemaHelper.IsGuidanceItemNode(reader, SchemaHelper.GuidanceItem.Content))
                        {
                            return reader.ReadElementContentAsString();
                        }
                        else if (SchemaHelper.IsGuidanceItemNode(reader, SchemaHelper.GuidanceItem.Images))
                        {
                            ProcessGuidanceItemImages(reference, reader);
                        }
                    }
                }
            }
            catch
            {
                //do nothing
            }

            return String.Empty;
        }

        private void ProcessGuidanceItemImages(GuidanceItemFileReference reference, XmlReader reader)
        {
            ModelStorageService storageService = new ModelStorageService();
            storageService.ConvertXmlReaderToImages(reference, reader);
        }

        /// <summary>
        /// Exports the selected items to Word with an index page
        /// </summary>
        /// <param name="itemsContainer"></param>
        /// <param name="file"></param>
        public void RenderGuidanceItemsToWordFileInfoWithIndex(IList<GuidanceItemFileReference> itemsContainer, FileInfo file)
        {
            string extension = ".doc";
            ArrayList indexedList = new ArrayList(itemsContainer.Count);

            using (StreamWriter outFile = new StreamWriter(file.FullName, false, Encoding.UTF8))
            {
                outFile.WriteLine("<html>");

                string cssFile = GetSaveAsWordCSS(ModelStorageService.DefaultCSSFileName);
                if (File.Exists(cssFile))
                {
                    outFile.WriteLine("<head>");
                    outFile.WriteLine("<style type=\"text/css\">");
                    outFile.WriteLine(File.ReadAllText(cssFile));
                    outFile.WriteLine("</style>\n");
                    //outFile.WriteLine("</head>");
                }

                outFile.WriteLine("<body>");
                outFile.WriteLine(BuildIndexofTitles(indexedList, itemsContainer, extension, null));
                foreach (GuidanceItem item in indexedList)
                {
                    GuidanceItemFileReference gifr = item as GuidanceItemFileReference;
                    if (gifr != null)
                    {
                        outFile.WriteLine("<hr>");
                        WriteDocumentBody(gifr.FileName, ReadProtection.ReadOnly, gifr, true, outFile, file.DirectoryName);
                        outFile.WriteLine("<br>");
                        outFile.WriteLine("<br>");
                    }
                }
                outFile.WriteLine("</body>");
                outFile.WriteLine("</html>");
            }
        }

        /// <summary>
        /// Exports the selected items to Html with an index page
        /// </summary>
        /// <param name="itemsContainer"></param>
        /// <param name="file"></param>
        public void RenderGuidanceItemsToHtmlFileInfoWithIndex(IList<GuidanceItemFileReference> itemsContainer, FileInfo file)
        {
            string directory = file.DirectoryName;
            string extension = ".htm";
            ArrayList indexedList = new ArrayList(itemsContainer.Count);

            using (StreamWriter outFile = new StreamWriter(file.FullName, false, Encoding.UTF8))
            {
                directory += "\\" + Path.GetFileNameWithoutExtension(file.FullName) + "_files";
                outFile.WriteLine("<html>");

                string cssFile = GetCSS(ModelStorageService.DefaultCSSFileName);
                if (File.Exists(cssFile))
                {
                    outFile.WriteLine("<head>");
                    outFile.WriteLine("<style type=\"text/css\">");
                    outFile.WriteLine(File.ReadAllText(cssFile));
                    outFile.WriteLine("</style>\n");
                    outFile.WriteLine("</head>");
                }

                outFile.WriteLine("<body>");
                outFile.Write(BuildIndexofTitles(indexedList, itemsContainer, extension, directory));
                outFile.WriteLine("</body>");
                outFile.WriteLine("</html>");
            }

            Directory.CreateDirectory(directory);

            foreach (GuidanceItem item in indexedList)
            {
                GuidanceItemFileReference gifr = item as GuidanceItemFileReference;
                if (gifr != null)
                {
                    string newPath = Path.Combine(directory, Path.GetFileNameWithoutExtension(gifr.FileName) + extension);
                    using (StreamWriter itemFile = new StreamWriter(newPath, false, Encoding.UTF8))
                    {
                        using (StreamReader reader = new StreamReader(WriteDocumentStream(gifr.FileName, ReadProtection.ReadOnly, gifr, true, true, GetCSS(gifr.CSSFile), Path.GetDirectoryName(newPath))))
                        {
                            itemFile.Write(reader.ReadToEnd());
                        }
                    }
                }
            }
        }

        private static string GetValidCssFile(GuidanceItemFileReference item)
        {
            string cssFile = item.CSSFile;

            if (!File.Exists(cssFile))
            {
                cssFile = Path.Combine(ModelStorageService.TemplateFolder, item.CSSFile);

                if (!File.Exists(cssFile))
                {
                    cssFile = Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName);
                }
            }

            return cssFile;
        }

        private string BuildIndexofTitles(ArrayList indexedList, IList<GuidanceItemFileReference> itemList, string fileExtension, string directory)
        {
            StringBuilder indexString = new StringBuilder();
            SortedList technologyList = new SortedList();
            SortedList topicList;
            SortedList typeList;
            SortedList categoryList;
            SortedList titleList;

            //When building an Index document, use the CSS of the first Guideline item in the list

            foreach (GuidanceItem guidanceItem in itemList)
            {
                GuidanceItemFileReference item = guidanceItem as GuidanceItemFileReference;

                if (item != null)
                {
                    string fileName = Path.GetFileNameWithoutExtension(item.FileName);
                    if (!technologyList.ContainsKey(item["Technology"])) //TODO: Find a better way to do this
                    {
                        topicList = new SortedList();
                        typeList = new SortedList();
                        categoryList = new SortedList();
                        titleList = new SortedList();
                        titleList.Add(fileName, item);
                        categoryList.Add(item["Category"], titleList); //TODO: Find a better way to do this
                        typeList.Add(item.Type, categoryList);
                        topicList.Add(item["Topic"], typeList); //TODO: Find a better way to do this
                        technologyList.Add(item["Technology"], topicList); //TODO: Find a better way to do this
                    }
                    else
                    {
                        topicList = (SortedList)technologyList[item["Technology"]]; //TODO: Find a better way to do this
                        if (!topicList.ContainsKey(item["Topic"])) //TODO: Find a better way to do this
                        {
                            typeList = new SortedList();
                            categoryList = new SortedList();
                            titleList = new SortedList();
                            titleList.Add(fileName, item);
                            categoryList.Add(item["Category"], titleList); //TODO: Find a better way to do this
                            typeList.Add(item.Type, categoryList);
                            topicList.Add(item["Topic"], typeList); //TODO: Find a better way to do this
                        }
                        else
                        {
                            typeList = (SortedList)topicList[item["Topic"]]; //TODO: Find a better way to do this
                            if (!typeList.ContainsKey(item.Type))
                            {
                                categoryList = new SortedList();
                                titleList = new SortedList();
                                titleList.Add(fileName, item);
                                categoryList.Add(item["Category"], titleList); //TODO: Find a better way to do this
                                typeList.Add(item.Type, categoryList);
                            }
                            else
                            {
                                categoryList = (SortedList)typeList[item.Type];
                                if (!categoryList.ContainsKey(item["Category"]))
                                {
                                    titleList = new SortedList();
                                    titleList.Add(fileName, item);
                                    categoryList.Add(item["Category"], titleList);
                                }
                                else
                                {
                                    titleList = (SortedList)categoryList[item["Category"]];
                                    if (!titleList.ContainsKey(fileName))
                                    {
                                        titleList.Add(fileName, item);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            indexString.Append("<h1 class=\"title\">Guidance Index</h1>\n");

            for (int technologyListIndex = 0; technologyListIndex < technologyList.Count; technologyListIndex++)
            {
                SortedList topicList1 = (SortedList)technologyList.GetByIndex(technologyListIndex);
                for (int topicListIndex = 0; topicListIndex < topicList1.Count; topicListIndex++)
                {
                    SortedList typeList1 = (SortedList)topicList1.GetByIndex(topicListIndex);
                    indexString.Append("<h1>" + technologyList.GetKey(technologyListIndex) + " " + topicList1.GetKey(topicListIndex) + "</h1>");
                    int index = typeList1.IndexOfKey("Design");
                    if (index >= 0)
                    {
                        ExportType(indexedList, typeList1, index, indexString, directory, fileExtension);
                    }

                    index = typeList1.IndexOfKey("Implementation");
                    if (index >= 0)
                    {
                        ExportType(indexedList, typeList1, index, indexString, directory, fileExtension);
                    }

                    index = typeList1.IndexOfKey("Deployment");
                    if (index >= 0)
                    {
                        ExportType(indexedList, typeList1, index, indexString, directory, fileExtension);
                    }

                    for (int typeListIndex = 0; typeListIndex < typeList1.Count; typeListIndex++)
                    {
                        string key = (string)typeList1.GetKey(typeListIndex);
                        if (string.Compare(key, "Design") != 0 && string.Compare(key, "Deployment") != 0 &&
                            string.Compare(key, "Implementation") != 0) //TODO: resources
                        {
                            ExportType(indexedList, typeList1, typeListIndex, indexString, directory, fileExtension);
                        }
                    }
                }
            }

            return CleanHTML(indexString.ToString());
        }

        private void ExportType(ArrayList indexedList, SortedList typeList1, int typeListIndex, StringBuilder indexString,
            string directory, string fileExtension)
        {
            //TODO: this method is a little complicated now. Maybe, there should be a ChecklistCollection, that inherits form ItemCollection
            SortedList categoryList1 = (SortedList)typeList1.GetByIndex(typeListIndex);

            indexString.Append("<h2>" + typeList1.GetKey(typeListIndex) + "</h2>");
            for (int categoryListIndex = 0; categoryListIndex < categoryList1.Count; categoryListIndex++)
            {
                SortedList titleList1 = (SortedList)categoryList1.GetByIndex(categoryListIndex);
                indexString.Append("<h3>" + categoryList1.GetKey(categoryListIndex) + "</h3>");
                indexString.Append("<ul>\n");

                for (int titleListIndex = 0; titleListIndex < titleList1.Count; titleListIndex++)
                {
                    string filename = (string)titleList1.GetKey(titleListIndex);
                    GuidanceItemFileReference item = (GuidanceItemFileReference)titleList1.GetByIndex(titleListIndex);
                    string title = item.Title;
                    //add to indexed list so that later the document will be in same order as the index
                    indexedList.Add(item);
                    indexString.Append("<li>");
                    if (directory != null)
                    {
                        indexString.Append("<a href = 'file://" + Path.Combine(directory, filename) + fileExtension + "'>" + title + "</a>\n");
                    }
                    else
                    {
                        string link = title;//Links lenght is up to 40 chars
                        if (link.Length > 40)
                        {
                            link = link.Substring(0, 39);
                        }
                        indexString.Append("<a href = '#" + link.Replace(" ", "_") + "'>" + title + "</a>\n");
                    }
                    indexString.Append("</li>");
                }

                indexString.Append("</ul>\n");
            }
            indexString.Append("<br><br>");
        }
    }
}

