//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using GuidanceExplorer.Model;
using System.IO;
using GuidanceExplorer.Services;
using Microsoft.Practices.CompositeUI;
using GuidanceExplorer.WorkItems;

namespace GuidanceExplorer.Views
{
    public partial class GuidanceItemDetailView : UserControl
    {
        #region Private members

        /// <summary>
        /// The main model service instance.
        /// </summary>
        private IModelStorageService _modelService;

        /// <summary>
        /// The HTML rendering service for the guidance items.
        /// </summary>
        private IHtmlRenderService _htmlRenderService;

        /// <summary>
        /// The main work item instance.
        /// </summary>
        private GuidanceExplorerWorkItem _workItem;

        #endregion

        #region Services properties

        /// <summary>
        /// The HTML rendering service for the guidance items.
        /// </summary>
        [ServiceDependency]
        public IHtmlRenderService HtmlRenderService
        {
            set { _htmlRenderService = value; }
        }

        /// <summary>
        /// The main model service instance.
        /// </summary>
        [ServiceDependency]
        public IModelStorageService ModelService
        {
            set { _modelService = value; }
        }

        /// <summary>
        /// The main work item instance.
        /// </summary>
        [ServiceDependency]
        public GuidanceExplorerWorkItem WorkItem
        {
            set { _workItem = value; }
        }

        #endregion

        /// <summary>
        /// Default constructor.
        /// </summary>
        public GuidanceItemDetailView()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Show the details of the item in the details pane.
        /// </summary>
        /// <param name="guidanceItem"></param>
        public void ShowDetails(GuidanceItem guidanceItem)
        {
            GuidanceItemFileReference fileReference = guidanceItem as GuidanceItemFileReference;
            if (fileReference != null)
            {
                webBrowser.DocumentStream = _htmlRenderService.GetDocumentStreamForShowing(fileReference);
            }
            else
            {
                //Clean the contents.
                webBrowser.DocumentStream = null;
            }
        }

        /// <summary>
        /// Occurs when the browser is navigating to an address.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void webBrowser_Navigating(object sender, WebBrowserNavigatingEventArgs e)
        {
            GuidanceItem guidanceItem = null;
            if (e.Url.Scheme == Uri.UriSchemeHttp)
            {
                e.Cancel = true;
                WebBrowser browser = (WebBrowser)sender;
                browser.Navigate(e.Url, true);
            }
            else if (e.Url.Scheme == "ruleid")
            {
                e.Cancel = true;
                string ruleId = e.Url.AbsolutePath;
                guidanceItem = _modelService.FindItemById(ruleId);
                if (guidanceItem != null)
                {
                    GuidanceItemFileReference reference = guidanceItem as GuidanceItemFileReference;
                    if (reference != null)
                    {
                        webBrowser.DocumentStream = _htmlRenderService.GetReadOnlyDocumentStream(reference);
                    }
                }
            }
            else if (e.Url.Scheme == "ruledisplay")
            {
                e.Cancel = true;
                string ruleId = e.Url.AbsolutePath;
                guidanceItem = _modelService.Guidance.Items.FindItemById(ruleId);
                GuidanceItemFileReference gifr = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItem);
                _workItem.GuidanceItemOpened(this, new GuidanceItemFileReferenceEventArgs(new GuidanceItemFileReference[] { gifr }, ReadProtection.ReadOnly));
            }
        }
    }
}

