//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.Commands;

using GuidanceExplorer.Model;
using GuidanceExplorer.Controls;
using GuidanceExplorer.Views;
using GuidanceExplorer.Services;
using System.IO;
using Microsoft.Practices.CompositeUI.EventBroker;
using System.ComponentModel;
using Microsoft.Practices.CompositeUI.WinForms;
using Microsoft.Practices.CompositeUI.SmartParts;

namespace GuidanceExplorer.WorkItems
{
    public partial class GuidanceExplorerWorkItem
    {
        [EventSubscription("event://AuthoringToolkitNodeSelected")]
        public void AuthoringToolkitNodeSelected(object sender, EventArgs eventArgs)
        {
            //Get the auttoolkit node
            AuthoringToolkitNode authoringToolkit = _treeView.GetCurrentNode() as AuthoringToolkitNode;
            if (authoringToolkit == null)
            {
                throw new NotImplementedException();
            }

            //If the node does not have columns, create a new one.
            if (authoringToolkit.Columns == null)
            {
                authoringToolkit.Columns = ColumnCollection.DefaultColumns(GuidanceType.New("template"));
            }

            //Create the collection of guidance types
            GuidanceItemFileReferenceCollection guidanceItems = GuidanceItemFileReferenceCollection.New();

            //Set the columns to the collection.
            guidanceItems.Columns = authoringToolkit.Columns;

            //Populate the collection
            foreach (GuidanceType guidanceItemType in GuidanceType.AllGuidanceTypes)
            {
                if (!String.IsNullOrEmpty(guidanceItemType.DisplayName))
                {
                    GuidanceItemFileReference gifr = guidanceItemType as GuidanceItemFileReference;
                    gifr.FileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceItemType.TemplateFileName));
                    gifr.CSSFile = ModelStorageService.DefaultCSSFileName;
                    gifr.GuidanceType = GuidanceType.New(GuidanceType.TemplateTypeName);
                    guidanceItems.Add(gifr.Id, gifr);
                }
            }

            //Show the items in the list view.
            _listView.ShowContents(guidanceItems, authoringToolkit.Columns);
        }

        [EventSubscription("event://MyViewsSelected")]
        public void MyViewsSelected(object sender, ModelNodeEventArgs<UserViews> eventArgs)
        {
            GuidanceItem selectedItem = _listView.GetCurrentGuidanceItem();
            IModelItem lastNode = TreeView.LastSelectedNode as IModelItem;
            LibrariesNode libsNode = TreeView.LastSelectedNode as LibrariesNode;

            _searchToolbar.SearchTip = "Search all guidance";

            if (selectedItem != null && lastNode != null)
            {
                lastNode.SelectedItem = selectedItem.Id;
            }
            else if (selectedItem != null && libsNode != null)
            {
                libsNode.SelectedItem = selectedItem.Id;
            }

            _listView.ShowContents(eventArgs.ModelInstance, eventArgs.ModelInstance.Columns);

            if (!string.IsNullOrEmpty(eventArgs.ModelNode.SelectedItem))
            {
                _listView.SelectSingleItem(eventArgs.ModelNode.SelectedItem);
            }
        }

        [EventSubscription("event://GlobalViewsSelected")]
        public void GlobalViewsSelected(object sender, ModelNodeEventArgs<GlobalViews> eventArgs)
        {
            GuidanceItem selectedItem = _listView.GetCurrentGuidanceItem();
            IModelItem lastNode = TreeView.LastSelectedNode as IModelItem;
            LibrariesNode libsNode = TreeView.LastSelectedNode as LibrariesNode;

            _searchToolbar.SearchTip = "Search all guidance";
            
            if (selectedItem != null && lastNode != null)
            {
                lastNode.SelectedItem = selectedItem.Id;
            }
            else if (libsNode != null)
            {
                libsNode.SelectedItem = selectedItem.Id;
            }

            _listView.ShowContents(eventArgs.ModelInstance, eventArgs.ModelInstance.Columns);

            if (!string.IsNullOrEmpty(eventArgs.ModelNode.SelectedItem))
            {
                _listView.SelectSingleItem(eventArgs.ModelNode.SelectedItem);
            }
        }

        [EventSubscription("event://LibrariesNodeSelected")]
        public void LibrariesNodeSelected(object sender, EventArgs eventArgs)
        {
            GuidanceItem selectedItem = _listView.GetCurrentGuidanceItem();
            TreeNode lastSelectedNode = TreeView.LastSelectedNode;

            _searchToolbar.SearchTip = "Search all guidance";

            if ((selectedItem != null) && (lastSelectedNode != null))
            {
                ((IModelItem)lastSelectedNode).SelectedItem = selectedItem.Id;
            }

            //Show all items
            _listView.ShowContents(_modelService.Guidance, _modelService.Guidance.Columns);

            string selectedItemId = ((LibrariesNode)TreeView.TreeView.SelectedNode).SelectedItem;
            if (!string.IsNullOrEmpty(selectedItemId))
            {
                _listView.SelectSingleItem(selectedItemId);
            }
        }

        [EventSubscription("event://LibraryNodeSelected")]
        public void LibraryNodeSelected(object sender, ModelNodeEventArgs<Library> eventArgs)
        {
            GuidanceItem selectedItem = _listView.GetCurrentGuidanceItem();
            IModelItem lastNode = TreeView.LastSelectedNode as IModelItem;
            LibrariesNode libsNode = TreeView.LastSelectedNode as LibrariesNode;

            Library library = eventArgs.ModelInstance;
            if (library != null)
            {
                _searchToolbar.SearchTip = "Search in " + library.Caption;
            }
            else
            {
                _searchToolbar.SearchTip = "Search all guidance";
            }

            if (selectedItem != null && lastNode != null)
            {
                lastNode.SelectedItem = selectedItem.Id;
            }
            else if (libsNode != null && selectedItem != null)
            {
                libsNode.SelectedItem = selectedItem.Id;
            }

            _listView.ShowContents(eventArgs.ModelInstance, eventArgs.ModelInstance.Columns);

            if (!string.IsNullOrEmpty(eventArgs.ModelNode.SelectedItem))
            {
                _listView.SelectSingleItem(eventArgs.ModelNode.SelectedItem);
            }
        }

        [EventSubscription("event://LibraryNodeDeleted")]
        public void LibraryNodeDeleted(object sender, ModelNodeEventArgs<Library> eventArgs)
        {
            //Update the model

            //Get the model instance.
            Library library = eventArgs.ModelInstance;

            if (library.ReadProtection == ReadProtection.ReadOnly)
            {
                MessageBox.Show("Read-only Libraries can't be deleted", Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                //Remove the library from the Guidance
                _modelService.Guidance.Libraries.Remove(library.Name);

                if (library.FileSubscribed && library.HasChanges)
                {
                    library.FileSubscribed = false;
                    _modelService.RenderLibraryToFileInfo(library, library.OriginalSourceFileInfo);
                }
                //Add the library to the deleted library collection so it will be removed when the model is saved.
                else if (!_modelService.Guidance.DeletedLibraries.ContainsKey(library.Name))
                {
                    _modelService.Guidance.DeletedLibraries.Add(library.Name, library);
                }

                //remove the library from storage
                if (library.WebSubscribed)
                {
                    _onlineStorage.RemoveLibrary(library);
                }

                //Update the UI, remove the node from its parent and then remove all the view nodes from the global views node
                eventArgs.ModelNode.Remove();
            }
        }

        [EventSubscription("event://FolderNodeSelected")]
        public void FolderNodeSelected(object sender, ModelNodeEventArgs<Folder> eventArgs)
        {
            GuidanceItem selectedItem = _listView.GetCurrentGuidanceItem();
            IModelItem lastNode = TreeView.LastSelectedNode as IModelItem;
            LibrariesNode libsNode = TreeView.LastSelectedNode as LibrariesNode;
            ColumnCollection columns = null;

            Library library = eventArgs.ModelInstance.GetContainerLibrary();
            if (library != null)
            {
                _searchToolbar.SearchTip = "Search in " + library.Caption;
                columns = library.Columns;
            }
            else
            {
                _searchToolbar.SearchTip = "Search all guidance";
            }

            if (selectedItem != null && lastNode != null)
            {
                lastNode.SelectedItem = selectedItem.Id;
            }
            else if (libsNode != null)
            {
                libsNode.SelectedItem = selectedItem.Id;
            }

            _listView.ShowContents(eventArgs.ModelInstance, columns);

            if (!string.IsNullOrEmpty(eventArgs.ModelNode.SelectedItem))
            {
                _listView.SelectSingleItem(eventArgs.ModelNode.SelectedItem);
            }
        }

        [EventSubscription("event://FolderNodeDeleted")]
        public void FolderNodeDeleted(object sender, ModelNodeEventArgs<Folder> eventArgs)
        {
            //Get the model instance.
            Folder folder = eventArgs.ModelInstance;

            //Remove the folder from its container.
            folder.Remove();

            //update the online store
            Library parentLibrary =  folder.GetContainerLibrary();
            if (parentLibrary != null && parentLibrary.WebSubscribed)
            {
                foreach (Model.View view in folder.AllViews.Values)
                {
                    _onlineStorage.RemoveView(view);
                }
            }

            //Update the UI
            eventArgs.ModelNode.Remove();

            if (folder.GetContainerLibrary() != null)
            {
                folder.GetContainerLibrary().HasChanges = true;
            }
        }

        [EventSubscription("event://ViewNodeSelected")]
        public void ViewNodeSelected(object sender, ModelNodeEventArgs<GuidanceExplorer.Model.View> eventArgs)
        {
            GuidanceItem selectedItem = _listView.GetCurrentGuidanceItem();
            IModelItem lastNode = TreeView.LastSelectedNode as IModelItem;
            LibrariesNode libsNode = TreeView.LastSelectedNode as LibrariesNode;
            ColumnCollection columnCollection = null;

            Library library = eventArgs.ModelInstance.GetContainerLibrary();
            if (library != null)
            {
                _searchToolbar.SearchTip = "Search in " + library.Caption;
            }
            else
            {
                _searchToolbar.SearchTip = "Search all guidance";
            }

            if (selectedItem != null && lastNode != null)
            {
                lastNode.SelectedItem = selectedItem.Id;
            }
            else if ((selectedItem != null) && (libsNode != null))
            {
                libsNode.SelectedItem = selectedItem.Id;
            }

            //fill in the columns collection to display the listview
            if (library == null)
            {
                FolderStructure folderStructure = eventArgs.ModelInstance.Container;
                if (folderStructure != null)
                {
                    columnCollection = folderStructure.Columns;
                }
            }
            else
            {
                columnCollection = library.Columns;
            }

            _listView.ShowContents(eventArgs.ModelInstance, columnCollection);

            if (!string.IsNullOrEmpty(eventArgs.ModelNode.SelectedItem))
            {
                _listView.SelectSingleItem(eventArgs.ModelNode.SelectedItem);
            }
        }

        [EventSubscription("event://SearchViewNodeDeleted")]
        public void SearchViewNodeDeleted(object sender, ModelNodeEventArgs<SearchView> eventArgs)
        {
            //Get the model instance.
            SearchView searchView = eventArgs.ModelInstance;

            //Remove the search view from its container.
            searchView.Remove(false);

            //Remove from online store
            Library parentLibrary = searchView.GetContainerLibrary();
            if (parentLibrary != null && parentLibrary.WebSubscribed)
            {
                _onlineStorage.RemoveView(searchView);
            }       

            //Update the UI
            eventArgs.ModelNode.Remove();

            if (searchView.GetContainerLibrary() != null)
            {
                searchView.GetContainerLibrary().HasChanges = true;
            }
        }

        [EventSubscription("event://SearchViewNodeSelected")]
        public void SearchViewNodeSelected(object sender, ModelNodeEventArgs<SearchView> eventArgs)
        {
            GuidanceItem selectedItem = _listView.GetCurrentGuidanceItem();
            IModelItem lastNode = TreeView.LastSelectedNode as IModelItem;
            LibrariesNode libsNode = TreeView.LastSelectedNode as LibrariesNode;

            Library library = eventArgs.ModelInstance.GetContainerLibrary();
            if (library != null)
            {
                _searchToolbar.SearchTip = "Search in " + library.Caption;
            }
            else
            {
                _searchToolbar.SearchTip = "Search all guidance";
            }

            if (selectedItem != null && lastNode != null)
            {
                lastNode.SelectedItem = selectedItem.Id;
            }
            else if (libsNode != null && selectedItem != null)
            {
                libsNode.SelectedItem = selectedItem.Id;
            }

            _listView.ShowContents(eventArgs.ModelInstance, eventArgs.ModelInstance.ContainerScope.Columns);

            if (!string.IsNullOrEmpty(eventArgs.ModelNode.SelectedItem))
            {
                _listView.SelectSingleItem(eventArgs.ModelNode.SelectedItem);
            }
        }

        [EventSubscription("event://ViewNodeDeleted")]
        public void ViewNodeDeleted(object sender, ModelNodeEventArgs<GuidanceExplorer.Model.View> eventArgs)
        {
            //Get the model instance.
            GuidanceExplorer.Model.View view = eventArgs.ModelInstance;

            //Remove the folder from its container.
            view.Remove(false);

            //Remove from online store
            Library parentLibrary = view.GetContainerLibrary();
            if (parentLibrary != null && parentLibrary.WebSubscribed)
            {
                _onlineStorage.RemoveView(view);
            }

            //Update the UI
            eventArgs.ModelNode.Remove();

            if (view.GetContainerLibrary() != null)
            {
                view.GetContainerLibrary().HasChanges = true;
            }
        }

        [EventSubscription("event://GuidanceItemSelected")]
        public void GuidanceItemSelected(object sender, GuidanceItemEventArgs eventArgs)
        {
            _detailView.ShowDetails(eventArgs.GuidanceItem);
        }

        [EventSubscription("event://GuidanceItemsDeleted")]
        public void GuidanceItemsDeleted(object sender, GuidanceItemCollectionEventArgs eventArgs)
        {
            //Refresh the data in the list view.
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            
            //hack - use this to turn off warnings for the move item case
            bool showWarnings = true;
            if (sender == null)
            {
                showWarnings = false;
            }

            if (modelNode != null)
            {
                LibraryNode libraryNode = modelNode as LibraryNode;
                MyViewsNode myViewsNode = modelNode as MyViewsNode;
                GlobalViewsNode globalViewsNode = modelNode as GlobalViewsNode;
                ViewNode viewNode = modelNode as ViewNode;
                FolderNode folderNode = modelNode as FolderNode;
                GuidanceTypeNode guidanceTypeNode = modelNode as GuidanceTypeNode;
                SearchViewNode searchViewNode = modelNode as SearchViewNode;
                if (viewNode != null)
                {
                    Library containingLibrary = viewNode.ModelInstance.GetContainerLibrary();

                    if (containingLibrary != null && containingLibrary.ReadProtection == ReadProtection.ReadOnly)
                    {
                        MessageBox.Show(Messages.CANNOT_DELETE_ITEMS_FROM_READ_ONLY_LIBRARIES, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    else if (!showWarnings || MessageBox.Show(Messages.DELETE_ITEMS_FROM_VIEW, Messages.WARNING, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                    {
                        //Update the model.
                        foreach (GuidanceItem guidanceItem in eventArgs.GuidanceItemCollection)
                        {
                            _modelService.DeleteGuidanceItemFromView(viewNode.ModelInstance, guidanceItem);
                        }

                        if (viewNode.ModelInstance.GetContainerLibrary() != null)
                        {
                            viewNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                        }

                        this.ViewNodeSelected(this, new ModelNodeEventArgs<GuidanceExplorer.Model.View>(viewNode));

                        return;
                    }
                }
                else if (libraryNode != null)
                {
                    if (libraryNode.ModelInstance.ReadProtection == ReadProtection.ReadOnly)
                    {
                        MessageBox.Show(Messages.CANNOT_DELETE_ITEMS_FROM_READ_ONLY_LIBRARIES, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    else if (libraryNode.ModelInstance.ReadProtection == ReadProtection.ReadWrite)
                    {
                        bool messageConfirmed = false;
                        libraryNode.ModelInstance.HasChanges = true;
                        foreach (GuidanceItem guidanceItem in eventArgs.GuidanceItemCollection)
                        {
                            if (!showWarnings || (messageConfirmed) || MessageBox.Show(Messages.DELETE_ITEMS_FROM_LIBRARY, Messages.WARNING, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                            {
                                messageConfirmed = true;

                                //Remove guidance across library.
                                _modelService.RemoveGuidanceItemFromLibrary(libraryNode.ModelInstance, guidanceItem);

                                //Remove the pointers from the user views.
                                foreach (GuidanceExplorer.Model.View view in _modelService.Guidance.UserViews.Views.Values)
                                {
                                    _modelService.DeleteGuidanceItemFromView(view, guidanceItem);
                                }

                                //Remove the pointers from the user views.
                                foreach (GuidanceExplorer.Model.View view in _modelService.Guidance.GlobalViews.Views.Values)
                                {
                                    _modelService.DeleteGuidanceItemFromView(view, guidanceItem);
                                }

                                //Remove guidance across.
                                _modelService.RemoveGuidanceItemFromAllViews(_modelService.Guidance.UserViews.Folders, guidanceItem);
                                _modelService.RemoveGuidanceItemFromAllViews(_modelService.Guidance.GlobalViews.Folders, guidanceItem);

                                //Remove the guidance from the online store
                                if (guidanceItem.Library.WebSubscribed)
                                {
                                    _onlineStorage.PublishGuidanceItem(guidanceItem.Library, guidanceItem, null, GuidanceItemOperation.Delete, false);
                                }

                                //Update the presentation.
                                _winformRender.RefreshGuidanceTypeNodes(libraryNode, guidanceItem.GuidanceType);

                                //Refresh the list view.
                                LibraryNodeSelected(this, new ModelNodeEventArgs<Library>(libraryNode));
                            }
                            else
                            {
                                return;
                            }
                        } //foreach (GuidanceItem guidanceItem in eventArgs.GuidanceItemCollection)
                      
                        //Delete all of the items from the model.
                        _modelService.DeleteGuidanceItems(eventArgs.GuidanceItemCollection, libraryNode.ModelInstance);

                    }
                    else
                    {
                        throw new NotSupportedException("Protection type not supported.");
                    }
                }
                else if (folderNode != null)
                {
                    MessageBox.Show("Items can't be deleted from the folders", Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else if (guidanceTypeNode != null)
                {
                    //Get the parent node from the tree view.
                    LibraryNode parentLibrary = guidanceTypeNode.Parent as LibraryNode;

                    if (parentLibrary.ModelInstance.ReadProtection == ReadProtection.ReadOnly)
                    {
                        MessageBox.Show(Messages.CANNOT_DELETE_ITEMS_FROM_READ_ONLY_LIBRARIES, Messages.ERROR,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    else
                    {
                        if (!showWarnings || MessageBox.Show(Messages.DELETE_ITEMS_FROM_LIBRARY, Messages.WARNING,
                            MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                        {
                            parentLibrary.ModelInstance.HasChanges = true;
                            foreach (GuidanceItem guidanceItem in eventArgs.GuidanceItemCollection)
                            {
                                //Remove guidance across 
                                _modelService.RemoveGuidanceItemFromLibrary(parentLibrary.ModelInstance, guidanceItem);

                                //Delete from the model
                                _modelService.DeleteGuidanceItem(guidanceItem, parentLibrary.ModelInstance);

                                //Remove the guidance from the online store
                                if (guidanceItem.Library.WebSubscribed)
                                {
                                    _onlineStorage.PublishGuidanceItem(guidanceItem.Library, guidanceItem, null, GuidanceItemOperation.Delete, false);
                                }

                                //Remove the pointers from the user views.
                                foreach (GuidanceExplorer.Model.View view in _modelService.Guidance.UserViews.Views.Values)
                                {
                                    _modelService.DeleteGuidanceItemFromView(view, guidanceItem);
                                }

                                //Remove the pointers from the global views.
                                foreach (GuidanceExplorer.Model.View view in _modelService.Guidance.GlobalViews.Views.Values)
                                {
                                    _modelService.DeleteGuidanceItemFromView(view, guidanceItem);
                                }

                                //Remove guidance across 
                                _modelService.RemoveGuidanceItemFromAllViews(_modelService.Guidance.UserViews.Folders, guidanceItem);
                                _modelService.RemoveGuidanceItemFromAllViews(_modelService.Guidance.GlobalViews.Folders, guidanceItem);

                                _winformRender.RefreshGuidanceTypeNodes(parentLibrary, guidanceItem.GuidanceType);
                            }

                            //Remove guidance across 
                            if (guidanceTypeNode.Parent != null)
                            {
                                GuidanceTypeNodeSelected(this, new ModelNodeEventArgs<GuidanceType>(guidanceTypeNode));
                            }
                            else
                            {
                                _treeView.TreeView.SelectedNode = parentLibrary;
                            }
                        }
                        else
                        {
                            return;
                        }
                    }
                }
                else if (myViewsNode != null)
                {
                    MessageBox.Show(Messages.CANNOT_DELETE_ITEMS_FROM_MY_VIEWS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }               
                else if (globalViewsNode != null)
                {
                    MessageBox.Show(Messages.CANNOT_DELETE_ITEMS_FROM_GLOBAL_VIEWS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                else if (searchViewNode != null)
                {
                    MessageBox.Show(Messages.CANNOT_DELETE_ITEMS_FROM_SEARCH_VIEW, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                else
                {
                    throw new NotSupportedException();
                }

                //Add the deleted items to delete when closing.
                foreach (GuidanceItem guidanceItem in eventArgs.GuidanceItemCollection)
                {
                    if (!_modelService.Guidance.DeletedGuidanceItems.ContainsKey(guidanceItem.Id))
                    {
                        _modelService.Guidance.DeletedGuidanceItems.Add(guidanceItem.Id, guidanceItem);
                    }
                    else
                    {
                        //If the key is contained, then ignore.
                    }
                }
            }
            else
            {
                AuthoringToolkitNode authoringToolkitNode = treeNode as AuthoringToolkitNode;
                if (authoringToolkitNode != null)
                {
                    foreach (GuidanceItem guidanceItem in eventArgs.GuidanceItemCollection)
                    {
                        //Delete the GuidanceType
                        GuidanceType.RemoveGuidanceType(guidanceItem.Title);
                    }

                    //Update the list.
                    AuthoringToolkitNodeSelected(this, EventArgs.Empty);
                }
                else if (treeNode is LibrariesNode)
                {
                    bool messageConfirmed = false;
                    LibrariesNode librariesNode = treeNode as LibrariesNode;
                    foreach (GuidanceItem guidanceItem in eventArgs.GuidanceItemCollection)
                    {
                        Library library = guidanceItem.Library;
                        if (library.ReadProtection == ReadProtection.ReadOnly)
                        {
                            MessageBox.Show(Messages.CANNOT_DELETE_ITEMS_FROM_READ_ONLY_LIBRARIES, Messages.ERROR,
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                        else if (!showWarnings || messageConfirmed ||
                                 MessageBox.Show(Messages.DELETE_ITEMS_FROM_LIBRARY, Messages.WARNING, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                        {
                            messageConfirmed = true;
                            library.HasChanges = true;

                            //Update the model.
                            _modelService.DeleteGuidanceItem(guidanceItem, library);

                            //Remove guidance across 
                            _modelService.RemoveGuidanceItemFromLibrary(library, guidanceItem);

                            //Remove the pointers from the user views.
                            foreach (GuidanceExplorer.Model.View view in _modelService.Guidance.UserViews.Views.Values)
                            {
                                _modelService.DeleteGuidanceItemFromView(view, guidanceItem);
                            }

                            //Remove the pointers from the global views.
                            foreach (GuidanceExplorer.Model.View view in _modelService.Guidance.GlobalViews.Views.Values)
                            {
                                _modelService.DeleteGuidanceItemFromView(view, guidanceItem);
                            }

                            //Remove guidance across 
                            _modelService.RemoveGuidanceItemFromAllViews(_modelService.Guidance.UserViews.Folders, guidanceItem);
                            _modelService.RemoveGuidanceItemFromAllViews(_modelService.Guidance.GlobalViews.Folders, guidanceItem);

                            _winformRender.RefreshGuidanceTypeNodes(_winformRender.GetLibraryNodeByLibrary(_treeView, library), guidanceItem.GuidanceType);

                            //Remove guidance across 
                            LibrariesNodeSelected(this, null);
                        }
                        else
                        {
                            return;
                        }
                    }
                }
                else
                {
                    //Not a model element
                    throw new NotSupportedException();
                }
            }
        }

        [EventSubscription("event://GuidanceItemOpened")]
        public void GuidanceItemOpened(object sender, GuidanceItemFileReferenceEventArgs eventArgs)
        {
            if (eventArgs.GuidanceItemFileReferences != null)
            {
                if (eventArgs.GuidanceItemFileReferences.Length > 4)
                {
                    if (MessageBox.Show(string.Format(Messages.OPENING_A_LOT_OF_ITEMS, eventArgs.GuidanceItemFileReferences.Length),
                        Messages.WARNING, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No)
                    {
                        return;
                    }
                }

                if (eventArgs.ReadProtection == ReadProtection.ReadWrite)
                {
                    foreach (GuidanceItemFileReference item in eventArgs.GuidanceItemFileReferences)
                    {
                        IWorkspace workspace = RootWorkItem.Workspaces.Get<WindowWorkspace>("GuidanceItemEditor");
                        if (workspace != null)
                        {
                            RootWorkItem.Workspaces.Remove(workspace);
                        }

                        HtmlEditControl htmlEditor = RootWorkItem.SmartParts.Get<HtmlEditControl>("htmlEditControl");
                        if (htmlEditor != null)
                        {
                            RootWorkItem.SmartParts.Remove(htmlEditor);
                        }

                        GuidanceItemMetadata itemMetadata = RootWorkItem.SmartParts.Get<GuidanceItemMetadata>("GuidanceItemMetadata");
                        if (itemMetadata != null)
                        {
                            RootWorkItem.SmartParts.Remove(itemMetadata);
                        }

                        workspace = new WindowWorkspace(new Form());
                        RootWorkItem.Workspaces.Add(workspace, "GuidanceItemEditor");
                        itemMetadata = new GuidanceItemMetadata();
                        RootWorkItem.SmartParts.Add(itemMetadata, "GuidanceItemMetadata");
                        WindowSmartPartInfo info = new WindowSmartPartInfo();
                        
                        if (File.Exists(item.FileName))
                        {
                            info.Title = string.Format(Messages.EDIT_ITEM, item.GuidanceType.DisplayName, item.Title);
                        }
                        else
                        {
                            info.Title = string.Format(Messages.NEW_ITEM, item.GuidanceType.DisplayName);
                        }

                        workspace.Show(itemMetadata, info);
                        itemMetadata.Dock = DockStyle.Fill;
                        itemMetadata.SetGuidanceItem(_winformRender.GetLibraryNodeByLibrary(_treeView, item.Library), item, eventArgs.ReadProtection, eventArgs.View);
                    }
                }
                else
                {
                    foreach (GuidanceItemFileReference item in eventArgs.GuidanceItemFileReferences)
                    {
                        IWorkspace workspace = RootWorkItem.Workspaces.Get<WindowWorkspace>("GuidanceItemViewForm");
                        if (workspace != null)
                        {
                            RootWorkItem.Workspaces.Remove(workspace);
                        }

                        GuidanceItemView viewControl = RootWorkItem.SmartParts.Get<GuidanceItemView>("GuidanceItemView");
                        if (viewControl != null)
                        {
                            RootWorkItem.SmartParts.Remove(viewControl);
                        }

                        workspace = new WindowWorkspace(new Form());
                        WindowSmartPartInfo info = new WindowSmartPartInfo();
                        info.Title = "View " + item.GuidanceType.DisplayName;
                        RootWorkItem.Workspaces.Add(workspace, "GuidanceItemViewForm");
                        viewControl = new GuidanceItemView(item);
                        RootWorkItem.SmartParts.Add(viewControl, "GuidanceItemView");
                        workspace.Show(viewControl, info);
                        viewControl.Dock = DockStyle.Fill;
                    }
                }
            }
        }

        [EventSubscription("event://OpeningContextMenuListView")]
        public void OpeningContextMenuListView(object sender, CancelEventArgs eventArgs)
        {
            ToolStripDropDownMenu contextMenu = sender as ToolStripDropDownMenu;

            //Add menu items for the current node selected in the view.
            GuidanceItem[] guidanceItems = _listView.GetSelectedGuidanceItems();
            List<ToolStripMenuItem> guidanceItemRoots = null;
            if (guidanceItems == null)
            {
                MenuItemAttribute[] attributes = (MenuItemAttribute[])typeof(GuidanceItemFileReference).GetCustomAttributes(typeof(MenuItemAttribute), false);
                guidanceItemRoots = CreateRootMenues(attributes, null);
            }
            else
            {
                MenuItemAttribute[] attributes = (MenuItemAttribute[])guidanceItems[0].GetType().GetCustomAttributes(typeof(MenuItemAttribute), false);
                guidanceItemRoots = CreateRootMenues(attributes, guidanceItems);
            }

            //Update the menu.
            contextMenu.Items.Clear();

            foreach (ToolStripMenuItem menuItem in guidanceItemRoots)
            {
                if (menuItem.Text == "context")
                {
                    List<ToolStripItem> subMenus = new List<ToolStripItem>();
                    foreach (ToolStripItem childItem in menuItem.DropDown.Items)
                    {
                        subMenus.Add(childItem);
                    }
                    contextMenu.Items.AddRange(subMenus.ToArray());
                    break;
                }
            }
        }

        [EventSubscription("event://OpeningContextMenuTreeView")]
        public void OpeningContextMenuTreeView(object sender, CancelEventArgs eventArgs)
        {
            ToolStripDropDownMenu contextMenu = sender as ToolStripDropDownMenu;

            //Add menu items for the current node selected in the view.
            TreeNode treeNode = _treeView.GetCurrentNode();
            MenuItemAttribute[] attributes = (MenuItemAttribute[])treeNode.GetType().GetCustomAttributes(typeof(MenuItemAttribute), false);
            List<ToolStripMenuItem> treeNodeRoots = CreateRootMenues(attributes, new IModelItem[] { treeNode as IModelItem });

            //Update the menu.
            if (contextMenu != null)
            {
                contextMenu.Items.Clear();

                foreach (ToolStripMenuItem menuItem in treeNodeRoots)
                {
                    if (menuItem.Text == "context")
                    {
                        List<ToolStripItem> subMenus = new List<ToolStripItem>();
                        foreach (ToolStripItem childItem in menuItem.DropDown.Items)
                        {
                            subMenus.Add(childItem);
                        }
                        contextMenu.Items.AddRange(subMenus.ToArray());
                        break;
                    }
                }
            }
        }

        [EventSubscription("event://GuidanceTypeNodeSelected")]
        public void GuidanceTypeNodeSelected(object sender, ModelNodeEventArgs<GuidanceType> eventArgs)
        {
            LibraryNode parentLibraryNode = eventArgs.ModelNode.Parent as LibraryNode;
            GuidanceTypeNode guidanceTypeNode = eventArgs.ModelNode as GuidanceTypeNode;

            //Get the container library
            LibraryNode libraryNode = eventArgs.ModelNode.Parent as LibraryNode;
            if (libraryNode != null)
            {
                _searchToolbar.SearchTip = "Search " + guidanceTypeNode.Text + " in " + libraryNode.Text;
            }
            else
            {
                _searchToolbar.SearchTip = "Search all guidance";
            }

            //Set the default columns for the guidance type node.
            if (guidanceTypeNode.Columns == null)
            {
                guidanceTypeNode.Columns = ColumnCollection.DefaultGuidanceTypeColumns(eventArgs.ModelInstance);

                //If there's a field that isn't visible in the GuidanceType node
                foreach (Column col in guidanceTypeNode.Columns.Values)
                {
                    col.Visible = col.Field.ShowInGuidanceTypeNode;
                }
            }

            GuidanceItem selectedItem = _listView.GetCurrentGuidanceItem();
            IModelItem lastNode = TreeView.LastSelectedNode as IModelItem;
            LibrariesNode libsNode = TreeView.LastSelectedNode as LibrariesNode;

            if (selectedItem != null && lastNode != null)
            {
                lastNode.SelectedItem = selectedItem.Id;
            }
            else if (libsNode != null)
            {
                libsNode.SelectedItem = selectedItem.Id;
            }

            //Create the list to display
            GuidanceItemFileReferenceCollection guidanceItems = libraryNode.ModelInstance.GetGuidanceItemFileReferenceForGuidanceType(eventArgs.ModelInstance);
            guidanceItems.Columns = guidanceTypeNode.Columns;
            
            //Show the contents in the list view
            _listView.ShowContents(guidanceItems, guidanceTypeNode.Columns);

            if (!string.IsNullOrEmpty(eventArgs.ModelNode.SelectedItem))
            {
                _listView.SelectSingleItem(eventArgs.ModelNode.SelectedItem);
            }
        }

        [EventSubscription("event://DropSearchViewToUserViews")]
        public void DropSearchViewToUserViews(object sender, DropOperationEventArgs<SearchView, UserViews> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the search view.
                SearchView newSearchView = (SearchView)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Views.Add(newSearchView.Caption, newSearchView);

                newSearchView.Container = eventArgs.DropModelInstance;
                newSearchView.ParentFolder = null;
                newSearchView.ReadedScope = SearchViewScope.UserViews;
                newSearchView.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                SearchViewNode dragNode = (SearchViewNode)eventArgs.DragNode;
                SearchViewNode newNode = dragNode.Clone(newSearchView);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove(false);

                //Add the search view to the library.
                eventArgs.DropModelInstance.Views.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the search view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.ReadedScope = SearchViewScope.UserViews;
                eventArgs.DragModelInstance.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);
            }
        }

        [EventSubscription("event://DropSearchViewToLibrary")]
        public void DropSearchViewToLibrary(object sender, DropOperationEventArgs<SearchView, Library> eventArgs)
        {
             //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the search view.
                SearchView newSearchView = (SearchView)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Views.Add(newSearchView.Caption, newSearchView);

                newSearchView.Container = eventArgs.DropModelInstance;
                newSearchView.ParentFolder = null;
                newSearchView.ReadedScope = SearchViewScope.Library;
                newSearchView.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                SearchViewNode dragNode = (SearchViewNode)eventArgs.DragNode;
                SearchViewNode newNode = dragNode.Clone(newSearchView);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove(false);

                //Add the search view to the library.
                eventArgs.DropModelInstance.Views.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the search view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.ReadedScope = SearchViewScope.Library;
                eventArgs.DragModelInstance.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);
            }
        }

        [EventSubscription("event://DropSearchViewToGlobalViews")]
        public void DropSearchViewToGlobalViews(object sender, DropOperationEventArgs<SearchView, GlobalViews> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the search view.
                SearchView newSearchView = (SearchView)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Views.Add(newSearchView.Caption, newSearchView);

                newSearchView.Container = eventArgs.DropModelInstance;
                newSearchView.ParentFolder = null;
                newSearchView.ReadedScope = SearchViewScope.UserViews;
                newSearchView.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                SearchViewNode dragNode = (SearchViewNode)eventArgs.DragNode;
                SearchViewNode newNode = dragNode.Clone(newSearchView);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove(false);

                //Add the search view to the library.
                eventArgs.DropModelInstance.Views.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the search view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.ReadedScope = SearchViewScope.UserViews;
                eventArgs.DragModelInstance.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);
            }
        }
        
        [EventSubscription("event://DropSearchViewToFolder")]
        public void DropSearchViewToFolder(object sender, DropOperationEventArgs<SearchView, Folder> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view.
                SearchView newSearchView = (SearchView) eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Views.Add(newSearchView.Caption, newSearchView);

                newSearchView.Container = null;
                newSearchView.ParentFolder = eventArgs.DropModelInstance;
                newSearchView.ReadedScope = SearchViewScope.Folder;
                newSearchView.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                SearchViewNode dragNode = (SearchViewNode)eventArgs.DragNode;
                SearchViewNode newNode = dragNode.Clone(newSearchView);
                eventArgs.DropNode.Nodes.Add(newNode);

                if (eventArgs.DropNode.ModelInstance.GetContainerLibrary() != null)
                {
                    eventArgs.DropNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove(false);

                //Add the search view to the folder.
                eventArgs.DropModelInstance.Views.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the search view parent and container.
                eventArgs.DragModelInstance.Container = null;
                eventArgs.DragModelInstance.ParentFolder = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ReadedScope = SearchViewScope.Folder;
                eventArgs.DragModelInstance.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);

                if (eventArgs.DropNode.ModelInstance.GetContainerLibrary() != null)
                {
                    eventArgs.DropNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }

                if (eventArgs.DragNode.ModelInstance.GetContainerLibrary() != null)
                {
                    eventArgs.DragNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }
            }
        }

        [EventSubscription("event://DropViewToLibrary")]
        public void DropViewToLibrary(object sender, DropOperationEventArgs<GuidanceExplorer.Model.View, Library> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view
                GuidanceExplorer.Model.View newView = (GuidanceExplorer.Model.View)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Views.Add(newView.Caption, newView);

                newView.Container = eventArgs.DropModelInstance;
                newView.ParentFolder = null;
                newView.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                ViewNode dragNode = (ViewNode)eventArgs.DragNode;
                ViewNode newNode = dragNode.Clone(newView);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove(false);

                //Add the view to the library.
                eventArgs.DropModelInstance.Views.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);
            }

            eventArgs.DropNode.ModelInstance.HasChanges = true;
            _treeView.TreeView.SelectedNode = eventArgs.DropNode;
            LibraryNodeSelected(this, new ModelNodeEventArgs<Library>(eventArgs.DropNode));
        }

        [EventSubscription("event://DropViewToUserViews")]
        public void DropViewToUserViews(object sender, DropOperationEventArgs<GuidanceExplorer.Model.View, UserViews> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view
                GuidanceExplorer.Model.View newView = (GuidanceExplorer.Model.View)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Views.Add(newView.Caption, newView);

                newView.Container = eventArgs.DropModelInstance;
                newView.ParentFolder = null;
                newView.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                ViewNode dragNode = (ViewNode)eventArgs.DragNode;
                ViewNode newNode = dragNode.Clone(newView);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove(false);

                //Add the view to the library.
                eventArgs.DropModelInstance.Views.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);

                _treeView.TreeView.SelectedNode = eventArgs.DropNode;
                MyViewsSelected(this, new ModelNodeEventArgs<UserViews>(eventArgs.DropNode));
            }
        }

        [EventSubscription("event://DropViewToGlobalViews")]
        public void DropViewToGlobalViews(object sender, DropOperationEventArgs<GuidanceExplorer.Model.View, GlobalViews> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view.
                GuidanceExplorer.Model.View newView = (GuidanceExplorer.Model.View)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Views.Add(newView.Caption, newView);

                newView.Container = eventArgs.DropModelInstance;
                newView.ParentFolder = null;
                newView.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                ViewNode dragNode = (ViewNode)eventArgs.DragNode;
                ViewNode newNode = dragNode.Clone(newView);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove(false);

                //Add the view to the library.
                eventArgs.DropModelInstance.Views.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);
            }
            _treeView.TreeView.SelectedNode = eventArgs.DropNode;
            GlobalViewsSelected(this, new ModelNodeEventArgs<GlobalViews>(eventArgs.DropNode));
        }

        [EventSubscription("event://DropViewToFolder")]
        public void DropViewToFolder(object sender, DropOperationEventArgs<GuidanceExplorer.Model.View, Folder> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view.
                GuidanceExplorer.Model.View newView = (GuidanceExplorer.Model.View)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Views.Add(newView.Caption, newView);

                newView.Container = null;
                newView.ParentFolder = eventArgs.DropModelInstance;
                newView.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                ViewNode dragNode = (ViewNode)eventArgs.DragNode;
                ViewNode newNode = dragNode.Clone(newView);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove(false);

                //Add the view to the folder.
                eventArgs.DropModelInstance.Views.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the view parent and container.
                eventArgs.DragModelInstance.Container = null;
                eventArgs.DragModelInstance.ParentFolder = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.LastUpdate = DateTime.UtcNow;

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);

                if (eventArgs.DropNode.ModelInstance.GetContainerLibrary() != null)
                {
                    eventArgs.DropNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }
            }

            _treeView.TreeView.SelectedNode = eventArgs.DropNode;
            FolderNodeSelected(this, new ModelNodeEventArgs<Folder>(eventArgs.DropNode));
        }

        [EventSubscription("event://DropFolderToFolder")]
        public void DropFolderToFolder(object sender, DropOperationEventArgs<Folder, Folder> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view.
                Folder newFolder = (Folder) eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Folders.Add(newFolder.Caption, newFolder);

                newFolder.Container = null;
                newFolder.ParentFolder = eventArgs.DropModelInstance;
                newFolder.UpdateDateTime();

                //Reflect the change in the TreeView
                FolderNode dragNode = (FolderNode)eventArgs.DragNode;
                FolderNode newNode = dragNode.Clone(newFolder);
                eventArgs.DropNode.Nodes.Add(newNode);          
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove();

                //Add the view to the folder.
                eventArgs.DropModelInstance.Folders.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the view parent and container.
                eventArgs.DragModelInstance.Container = null;
                eventArgs.DragModelInstance.ParentFolder = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.UpdateDateTime();

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);
            }

            if (eventArgs.DragNode.ModelInstance.GetContainerLibrary() != null)
            {
                eventArgs.DragNode.ModelInstance.GetContainerLibrary().HasChanges = true;
            }

            _treeView.TreeView.SelectedNode = eventArgs.DropNode;
            FolderNodeSelected(this, new ModelNodeEventArgs<Folder>(eventArgs.DropNode));
        }

        [EventSubscription("event://DropFolderToLibrary")]
        public void DropFolderToLibrary(object sender, DropOperationEventArgs<Folder, Library> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view.
                Folder newFolder = (Folder)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Folders.Add(newFolder.Caption, newFolder);

                newFolder.Container = eventArgs.DropModelInstance;
                newFolder.ParentFolder = null;
                newFolder.UpdateDateTime();

                //Reflect the change in the TreeView
                FolderNode dragNode = (FolderNode)eventArgs.DragNode;
                FolderNode newNode = dragNode.Clone(newFolder);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove();

                //Add the view to the folder.
                eventArgs.DropModelInstance.Folders.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.UpdateDateTime(); 

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);
            }

            eventArgs.DropNode.ModelInstance.HasChanges = true;
            _treeView.TreeView.SelectedNode = eventArgs.DropNode;
            LibraryNodeSelected(this, new ModelNodeEventArgs<Library>(eventArgs.DropNode));
        }

        [EventSubscription("event://DropFolderToUserViews")]
        public void DropFolderToUserViews(object sender, DropOperationEventArgs<Folder, UserViews> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view.
                Folder newFolder = (Folder)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Folders.Add(newFolder.Caption, newFolder);

                newFolder.Container = eventArgs.DropModelInstance;
                newFolder.ParentFolder = null;
                newFolder.UpdateDateTime();

                //Reflect the change in the TreeView
                FolderNode dragNode = (FolderNode)eventArgs.DragNode;
                FolderNode newNode = dragNode.Clone(newFolder);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove();

                //Add the view to the folder.
                eventArgs.DropModelInstance.Folders.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.UpdateDateTime();

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);
            }

            _treeView.TreeView.SelectedNode = eventArgs.DropNode;
            MyViewsSelected(this, new ModelNodeEventArgs<UserViews>(eventArgs.DropNode));
        }

        [EventSubscription("event://DropFolderToGlobalViews")]
        public void DropFolderToGlobalViews(object sender, DropOperationEventArgs<Folder, GlobalViews> eventArgs)
        {
            //if readonly, then copy
            if ((eventArgs.DragModelInstance.GetContainerLibrary() != null) && (eventArgs.DragModelInstance.GetContainerLibrary().ReadProtection == ReadProtection.ReadOnly))
            {
                //Copy the view.
                Folder newFolder = (Folder)eventArgs.DragModelInstance.Clone();
                eventArgs.DropModelInstance.Folders.Add(newFolder.Caption, newFolder);

                newFolder.Container = eventArgs.DropModelInstance;
                newFolder.ParentFolder = null;
                newFolder.UpdateDateTime();

                //Reflect the change in the TreeView
                FolderNode dragNode = (FolderNode)eventArgs.DragNode;
                FolderNode newNode = dragNode.Clone(newFolder);
                eventArgs.DropNode.Nodes.Add(newNode);
            }
            //otherwise, move
            else
            {
                //Reflect the change in the model.
                eventArgs.DragModelInstance.Remove();

                //Add the view to the folder.
                eventArgs.DropModelInstance.Folders.Add(eventArgs.DragModelInstance.Caption, eventArgs.DragModelInstance);

                //Set the view parent and container.
                eventArgs.DragModelInstance.Container = eventArgs.DropModelInstance;
                eventArgs.DragModelInstance.ParentFolder = null;
                eventArgs.DragModelInstance.UpdateDateTime();

                //Reflect the change in the TreeView
                eventArgs.DragNode.Remove();
                eventArgs.DropNode.Nodes.Add(eventArgs.DragNode);

                _treeView.TreeView.SelectedNode = eventArgs.DropNode;
                GlobalViewsSelected(this, new ModelNodeEventArgs<GlobalViews>(eventArgs.DropNode));
            }
        }
    }
}
